import poser

#This EZskin3 plugin proves the powefull capabilities of EZSKin3, it can also be used the same as EZmat2.
#I made this new plugin as a version next to my Cycleskin plugin, this was getting so big that I decided to make a seperate plugin for materials.
#Made by Thijs Broekhuijsen AKA Bopperthijs, Bopperman.

# Global variables
EZBOOL, EZBUTTON, EZINT, EZCOLOUR, EZCOLOURP, EZFLOAT, EZCHOICE, EZSELECT, EZISELECT, EZFILE, EZLABEL, EZTEXT, EZPRESETS, EZDIR = range(14)
SKINTYPE, EYETYPE, NAILTYPE, MOUTHTYPE, TEETHTYPE, OTHERTYPE = range(6)
METHOD, SHADERTYPE = range(2)

# This class must exist - EZSkin instantiates it for building the shaders
class EZBuilder(object):

    def __init__(self, api):

        self.api = api                            # A reference to an EZAPI object
        self.parameters = None                     # The method to interrogate the value of UI controls (this will be set by EZSkin)

        self.has_controls = True                # True if you supply custom controls
        self.unique_name = '02_EZ_Cyclematerials'    # Sorry - no naming convention yet

        # This dictionary is used to associate an EZSkin shader with a method in your classs
        # In this case, we only have one method so all are set to invoke self.MakePBR
        # The second element of the tuple determines whether or not it should be processed,
        # As selected by the user on the main EZSkin UI
        self.shaders = {
            "ignore" : (None, SKINTYPE),
                        'cloth' : (self.CycleCloth, OTHERTYPE),
                        'satin' : (self.CycleSatin, OTHERTYPE),
                        'nylon' : (self.CycleNylon, OTHERTYPE),
                        'leather' : (self.CycleLeather, OTHERTYPE),
                        'plastic' : (self.CyclePlastic, OTHERTYPE),
                        'metal' : (self.CycleMetal, OTHERTYPE),                        
                        'velvet' : (self.CycleVelvet, OTHERTYPE),                        
            }

    def _PrepareRoot(self, mat, p):
        # Used to prepare the material room for a new Superfly shader
        # In this case it will return a PoserSurface root node
        # User preferences will be taken into account (use existing root or make a new one)
        # All existing roots are managed for you

        # Get original CyclesSurface root node
        n000 = mat.roots['superfly']
        if n000:
            if n000.Type() != poser.kNodeTypeCodeCYCLESSURFACE:
                n000 = None

        # Make new root
        if p('make_root', 'settings') or n000 is None:
            n000 = mat.MakeRoot(renderer='superfly', roottype=poser.kNodeTypeCodeCYCLESSURFACE)

        return n000

    def BuilderName(self):
        # REQUIRED: returns the name of the 'shader engine' to be displayed to the user
        return 'EZSkin CycleMaterials'

    def BuildShader(self, material, data, mat):

        # Parse the data
        shader = data[0]
        scattergroup = int(data[1])

        # Don't waste time if we don't have to do anything
        if shader == 'ignore': return 'ignored'

        # Stop if no PoserSurface is found
        if mat.Root() is None: return 'skipped - No Firefly PoserSurface found'

        # Configure imagemap nodes
        p = self.parameters
        if p('gc'): mat.GC_AllMaps()
        if p('filter'): mat.ApplyTextureFilters(p('filtering') + 1)

        # ... and build the shader
        if shader in self.Shaders():

            # Get the method which will build the shader
            method = self.shaders[shader][METHOD]

            # Check to see if we need to use a built-in shader
            if method is None and self.api.Build() >= 68:
                builder = self.api.GetBuilder(self.Renderer())
                if builder:
                    return builder.BuildShader(material, data, mat)
                else:
                    return 'failed'

            else:
                # Build the shader
                if method is not None:
                    method(mat, p)
                    mat.RemoveOrphans()
                    mat.UpdatePreview()

                return 'processed'

    def ButtonExample(self, event):
        print('Button pressed')

    def Controls(self):
        # If has_controls = True then this method must be present and return a tuple of tuples
        # (name, label, kind, minmax, default, increment=None, tooltip='', category=None)
        # See controls.txt for examples

        # Define the controls
        # (self, name, label, kind, minmax, default, increment=None, tooltip='', category=None)
        data = (
                        ('02_clothcolor', 'Cloth Colour', EZCOLOURP, (None), (0.95,0.95,0.95), None, 'Cloth color', 'Main'),
                        ('02_satincolor', 'Satin Colour', EZCOLOURP, (None), (0.95,0.95,0.95), None, 'Satin color', 'Main'),
                        ('02_velvetcolor', 'Velvet Colour', EZCOLOURP, (None), (0.95,0.95,0.95), None, 'Velvet color', 'Main'),
                        ('02_leathercolor', 'Leather Colour', EZCOLOURP, (None), (0.95,0.95,0.95), None, 'Leather color', 'Main'),
                        ('02_leatherscale', 'Leather Scale', EZFLOAT, (500.0, 2000.0), 900.0, 10.0, 'Leather Scale', 'Main'),
            ('02_leatherroughness', 'Leather Roughness', EZFLOAT, (0.05, 0.3), 0.15, 0.01, 'Leather Roughness', 'Main'),
            ('02_proceduralleather', 'Procedural leather', EZBOOL, (None), 0, None, 'Use procedural leather', 'Main'),
             ('02_nyloncolor', 'Nylon Colour', EZCOLOURP, (None), (0.0,0.0,0.0), None, 'Nylon Color', 'Main'),
            ('02_nylonshinecolor', 'Nylon Shine Colour', EZCOLOURP, (None), (0.9,0.9,0.9), None, 'Nylon Shine Color', 'Main'),
            ('02_nylondensity', 'Nylon Density', EZFLOAT, (0.1, 1.0), 0.2, 0.01, 'Nylon Density', 'Main'),
            ('02_nylonopacity', 'Nylon Opacity', EZFLOAT, (0.0, 1.0), 1.0, 0.01, 'Nylon Opacity', 'Main'),
            ('02_nylonroughness', 'Nylon Roughness', EZFLOAT, (0.05, 0.2), 0.1, 0.01, 'Nylon Roughness', 'Main'),
            ('02_nylonbumpheight', 'Nylon Bump Height', EZFLOAT, (0.0001, 0.005), 0.0005, 0.0001, 'Nylon Bump Height', 'Main'),
            ('02_plasticcolor', 'Plastic Colour', EZCOLOURP, (None), (1.0,0.0,0.0), None, 'Plastic Color', 'Main'),
            ('02_plasticroughness', 'Plastic Roughness', EZFLOAT, (0.05, 0.3), 0.1, 0.01, 'Plastic Roughness', 'Main'),
             ('02_plastictransparency', 'Plastic Transparency', EZFLOAT, (0.0, 1), 0.0, 0.01, 'Plastic Transparency', 'Main'),
            ('02_metal', 'Metal', EZCHOICE, ('Gold', 'Steel', 'Copper', 'Silver', 'Aluminum', 'Nickel'), 0, None, 'Choose type metal', 'Main'),
            ('02_metalbrushed', 'Brushed metal', EZBOOL, (None), 0, None, 'Make metal looks brushed', 'Main'),
                       )

        return data

    def Nix(self, mat, p):
        pass

    def Shaders(self):
        # Used internally by EZSkin
        return list(self.shaders.keys())

    def ShaderType(self, shader):
        # Used internally by EZSkin
        return self.shaders.get(shader, (None, 0))[SHADERTYPE]
        #Some Functions to use with the shader


    def FoamySkin(self, mat, p, nod1, nod2, transcheck):
        FSC_Dict = {'Color1' :(0.98,0.98,0.47), 'Color2' : (0.76,1.0,0.47), 'Color3':(0.976,0.619,0.99), 'Color4' :(1.0,0.976,0.929)}
        FS_Dict = {'x_Index' : 4.0, 'y_Index': 4.0, 'z_Index': 4.0, 'Octaves' : 5.0, 'Bottom': 0.4, 'Bias': 0.6, 'Gain' : 0.5, 'Offset_X': 0.5, 'Offset_Y': 0.5, 'Offset_Z': 0.5}
        TB_Dict = {'x_Index' : 0.5, 'y_Index': 0.5, 'z_Index': 0.5, 'Octaves' : 4.0, 'Bottom': 0.0, 'Bias': 0.5, 'Gain' : 0.5, 'Noise_Type' : 2,'Offset_X': 0.5, 'Offset_Y': 0.5, 'Offset_Z': 0.5}
        nf01 = mat.AddNode('ccl_MixClosure', name = 'FoamClosure', loc=(20,800), collapsed=False, preview=False,)
        nf02 = mat.AddNode('ccl_MixClosure', name = 'FoamClosure1', loc=(20,1000), collapsed=False, preview=False,)
        nf03 = mat.AddNode('ccl_MixClosure', name = 'FoamClosure2', loc=(200,1000), collapsed=False, preview=False,)
        nf04 = mat.AddNode('ccl_Fresnel', name='Fresnel foam', loc=(200,800), collapsed=False, preview=False, IOR=1.33)
        nf05 = mat.AddNode('ccl_SubsurfaceScattering', name='FoamSSS', loc=(400,800), collapsed=False, preview=False, Scale=0.75, Falloff=1,)
        nf06 = mat.AddNode('ccl_DiffuseBsdf', name = 'White foam', loc=(400,1000),  collapsed=False, preview=False, Color=(1,1,1), Roughness=0.2,)
        nf07 = mat.AddNode('ccl_GlossyBsdf', name = 'FoamGloss', loc=(400,1200), collapsed=False, preview=False, Roughness=0.03, Distribution = 3)
        nf08 = mat.AddNode('cellular', name='Final Foam Shape', loc=(600,800), collapsed=False, preview=False, Color1=(1.0,1.0,1.0), Color2=(0.98,0.98,0.98), ScaleX=0.1,ScaleY=0.2,ScaleZ=0.1, cellMode=3, Jitter=0.939, Turbulence=0.862, Noise_Type=2,)
        nf09 = mat.AddNode('hsv', name='Scattercolor', loc=(600,1200), collapsed=False, preview=False, Saturation =0.5,)
        nf10 = mat.AddNode('colorramp', name='FoamShineColor', loc=(600,1400), collapsed=False, preview=False, inputs=FSC_Dict)
        nf11 = mat.AddNode('ccl_Bump', name='FoamBump', loc=(600,1600), collapsed=False, preview=False, Distance=0.03)
        nf12 = mat.AddNode('marble', name='FoamPattern', loc=(800,800),collapsed=False, preview=False, Base_Color=(1,1,1), Veins=(0.77,0.776,0.776), Scale=1.59, Turbulence=2.264, Noise_Type=2,)
        nf13 = mat.AddNode('fractal_sum', name='FoamShape', loc=(800,1000),collapsed=False, preview=False, inputs=FS_Dict)
        nf14 = mat.AddNode('turbulence', name= 'Bubbles', loc=(800,1400),collapsed=False, preview=False, inputs=TB_Dict)

        # Connect Nodes
        if transcheck is False :
            mat.ConnectNodes(nf01, nod1, 'Surface')
        else:
            mat.ConnectNodes(nf01, nod1, 'Closure2')
        mat.ConnectNodes(nod2, nf01, 'Closure1')                       
        mat.ConnectNodes(nf02, nf01, 'Closure2')
        mat.ConnectNodes(nf08, nf01, 'Fac')
        mat.ConnectNodes(nf03, nf02, 'Closure1')
        mat.ConnectNodes(nf07, nf02, 'Closure2')
        mat.ConnectNodes(nf04, nf02, 'Fac')
        mat.ConnectNodes(nf08, nf03, 'Fac')
        mat.ConnectNodes(nf05, nf03, 'Closure1')
        mat.ConnectNodes(nf06, nf03, 'Closure2')
        mat.ConnectNodes(nf08, nf05, 'Color')
        mat.ConnectNodes(nf11, nf05, 'Normal')
        mat.ConnectNodes(nf09, nf05, 'Radius')
        mat.ConnectNodes(nf11, nf06, 'Normal')
        mat.ConnectNodes(nf10, nf07, 'Color')
        mat.ConnectNodes(nf11, nf07, 'Normal')
        mat.ConnectNodes(nf13, nf08, 'Color1')
        mat.ConnectNodes(nf13, nf08, 'Color2')
        mat.ConnectNodes(nf12, nf08, 'ScaleX')
        mat.ConnectNodes(nf12, nf08, 'ScaleY')
        mat.ConnectNodes(nf12, nf08, 'ScaleZ')
        mat.ConnectNodes(nf10, nf09, 'Color')
        mat.ConnectNodes(nf08, nf11, 'Height')
        mat.ConnectNodes(nf14, nf10, 'Input')


    def GreasySkin(self, mat, p, nod1, nod2, transcheck):
        clodict = {'Sky_Color' : (1.0,1.0,1.0), 'Cloud_Color' : (0.0,0.0,0.0), 'Scale': 3, 'Complexity' : 6.0, 'Bottom': 0.2, 'Bias': 0.5, 'Gain': 0.6}
        ng01=mat.AddNode('ccl_MixClosure', name = 'GreaseClosure1', loc=(200,1200),collapsed=False, preview=False,)
        ng02=mat.AddNode('ccl_MixClosure', name = 'GreaseClosure2', loc=(450,1300),collapsed=False, preview=False,)
        ng03=mat.AddNode('ccl_MixClosure', name = 'GreaseClosure1', loc=(700,1400),collapsed=False, preview=False, Fac=p('02_glossyeffect'),)
        ng04=mat.AddNode('ccl_Fresnel', name = 'GreaseFresnel', loc=(700,1200),collapsed=False, preview=False, IOR=1.5,)
        ng05=mat.AddNode('ccl_GlossyBsdf', name = 'HighGreaseGlow', loc=(700,1600),collapsed=False, preview=False, Color=(0.55,0.55,0.55), Roughness=0.05,)
        ng06=mat.AddNode('ccl_GlossyBsdf', name = 'LowGreaseGlow', loc=(950,1500),collapsed=False, preview=False, Color=(0.4,0.4,0.4), Roughness=0.1,)
        ng07=mat.AddNode('ccl_DiffuseBsdf', name = 'GreaseColor', loc=(950,1300),collapsed=False, preview=False, Color=(0.2235,0.1568,0), Roughness=0.3,)
        ng08=mat.AddNode('math_functions', name = 'ReverseClouds',loc=(950,1100),collapsed=False, preview=False, Math_Argument = 2, Value_1 = 1.0, Value_2=1.0,)
        ng09=mat.AddNode('clouds',name = 'GreasePattern', loc = (1200,1100),collapsed=False, preview=False, inputs = clodict,)

        # Connect Nodes

        if transcheck is False :
                mat.ConnectNodes(ng01, nod1, 'Surface')
        else:
                mat.ConnectNodes(ng01, nod1, 'Closure2')
        mat.ConnectNodes(nod2, ng01, 'Closure1')
        mat.ConnectNodes(ng02, ng01, 'Closure2')
        mat.ConnectNodes(ng08, ng01, 'Fac')
        mat.ConnectNodes(ng04, ng02, 'Fac')
        mat.ConnectNodes(ng03, ng02, 'Closure1')
        mat.ConnectNodes(ng05, ng02, 'Closure2')
        mat.ConnectNodes(ng07, ng03, 'Closure1')
        mat.ConnectNodes(ng06, ng03, 'Closure2')
        mat.ConnectNodes(ng09, ng08, 'Value_2')

                    
    def YellowCustard(self, mat, p, nod1, nod2, transcheck):

        fracdict = {'Octaves': 4.0 , 'Bottom': 0.2 , 'Bias' : 0.6 , 'Gain': 0.6}
        ny01=mat.AddNode('ccl_MixClosure', name = 'YecClosure1', loc=(200,1200),collapsed=False, preview=False,)
        ny02=mat.AddNode('ccl_MixClosure', name = 'YecClosure2', loc=(450,1200),collapsed=False, preview=False, Fac=p('02_glossyeffect'),)
    #                ny03=mat.AddNode('ccl_Fresnel', name = 'YecFresnel', loc=(700,800),collapsed=False, preview=False, IOR= 1.35,)
        ny04=mat.AddNode('ccl_SubsurfaceScattering', name = 'YecSSS', loc=(700,1200),collapsed=False, preview=False, Color = p('02_custardcolor'), Scale = 0.006,Radius = (1.0,1.0,0.5),)
        ny05=mat.AddNode('ccl_GlossyBsdf', name = 'YecGlossy', loc=(700,1600),collapsed=False, preview=False, Color = (0.77,0.77,0.77), Roughness = 0.05,)
        ny06=mat.AddNode('ccl_Bump', name = 'YecBump', loc = (900,1200),collapsed=False, preview=False, Distance = 0.01,)
        ny07=mat.AddNode('fractal_sum', name = 'Yecpattern', loc=(1150,1200),collapsed=False, preview=False, inputs = fracdict,)

        # ConnectNodes

        if transcheck is False :
                mat.ConnectNodes(ny01, nod1, 'Surface')
        else:
                mat.ConnectNodes(ny01, nod1, 'Closure2')
        mat.ConnectNodes(nod2, ny01, 'Closure1')
        mat.ConnectNodes(ny02, ny01, 'Closure2')
        mat.ConnectNodes(ny07, ny01, 'Fac')
    #                mat.ConnectNodes(ny03, ny02, 'Fac')
        mat.ConnectNodes(ny05, ny02, 'Closure2')
        mat.ConnectNodes(ny04, ny02, 'Closure1')
    #                mat.ConnectNodes(ny06, ny03, 'Normal')
        mat.ConnectNodes(ny06, ny04, 'Normal')
        mat.ConnectNodes(ny06, ny05, 'Normal')
        mat.ConnectNodes(ny07, ny06, 'Height')
                             



    def Mud(self, mat, p, nod1, nod2, transcheck):

        mapdict ={ 'Vector Type' : 4.0, 'Rotation' : (0.0,0.0,1.570796), 'Scale' : (2.5,1.0,2.5) }
        fracdict = {'Octaves': 5.0 , 'Bottom': 0.1 , 'Bias' : 0.8 , 'Gain': 0.6}
        nm01=mat.AddNode('ccl_MixClosure', name = 'MudClosure1', loc=(200,1200),collapsed=False, preview=False,)
        nm02=mat.AddNode('ccl_MixClosure', name = 'MudClosure2', loc=(450,1200),collapsed=False, preview=False, Fac=p('02_glossyeffect'))
        nm04=mat.AddNode('ccl_DiffuseBsdf', name = 'MudColor', loc=(700,1200),collapsed=False, preview=False, Color = p('02_mudcolor'),Roughness = 0.3,)
        nm05=mat.AddNode('ccl_GlossyBsdf', name = 'MudGlossy', loc=(700,1400),collapsed=False, preview=False, Color = (0.5,0.5,0.5), Roughness = 0.05,)
        nm06=mat.AddNode('ccl_Bump', name = 'MudBump', loc=(900,1200),collapsed=False, preview=False, Distance = 0.001,)
        nm07=mat.AddNode('fractal_sum', name = 'Mudpattern', loc=(1150,1200),collapsed=False, preview=False, inputs = fracdict,)
        nm08 = mat.AddNode('ccl_MusgraveTexture', name = 'Mudstructure', loc=(800,1200), collapsed=False, preview=False, Scale=100.0, Detail = 1, Type = 1.0,)
        nm09 = mat.AddNode('ccl_Mapping', name = 'Vertical Drops', loc=(1000,1200), collapsed=False, preview=False, inputs =mapdict)
        nm10 = mat.AddNode('ccl_TextureCoordinate', name = 'TextureCoordinates', loc=(1200,1200), collapsed=False, preview=False)

        # ConnectNodes

        if transcheck is False :
                mat.ConnectNodes(nm01, nod1, 'Surface')
        else:
                mat.ConnectNodes(nm01, nod1, 'Closure2')
        mat.ConnectNodes(nod2, nm01, 'Closure1')
        mat.ConnectNodes(nm02, nm01, 'Closure2')
        mat.ConnectNodes(nm07, nm01, 'Fac')
        mat.ConnectNodes(nm04, nm02, 'Closure1')
        mat.ConnectNodes(nm05, nm02, 'Closure2')
        mat.ConnectNodes(nm06, nm04, 'Normal')
        mat.ConnectNodes(nm06, nm05, 'Normal')
        mat.ConnectNodes(nm08, nm06, 'Height')
        mat.ConnectNodes(nm09, nm08, 'Vector')
        mat.ConnectNodes(nm10, nm09, 'Vector')
                                 
                

    # Cloth shader building method
    def CycleCloth(self, mat, p):
                

        # Prepare the root node
        n000 = self._PrepareRoot(mat, p)

        #Get Original Color
        D_color = mat.PSValue('Diffuse_Color')

        #Dictionary for the mix input
        mixdict = { 'Fac': 1.0, 'Color1' : D_color, 'Blend Type' : 22}
        

        #Prepare the Nodes
        weavedict = {'Color_1' : (0.937,0.937,0.937), 'Color_2' : (0.937,0.937,0.937), 'Base_Color' : (0.192,0.192,0.192), 'U_Scale' : 350.0, 'V_Scale' : 350.0, 'Height': 1.0, 'Bias' : 0.0, 'Gain': 0.2}
        nc01=mat.AddNode('ccl_MixClosure', name = 'ClothClosure', loc= (200,200), collapsed=False, preview=False, Fac=0.2)
        nc03=mat.AddNode('ccl_DiffuseBsdf', name = 'Cloth Color', loc = (450,400), collapsed=False, preview=False, Roughness = 0.3, )
        nc04=mat.AddNode('ccl_GlossyBsdf', name = 'ClothShine', loc = (450,700), collapsed=False, preview=False, Color = (0.6,0.6,0.6), Roughness = 0.5,)
        nc05=mat.AddNode('ccl_Bump', name = 'ClothBump', loc=(700,400), collapsed=False, preview=False, Distance = 0.003,)
        nc06=mat.GetMap('color')
        nc07=mat.GetMap('bump')
        nc08=mat.GetMap('transparency')
        
        if nc06 is None :
            nc11 = mat.AddNode('ccl_Color', name =  'ClothColor', loc = (700,200), collapsed=False, preview=False, Color = p('02_clothcolor'),)
        else :
            nc11 = mat.AddNode('ccl_Mix', name = 'Colormix', loc = (700,200), collapsed=False, preview=False, inputs = mixdict)
        if nc07 is None :
            nc07=mat.AddNode('weave', name = 'ProceduralCloth', loc = (700,700), collapsed=False, preview=False, inputs = weavedict)

        #Connect Nodes
        if nc08 is None:
            mat.ConnectNodes(nc01, n000, 'Surface')
        else:
            nc09=mat.AddNode('ccl_MixClosure', name = 'Transparant Closure', loc = (200,0), collapsed=False, preview=False,)
            nc10=mat.AddNode('ccl_TransparentBsdf', name = 'Transparent Cloth', loc=(450,0), collapsed=False, preview=False, Color = (1.0,1.0,1.0),)
            mat.ConnectNodes(nc09, n000, 'Surface')
            mat.ConnectNodes(nc08, nc09, 'Fac')
            mat.ConnectNodes(nc10, nc09, 'Closure1')
            mat.ConnectNodes(nc01, nc09, 'Closure2')
        mat.ConnectNodes(nc03, nc01, 'Closure1')
        mat.ConnectNodes(nc04, nc01, 'Closure2')
        mat.ConnectNodes(nc05, nc03, 'Normal')
        mat.ConnectNodes(nc05, nc04, 'Normal')
        mat.ConnectNodes(nc11, nc03, 'Color')
        if nc06 is not None :
            mat.ConnectNodes(nc06, nc11, 'Color2')
        mat.ConnectNodes(nc07, nc05, 'Height')




        # Foamaddon
        

        if p('02_foam') is True:
            if nc08 is None:
                self.FoamySkin(mat, p, n000, nc01, False)
            else:
                self.FoamySkin(mat, p, nc09, nc01, True)



        #Greasy

        if p('02_effects') == 1:
            if nc08 is None:
                    self.GreasySkin(mat, p, n000, nc01, False)
            else:
                    self.GreasySkin(mat, p, nc09, nc01, True)                                

        # Yellow Custard Cover

        if p('02_effects') == 2:
            if nc08 is None:
                self.YellowCustard(mat, p, n000, nc01, False)
            else:
                self.YellowCustard(mat, p, nc09, nc01, True)


        # Muddy

        if p('02_effects') == 3:
            if nc08 is None:
                self.Mud(mat, p, n000, nc01, False)
            else:
                self.Mud(mat, p, nc09, nc01, True)


    # Metal shader building mehod
    def CycleMetal(self, mat, p):

        # Prepare the root node
        n000 = self._PrepareRoot(mat, p)

        #setup the right color
        metalcolor=[(1,0.882,0.6078),(0.7608,0.7765,0.7804),(0.9804,0.8118,0.7490),(0.9843,0.9804,0.9608),(0.9608,0.9686,0.9647),(0.8235,0.7921,0.7411)]


        #Prepare the nodes
        if p('02_metalbrushed') is False :
                nm01 = mat.AddNode('ccl_GlossyBsdf', name = 'Glossy metal', loc=(400,0),collapsed=False, preview=False, Color = metalcolor[p('02_metal')], Roughness=0.02,)
        else :
                nm01 = mat.AddNode('ccl_AnisotropicBsdf', name = 'Brushed metal', loc=(400,0),collapsed=False, preview=False, Color = metalcolor[p('02_metal')], Roughness = 0.2,)

        #Connect the nodes
        mat.ConnectNodes(nm01, n000, 'Surface')
                        


        # Foamaddon
        

        if p('02_foam') is True:
            self.FoamySkin(mat, p, n000, nm01, nw01)



        #Greasy

        if p('02_effects') == 1:
            self.GreasySkin(mat, p, n000, nm01, False)

        # Yellow Custard Cover

        if p('02_effects') == 2:
            self.YellowCustard(mat, p, n000, nm01, False)

        # Muddy

        if p('02_effects') == 3:
            self.Mud(mat, p, n000, nm01, False)
        


    # Leather shader building mehod
    def CycleLeather(self, mat, p):

        # Prepare the root node
        n000 = self._PrepareRoot(mat, p)

        #Prepare the Nodes
        nl01=mat.AddNode('ccl_MixClosure', name =  'LeatherClosure', loc= (200,0),collapsed=False, preview=False,)
        nl02=mat.AddNode('ccl_Fresnel', name = 'LeatherFresnel', loc=(450,0),collapsed=False, preview=False, IOR=1.6)
        nl03=mat.AddNode('ccl_GlossyBsdf', name = 'Leather low Gloss', loc = (450,200),collapsed=False, preview=False, Color=p('02_leathercolor'), Roughness = 0.5,)
        nl04=mat.AddNode('ccl_GlossyBsdf', name =  'Leather high Gloss', loc = (450,400),collapsed=False, preview=False, Color=(0.973,0.973,0.973), Roughness = p('02_leatherroughness'),)
        nl05=mat.AddNode('ccl_Bump', name = 'Leather Bump', loc = (750,0),collapsed=False, preview=False, Distance=0.0005,)
        nl06=mat.AddNode('ccl_Math', name = 'Lower Bump', loc = (750,400),collapsed=False, preview=False, Value1=0.1, Type = 21,)
        nl07=mat.AddNode('ccl_Invert', name = 'Invert Texture', loc = (900,400),collapsed=False, preview=False, Fac =0.4)
        nl08=mat.GetMap('bump')
        if nl08 is None or p('02_proceduralleather') is True :
            nl10=mat.AddNode('ccl_VoronoiTexture', name = 'Leather Pattern', loc = (1150,400),collapsed=False, preview=False, Scale = p('02_leatherscale'), Coloring = 2,)
        nl09=mat.GetMap('color')

        #Connect the Nodes

        mat.ConnectNodes(nl01, n000, 'Surface')
        mat.ConnectNodes(nl02, nl01, 'Fac')
        mat.ConnectNodes(nl03, nl01, 'Closure1')
        mat.ConnectNodes(nl04, nl01, 'Closure2')
        mat.ConnectNodes(nl05, nl02, 'Normal')
        mat.ConnectNodes(nl05, nl03, 'Normal')
        mat.ConnectNodes(nl05, nl04, 'Normal')
        if nl08 is None or p('02_proceduralleather') is True :
            mat.ConnectNodes(nl06, nl05, 'Height')
            mat.ConnectNodes(nl07, nl06, 'Value2')
            mat.ConnectNodes(nl10, nl07, 'Color')
        else :
            mat.ConnectNodes(nl08, nl05, 'Height')
        if nl09 is True :
            mat.ConnectNodes(nl09, nl03, 'Color')




        # Foamaddon
        

        if p('02_foam') is True:
            self.FoamySkin(mat, p, n000, nl01, False)



        #Greasy

        if p('02_effects') == 1:
            self.GreasySkin(mat, p, n000, nl01, False)

        # Yellow Custard Cover

        if p('02_effects') == 2:
            self.YellowCustard(mat, p, n000, nl01, False)

        # Muddy

        if p('02_effects') == 3:
            self.Mud(mat, p, n000, nl01,False)
        

    #Nylon shader building mehod
    def CycleNylon(self, mat, p):

        # Prepare the root node
        n000 = self._PrepareRoot(mat, p)

        #Get Original Color
        D_color = mat.PSValue('Diffuse_Color')

        #Dictionary for the mix input
        mixdict = { 'Fac': 1.0, 'Color1' : D_color, 'Blend Type' : 22}

                #Prepare the Nodes
        weavedict = {'Color_1' : (0.937,0.937,0.937), 'Color_2' : (0.937,0.937,0.937), 'Base_Color' : (0.192,0.192,0.192), 'U_Scale' : 700.0, 'V_Scale' : 700.0, 'Height': 1.0, 'Bias' : 0.0, 'Gain': 0.2}
        nn01=mat.AddNode('ccl_MixClosure', name = 'NylonClosure1', loc=(100,500),collapsed=False, preview=False,)
        nn02=mat.AddNode('ccl_MixClosure', name = 'NylonClosure2', loc=(300,600),collapsed=False, preview=False,)
        nn03=mat.AddNode('ccl_TransparentBsdf', name = 'TransparentBsdf', loc=(300,500),collapsed=False, preview=False,Color=(1.0,1.0,1.0),)
        nn04=mat.AddNode('ccl_Fresnel', name = 'Nylon Fresnel', loc=(500,500),collapsed=False, preview=False, IOR=1.5)
        nn05=mat.AddNode('ccl_DiffuseBsdf', name = 'NylonColor', loc=(500,600),collapsed=False, preview=False, Color =p('02_nyloncolor'), Roughness = 0.3)
        nn06=mat.AddNode('ccl_AnisotropicBsdf', name = 'NylonShine', loc=(500,700),collapsed=False, preview=False, Color =p('02_nylonshinecolor'), Anisotropy = 0.6,Roughness = p('02_nylonroughness'),)
        nn07=mat.AddNode('math_functions',name= '**factor', loc=(100,0),collapsed=False, preview=False,Math_Argument=9, Value_1=1.0, Value_2=1.0)
        nn08=mat.AddNode('math_functions',name= 'TransValue', loc=(100,100),collapsed=False, preview=False,Math_Argument=2, Value_1=1.0, Value_2=1.0)
        nn09=mat.AddNode('math_functions',name= 'Factor', loc=(300,0),collapsed=False, preview=False,Math_Argument=4, Value_1=1.0, Value_2=1.0)
        nn10=mat.AddNode('math_functions',name= 'y3*b', loc=(300,100),collapsed=False, preview=False,Math_Argument=3, Value_1=1.0, Value_2=1.0)
        nn11=mat.AddNode('math_functions',name= '1-y3*b', loc=(300,200),collapsed=False, preview=False,Math_Argument=2, Value_1=1.0, Value_2=1.0)
        nn12=mat.AddNode('math_functions',name= 'Sqrt', loc=(500,0),collapsed=False, preview=False,Math_Argument=8, Value_1=1.0, Value_2=0.0)
        nn13=mat.AddNode('math_functions',name= 'Add', loc=(500,100),collapsed=False, preview=False,Math_Argument=1, Value_1=1.0, Value_2=1.0)
        nn14=mat.AddNode('math_functions',name= '*gx', loc=(500,200),collapsed=False, preview=False,Math_Argument=3, Value_1=1.0, Value_2=1.0)
        nn15=mat.AddNode('math_functions',name= '1-b', loc=(500,300),collapsed=False, preview=False,Math_Argument=2, Value_1=1.0, Value_2=1.0)
        nn16=mat.AddNode('math_functions',name= 'gx', loc=(700,0),collapsed=False, preview=False,Math_Argument=2, Value_1=1.0, Value_2=1.0)
        nn17=mat.AddNode('math_functions',name= 'Divide', loc=(700,100),collapsed=False, preview=False,Math_Argument=4, Value_1=1.0, Value_2=1.0)
        nn18=mat.AddNode('math_functions',name= 'b*fx', loc=(700,200),collapsed=False, preview=False,Math_Argument=3, Value_1=1.0, Value_2=1.0)
        nn19=mat.AddNode('math_functions',name= 'y3', loc=(700,300),collapsed=False, preview=False,Math_Argument=2, Value_1=1.0, Value_2=1.0)
        nn20=mat.AddNode('math_functions',name= 'fx', loc=(900,0),collapsed=False, preview=False,Math_Argument=9, Value_1=1.0, Value_2=2.0)
        nn21=mat.AddNode('math_functions',name= '1-fx', loc=(900,100),collapsed=False, preview=False,Math_Argument=2, Value_1=1.0, Value_2=1.0)
        nn22=mat.AddNode('math_functions',name= '*IOR**2', loc=(900,200),collapsed=False, preview=False,Math_Argument=9, Value_1=1.0, Value_2=2.0)
        nn23=mat.AddNode('math_functions',name= '2*a', loc=(900,300),collapsed=False, preview=False,Math_Argument=3, Value_1=1.0, Value_2=2.0)
        nn24=mat.AddNode('math_functions',name= 'a*a', loc=(900,400),collapsed=False, preview=False,Math_Argument=9, Value_1=1.0, Value_2=2.0)
        nn25=mat.AddNode('math_functions',name= 'Density(a)', loc=(1100,100),collapsed=False, preview=False,Math_Argument=1, Value_1=p('02_nylondensity'), Value_2=0.0)
        nn26=mat.AddNode('math_functions',name= 'Opacity(b)', loc=(1100,200),collapsed=False, preview=False,Math_Argument=1, Value_1=p('02_nylonopacity'), Value_2=0.0)
        nn27=mat.AddNode('math_functions',name= 'IOR', loc=(1100,300),collapsed=False, preview=False,Math_Argument=1, Value_1=1.53, Value_2=0.0)
        nn29=mat.AddNode('edge_blend', name='Edge_Blend', loc=(1100,0),collapsed=False, preview=False, Inner_Color=(1.0,1.0,1.0), Outer_Color=(0.0,0.0,0.0),Attenuation=1.0)
        nn30=mat.GetMap('color')
        if nn30 is None:
            nn34=mat.AddNode('ccl_Color', name= 'NylonColor', loc=(700,400),collapsed=False, preview=False,Color=p('02_nyloncolor'),)
        else :
            nn34=mat.AddNode('ccl_Mix', name = 'Mixcolor', loc=(700,400),collapsed=False, preview=False, inputs = mixdict)
        nn31=mat.GetMap('bump')
        if nn31 is None:
            nn31=mat.AddNode('weave', name= 'procedural bump', loc=(900,500),collapsed=False, preview=False, inputs=weavedict,)
        nn32=mat.GetMap('transparency')
        nn33=mat.AddNode('ccl_Bump',name= 'Bump2Normal', loc=(700,600),collapsed=False, preview=False, Distance=0.001,)


        #Connect the Nodes
        mat.ConnectNodes(nn01, n000, 'Surface')
        mat.ConnectNodes(nn07, nn08, 'Value_2')
        mat.ConnectNodes(nn08, nn01, 'Fac')
        mat.ConnectNodes(nn09, nn07, 'Value_2')
        mat.ConnectNodes(nn10, nn11, 'Value_2')
        mat.ConnectNodes(nn11, nn07, 'Value_1')
        mat.ConnectNodes(nn03, nn01, 'Closure1')
        mat.ConnectNodes(nn02, nn01, 'Closure2')
        mat.ConnectNodes(nn12, nn09, 'Value_2')
        mat.ConnectNodes(nn13, nn12, 'Value_1')
        mat.ConnectNodes(nn14, nn13, 'Value_1')
        mat.ConnectNodes(nn15, nn14, 'Value_2')
        mat.ConnectNodes(nn04, nn02, 'Fac')
        mat.ConnectNodes(nn05, nn02, 'Closure1')
        mat.ConnectNodes(nn06, nn02, 'Closure2')
        mat.ConnectNodes(nn16, nn14, 'Value_1')
        mat.ConnectNodes(nn17, nn16, 'Value_2')
        mat.ConnectNodes(nn18, nn13, 'Value_2')
        mat.ConnectNodes(nn19, nn10, 'Value_1')
        mat.ConnectNodes(nn33, nn04, 'Normal')
        mat.ConnectNodes(nn33, nn05, 'Normal')
        mat.ConnectNodes(nn33, nn06, 'Normal')
        mat.ConnectNodes(nn20, nn18, 'Value_2')
        mat.ConnectNodes(nn20, nn21, 'Value_2')
        mat.ConnectNodes(nn21, nn17, 'Value_1')
        mat.ConnectNodes(nn22, nn17, 'Value_2')
        mat.ConnectNodes(nn23, nn19, 'Value_1')
        mat.ConnectNodes(nn24, nn19, 'Value_2')
        mat.ConnectNodes(nn31, nn33, 'Height')
        mat.ConnectNodes(nn34, nn05, 'Color')
        if nn30 is not None:
            mat.ConnectNodes(nn30, nn34, 'Color2')
        mat.ConnectNodes(nn29, nn20, 'Value_1')
        mat.ConnectNodes(nn25, nn23, 'Value_1')
        mat.ConnectNodes(nn25, nn24, 'Value_1')
        mat.ConnectNodes(nn26, nn10, 'Value_2')
        mat.ConnectNodes(nn26, nn18, 'Value_1')
        mat.ConnectNodes(nn26, nn15, 'Value_2')
        mat.ConnectNodes(nn27, nn22, 'Value_1')
        mat.ConnectNodes(nn27, nn04, 'IOR')
        if nn32 is not None:
            mat.ConnectNodes(nn32, nn26, 'Value_1')
                
                
                


                
                



        # Foamaddon
        

        if p('02_foam') is True:
            self.FoamySkin(mat, p, n000, nn01, False)



        #Greasy

        if p('02_effects') == 1:
            self.GreasySkin(mat, p, n000, nn01, False)

        # Yellow Custard Cover

        if p('02_effects') == 2:
            self.YellowCustard(mat, p, n000, nn01, False)

        # Muddy

        if p('02_effects') == 3:
            self.Mud(mat, p, n000, nn01, False)
        
                        
    #Plastic shader building mehod
    def CyclePlastic(self, mat, p):

        # Prepare the root node
        n000 = self._PrepareRoot(mat, p)

        #Prepare the nodes
        np01 = mat.AddNode('ccl_MixClosure', name= 'Plastic Closure', loc = (200,0),collapsed=False, preview=False, Fac =p('02_plastictransparency'), )
        np02 = mat.AddNode('ccl_MixClosure', name= 'Plastic Closure', loc = (450,150),collapsed=False, preview=False,)
        np03 = mat.AddNode('ccl_TransparentBsdf', name= 'Plastic Transparency', loc= (450,0),collapsed=False, preview=False, Color = (1.0,1.0,1.0),)
        np04 = mat.AddNode('ccl_Fresnel', name = 'Plastic Fresnel', loc = (600,0),collapsed=False, preview=False, IOR = 1.5,)
        np05 = mat.AddNode('ccl_DiffuseBsdf', name =  'Plastic Color', loc=(600,150),collapsed=False, preview=False, Color=p('02_plasticcolor'),)
        np06 = mat.AddNode('ccl_GlossyBsdf', name =  'Plastic Reflections', loc=(600,300),collapsed=False, preview=False, Color = (0.98,0.98,0.98), Roughness = p('02_plasticroughness'),)
        np07 = mat.GetMap('color')
        np08 = mat.GetMap('transparency')
        np09 = mat.GetMap('bump')

        #Connect the Nodes
        mat.ConnectNodes(np01, n000, 'Surface')
        mat.ConnectNodes(np02, np01, 'Closure1')
        mat.ConnectNodes(np03, np01, 'Closure2')
        mat.ConnectNodes(np04, np02, 'Fac')
        mat.ConnectNodes(np05, np02, 'Closure1')
        mat.ConnectNodes(np06, np02, 'Closure2')
        if np07 is not None:
             mat.ConnectNodes(np07, np05, 'Color')
        if np08 is not None:
             mat.ConnectNodes(np08, np01, 'Fac')
        if np09 is not None:
             np10=mat.AddNode('ccl_Bump', name= 'Plastic Bump', loc=(900,0),collapsed=False, preview=False, Distance = 0.001,)
             mat.ConnectNodes(np10, np04, 'Normal')
             mat.ConnectNodes(np10, np05, 'Normal')
             mat.ConnectNodes(np10, np06, 'Normal')
             mat.ConnectNodes(np09, np10, 'Height')





        # Foamaddon
        

        if p('02_foam') is True:
            self.FoamySkin(mat, p, n000, np01, False)



        #Greasy

        if p('02_effects') == 1:
            self.GreasySkin(mat, p, n000, np01, False)

        # Yellow Custard Cover

        if p('02_effects') == 2:
            self.YellowCustard(mat, p, n000, np01, False)

        # Muddy

        if p('02_effects') == 3:
            self.Mud(mat, p, n000, np01,False)
        
    #Satin shader building mehod
    def CycleSatin(self, mat, p):

        # Prepare the root node
        n000 = self._PrepareRoot(mat, p)

        #Prepare the root node
        ns01 = mat.AddNode('ccl_MixClosure', name = 'Satin Closure', loc=(200,0),collapsed=False, preview=False, Fac=0.2,)
        ns02 = mat.AddNode('ccl_DiffuseBsdf', name = 'Satin Color', loc = (450,0),collapsed=False, preview=False, Color = p('02_satincolor'), Roughness = 0.3)
        ns03 = mat.AddNode('ccl_GlossyBsdf', name = 'Satin Shine', loc = (450,150),collapsed=False, preview=False, Color = (0.92,0.92,0.92), Roughness =0.35)
        ns04 = mat.GetMap('color')
        ns06 = mat.GetMap('transparency')
        ns05 = mat.GetMap('bump')
        if ns05 is not None:
            ns07=mat.AddNode('ccl_Bump', name = 'Satin Bump', loc = (700,0),collapsed=False, preview=False, Distance = 0.001,)

        #Connect the Nodes
        if ns06 is None:
            mat.ConnectNodes(ns01, n000, 'Surface')
        else:
            ns07=mat.AddNode('ccl_MixClosure', name = 'Transparant Closure', loc = (200,0), collapsed=False, preview=False,)
            ns08=mat.AddNode('ccl_TransparentBsdf', name = 'Transparent Cloth', loc=(450,0), collapsed=False, preview=False, Color = (1.0,1.0,1.0),)
            mat.ConnectNodes(ns07, n000, 'Surface')
            mat.ConnectNodes(ns06, ns07, 'Fac')
            mat.ConnectNodes(ns08, ns07, 'Closure1')
            mat.ConnectNodes(ns01, ns07, 'Closure2')                       
        mat.ConnectNodes(ns02, ns01, 'Closure1')
        mat.ConnectNodes(ns03, ns01, 'Closure2')
        if ns04 is not None:
            mat.ConnectNodes(ns04, ns02, 'Color')
        if ns05 is not None:
            mat.ConnectNodes(ns07, ns02, 'Normal')
            mat.ConnectNodes(ns07, ns03, 'Normal')
            mat.ConnectNodes(ns05, ns07, 'Height')
                
                
        # Foamaddon
        

        if p('02_foam') is True:
            if ns06 is None:
                    self.FoamySkin(mat, p, n000, ns01, False)
            else:
                    self.FoamySkin(mat, p, ns07, ns01, True)        

        
        
        #Greasy

        if p('02_effects') == 1:
            if ns06 is None:
                self.GreasySkin(mat, p, n000, ns01, False)
            else:
                self.GreasySkin(mat, p, ns07, ns01, True)                                

        # Yellow Custard Cover

        if p('02_effects') == 2:
            if ns06 is None:
                self.YellowCustard(mat, p, n000, ns01, False)
            else:
                self.YellowCustard(mat, p, ns07, ns01, True)


        # Muddy

        if p('02_effects') == 3:
            if ns06 is None:
                self.Mud(mat, p, n000, ns01, False)
            else:
                self.Mud(mat, p, ns07,ns01, True)
                                
    #Velvet shader building mehod
    def CycleVelvet(self, mat, p):

        # Prepare the root node
        n000 = self._PrepareRoot(mat, p)

        #Prepare the root node
        nv01 = mat.AddNode('ccl_MixClosure', name = 'Velvet Closure', loc =(200,0),collapsed=False, preview=False, Fac=0.2,)
        nv02 = mat.AddNode('ccl_DiffuseBsdf', name = 'Velvet Color' , loc =(450,0),collapsed=False, preview=False, Color = p('02_velvetcolor'), Roughness = 0.3,)
        nv03 = mat.AddNode('ccl_VelvetBsdf', name = 'VelvetShader', loc = (450,200),collapsed=False, preview=False, Color = (1.0,1.0,1.0), Sigma = 0.4)
        nv04 = mat.AddNode('ccl_Bump' , name = 'Bump2Normal', loc = (600,100),collapsed=False, preview=False, Distance = 0.001)
        nv05 = mat.AddNode('granite', name = 'Structure', loc = (850,100),collapsed=False, preview=False, Base_Color =(0.6,0.6,0.6), Spot_Color = (1,1,1), Shades = 3, Balance = 0.5)
        nv06 = mat.GetMap('color')

        #Connect the Nodes
        mat.ConnectNodes(nv01, n000, 'Surface')
        mat.ConnectNodes(nv02, nv01, 'Closure1')
        mat.ConnectNodes(nv03, nv01, 'Closure2')
        mat.ConnectNodes(nv04, nv02, 'Normal')
        mat.ConnectNodes(nv04, nv03, 'Normal')
        mat.ConnectNodes(nv05, nv04, 'Height')
        if nv06 is True :
            mat.ConnectNodes(nv06, nv02, 'Color')
        
        # Foamaddon
        

        if p('02_foam') is True:
            self.FoamySkin(mat, p, n000, nv01, False)

        
        
        #Greasy

        if p('02_effects') == 1:
            self.GreasySkin(mat, p, n000, nv01, False)
                       

        # Yellow Custard Cover

        if p('02_effects') == 2:
            self.YellowCustard(mat, p, n000, nv01, False)



        # Muddy

        if p('02_effects') == 3:
            self.Mud(mat, p, n000, nv01, False)
        
                

    def Renderer(self):
        # Return the render engine this plugin is designed to work with
        # Must be 'Firefly' or 'Superfly'
        return 'Superfly'

