# Snarlygribbly's EZSkin v3
# Copyright 2020 Royston Harwood

# Import libraries
import poser

import wx, wx.adv, wx.aui, wx.lib.colourdb
import wx.lib.agw.floatspin as FS
import wx.lib.filebrowsebutton as FB
import wx.lib.colourdb
from wx.lib.embeddedimage import PyEmbeddedImage

import os, os.path, webbrowser
import configparser
from ast import literal_eval
from collections import OrderedDict
import imp
import glob
import sys
import re

# Fixes
poser.kNodeTypeCodeCYCLESSURFACE = 'CyclesSurface'

# Make WX colours available
wx.StockGDI._initStockObjects()

# Debugging
DEBUG = False
VERBOSE = False

# Global constants
BUILD = 91
APPNAME = "EZSkin 3.2"
APPTITLE = "%s by Snarlygribbly (build %s)" % (APPNAME, BUILD)
APPCAPTION = APPNAME
APPSIZE = (350, 525)
VALID_ROOMS = ['Pose', 'Material']
POSER_VERSION = int(float(poser.Version()))
POSER_BUILD = int(poser.AppVersion().rsplit('.', 1)[1])
EXPERIMENTAL = False

EZBOOL, EZBUTTON, EZINT, EZCOLOUR, EZCOLOURP, EZFLOAT, EZCHOICE, EZSELECT, EZISELECT, EZFILE, EZLABEL, EZTEXT, EZPRESETS, EZDIR = list(range(14))
SKINTYPE, EYETYPE, NAILTYPE, MOUTHTYPE, TEETHTYPE, OTHERTYPE = list(range(6))
METHOD, SHADERTYPE = list(range(2))

SSSTYPES = ("Apple", "Chicken 1", "Chicken 2", "Cream", "Ketchup", "Marble", "Potato", "Skimmilk", "Skin 1", "Skin 2", "Whole milk")

# Utility functions
def GenID(initialValue=0):
    v = initialValue
    while v < 1000:
        yield "%03d" % v
        v += 1
    return
def GetText(msg, title='Text entry', default=''):
    dlg = wx.TextEntryDialog(None, msg, title, default, style=wx.OK|wx.CANCEL)
    if dlg.ShowModal() == wx.ID_OK:
        result = dlg.GetValue()
    else:
        result = None
    dlg.Destroy()
    return result
def ShowAlert(message, title=None):
    if title == None: title = "%s has encountered an error" % APPNAME
    msg = wx.MessageDialog(None, message, caption=title, style=wx.OK | wx.STAY_ON_TOP)
    msg.ShowModal()
def ListDictionary(ctl, keycol=0, valcol=1):
    d = dict()
    for idx in range(ctl.GetItemCount()):
        d[ctl.GetItemText(idx, keycol)] = (ctl.GetItemText(idx, valcol), ctl.GetItemText(idx, valcol + 1))
    return d

# ICONS
class EZIcons(object):
    def __init__(self):
        self.apply_png = PyEmbeddedImage(
            "iVBORw0KGgoAAAANSUhEUgAAACgAAAAoCAYAAACM/rhtAAAABHNCSVQICAgIfAhkiAAACw5J"
            "REFUWIWtmXtw01UWxz/5JWmaPtK09AHhTYFaKrCILI2sFWgLKrCAA6gsFkSUVVhr+QOBcWHL"
            "zLKzw4wVXEQQx1bQyj4Qoay2UJCHLUVQoJRaBCr2AfSdtmneyf7xyy/Jry1VVs9MZnLPPfec"
            "b86995xzTxT8cooFhgGDgGhABTiBRqAWqAaafgU790UPAm8AX3qNe/r4NHrl1gFj7teQ4j7l"
            "HwNeA54EgiRmsFpLZEgMEdoo1MogHC47bZZmWs1N2JyWwPVW4HPgLeDUrwlwMPBX4DmJMSRq"
            "JCmjHsc4Io342CSiQw1oVFoEQYHb7cHmtNDUWc+1hsuUVR/n1LUvqGm9IS33AHuBDUDdLwU4"
            "F3gHMACMH5TM849k8djoOUSFanG7weEGlxs8ngDFClAKoFaCoIDmzi5OXjvMB6U5XK4tk8Rq"
            "gVeAw/8vwNeAHIDIkBjWpG1hwcTlaNUCFocI6ueSoIDIUDDbYN/ZXbx1/A3aupoA3F47b/e2"
            "TtmHzvXAVoAJg6ewa8kh0sakY3cqsDm7eQuvtwRQKUGp8DIDZNQqOFf9DZ3WLhY+nM7v4hdw"
            "seZr7nbUKoAnADNQ8nMBLpN+0bSEubz7h4P01/XHbJPZRK2EkCBxO02WTm6313PHdJsOmxlB"
            "oSJME4RWDYIAFoeN5XlpfFj6Js1mGymj0nn64Re4Un+ZW81VADOAG8DlQCC9bfFDwBlAOyV+"
            "JruWfEaQSoPd6RdQCRCshptNNRyt/A+nrn3O9YYKTJYWrE4rwapgIkL6MTI2iZSRTzB/wrMc"
            "KT9IdsFKn4443WCy57xDWuIMlrw/i7PVx/B68XfAxXsB1Hjd/NCwfgl88uJXRIf1w+rwC4QE"
            "QYu5nV2n/8b+r3fRbm2VKVCrVTgcThkvKjQOh8tKh9Uk47+z+L/MHf8Et1oaeXq3Ubrl54BH"
            "AXtvALOAN5WCivczipk6OoVOm38yTAMXay7x+oGlVN29BED//v2ZM2cO06dPJz4+Hp1Oh8lk"
            "4ubNmxw9epSioiJqa2u77xJpD8xn13MHsNghXAPFVSdY8WE6bo8LYDWwoztAPXAVGLBo4kq2"
            "LniXDqt/MjQIzt86z8qPZtFibkCj0bB27VpWrVpFXFxcDwASlZeXk5ycTFdXl1+XJpz8F84y"
            "xjDGtzthwbDmnys48O37ADWI2apdCND1LDAgVBPOi49ukJ05jQqqm2tYlT+PFnMDAwcOpKio"
            "iM2bN/cJDmD//v0ycABadSg6bZQsEjic8OKjGwgJCgUxMSwCkAAqgAyAmWMWMjpuCDYvQIXX"
            "x385/DINHXVERUVx5MgRUlJS+gQG8MMPP7Bt27Ye/KbOO2wueBkUfv02JyT2H0HaA09JYksD"
            "ASYAEwHmjMuQBeCQICi8WsCp748AsHPnTsaPH8+FCxfYunVrnwC3bNlCZ2enjDdx4kRUKhXH"
            "qw5yvLIQrdo/5/bA78dnSMNJQLwEcAqg7q8bzNiBk/zeAxwu+KhM9MKTTz7JokWL2Lt3L6mp"
            "qaxdu5Z9+/b1Cq68vJy8vDwZLzo6mkOHDpGamgrAvnPbcLv9F8HmhHGDkokNHwhiRJkiAXwY"
            "YMyAh4gMDcHt9aBaBTcaq7lw6zQKhYLly5eTmZlJRkYGJpMYMlavXk1VVVUPgNnZ2djtdhkv"
            "KysLg8HAypViPPy6+kuqm2tQqcR5lxv6hYaR2P83PodLAEcCjIx9EJXgzxZqJVypK8PusmEw"
            "GDAajZw8eVJm1GQysXTpUmw2fzwqKSnh008/lckNGTKE1atXAzBt2jSiY6KxOi1cqTtHUEA+"
            "UwkQH5skDUcJiB6OBRgQMVh2s5QKuNlUKUqOGoXBYCA/P5+IiAiZ8bKyMtavX+8bb9y4Ebdb"
            "XkmsX78enU4HgF6vZ0yiWLvebKpECAx2CjDoh0mjOAGx8NQCRGj7yZS6PdDS1QiAwWAAIDEx"
            "ke3bt9OdcnJyKCoq4tSpUxQXF8vmEhMTWbZsmYwXExMDgMnaKnMKQLBKK32Vdr93EouSnuk6"
            "IyOD06dPs2fPHhl/+fLlPi8F0qZNmwgODu7LVA+7EgmIOa8LkOozv6ACokJjAaivr5fN5eTk"
            "MG7cOBmvrq6OyspKGW/y5MksXLiwB4iGhgYA9Np+vlgoUZfDLH11CIh34i7AbdOPMmGXB+Jj"
            "xLNSUVHhu7kAYWFh5ObmEhIS0sN4IGVnZyMIgozX3NxMRUUFAPExiciOqwfq2qql0V1p5fcA"
            "3zdewenyyzqcMNbwWzQqLY2NjRw/flxmaMKECeTk5NwTXHp6OjNnzuzBP3bsGC0tLWjVoSQZ"
            "JmEPtOmG6w0V0vCaBPA8wNX6CzSbO1F6uQ4XjIgeyuTh0wB6TVsvvfQSixcv7sEXBIHs7Oxe"
            "gUt6kkekMqzfIJ9TVAI0dXbw3R1fOXheAngGsDd01HOp9izB3vTjQayGl0x+DYCTJ0/ywQcf"
            "9DC4Y8cOEhISZLwFCxZgNBp7yO7evZvS0lIAMpKzEBT+uKtRw8War2jqvANgAb6SAN4AygAO"
            "XcqT3SKLA6Y/kE564gIAMjMzKSmRPx30ej25ubloNBrRkEbDxo0be4A7c+YMa9asAeCJpGdI"
            "GTUVizzZcPCizwGlwA8SQA+QB3Cs8gBXb1/3e9EjxsONs99moH44HR0dzJ07l8LCQpni5ORk"
            "tmzZInomI4OkpCTZfEFBAfPmzcNsNjMkahR/nrVdfKp654PVUFF/jRNVvhdoHsgfTdeBxU63"
            "Q2+2dzBr7DxfTej05siJQ6dyouowjW13yc/Pp62tjcTERF9mMRqNtLa2sm7dOh+vurqaTZs2"
            "kZWVRVdXFwMihrJryWGGRw/DGlCUBKshuyCTyjvfgLijrwKO7lH4j8BOQSHw7pJCZo5Jo93q"
            "D5yhQVB19xqvH1jGpVrxHOn1embMmEFqaiqjR48mIiKC9vZ2qqqqKC4uprCw0BeeJgyZwt/n"
            "5zIydiRmu/9lGhEMBeVHeOXj2RKOFcD7EvhAUiH2TIwG/TA+WVHCQP0Auhx+ZSFqMNus5Ja+"
            "xb6yt2noqOenKDbcQEZyJkuNmWiDNFgcfsNaNdS01vHMe0butNcAnARSAVdvAAGSEA9o+MSh"
            "KexZUkC4Nlz2slMKItA6UxPFlQc5ff0Lqu6W02puxO60olFpiQyNZnTcWKaOns3UhNkYIqJl"
            "3QiPF1yHtZPn82ZysaYEoA0wAt9Jtu7V+lgEfAIoJg9P5R/P/IvosEjM3W6c9D52eaDD4sBk"
            "acHhshOk0qDTRhIerEapAKtDPMeBFKqBhvZmXt3/NGXVxRLmBcCBQLl7dRYqvL/m8bq2ak5f"
            "L2L8ICNDo+NwBjSJ3B6wu8DpApVSiU4bRoQ2gjBNKIJCicMlzrsDSzgBdMFw4da3rMqfz+Xa"
            "s9LUSuCj7kD66s2UAfXA402dd5SHLn+MgmCSDBPQaVW43XLDHi9g6dOtgkIliF4z2yzsOp3D"
            "hoPPS2fOBryE91J0p5/TfksFdgMjABLixpNhfJW0B54iTqcHxJQoxTSPR6yCfA0lpbcaaW+h"
            "6Oq/+bB0O9cb/bkWMXKcuJfxn9vAjAWygRcANYg385H4dIwj0kiIG0dM+CC06hAUCgGPx43F"
            "0UVDRw1Vdy9RcuMopTeKaey8LemzA+8Bm4Dmvgzfbwt4EvAnxKamrzIVFEoitJHotJEoFUpc"
            "HhftllZMlhbcHtntMAEHge3AN/dp+75oOGL/pACxTeGi9wa6C7gFfAasQvw34L7ofj3YG+kQ"
            "/4IYCEiPGg/i1tUDPwKdvS/9afofRnESsb27stwAAAAASUVORK5CYII=")
        self.close_png = PyEmbeddedImage(
            "iVBORw0KGgoAAAANSUhEUgAAACgAAAAoCAYAAACM/rhtAAAABHNCSVQICAgIfAhkiAAACORJ"
            "REFUWIW9mHtwVdUVh7/zvOc+AiQg2PHBQ9TptOMDAg2Qmwsa0NZxOrajnU47TDuFhBBSIRgT"
            "wETkobxp6zgGBWwFHbGdzhSqpUDVQAJRQaQ6dapTp0DqCCOVPLiP89r9Y98bb2IS7qW1a+b8"
            "c85Z+3z7t/ZZe+2lkL8FgJGAfwW+KnABSOXqoOT5gRFANRAF7Dx9AQygFXgKuHgF/kNaAbAR"
            "EIAwtIDQVSPny9ACIuObHqcgl4/mqmAEeAyoBbjrG/cz6bopJBw359kFDZ33P3mPV957EV94"
            "AFuAR4Geofz0HMYOAyszcA9MrmDNd59k9DCThJPbDAUQ0KEzDkXhUfzm2FbS4wmgCYgP5qtd"
            "ZmwLWA0sBbh/cgWr7t1C2AxyrhuSDsTty18JG7oSEDAgemM5nYluTnW0A0xH/nRHgAHDMRSg"
            "CTyegZtbsoTGe9YTCYS5GAdVvczU+pmiyAlZpsqMiXfQnYrz7tljGcgg8Abg5QqoZ8P9YMpC"
            "Vt67kQIreEVw2ZC2BwWWRsn4mXQl45zq6IW0gNfpl74GAtSA9cBDIJVb/u21XD0sRHcSIha4"
            "vryUPJKUL0DTYIQFtg8jwxqTro/RlUhkIGcAIeA15NocEFBBpoBaQPnJ9KU03L2aSCDCi2/9"
            "jh1tW7hwqYvJY29BQaqh5gDpC/mTDA/CS2/vYWfbk3zWE+eWa25j6vhSLqWSGcgSZPo5lIHM"
            "BlSADcBiQJ1bUkvdnMeIBML87ZOPeXDPfZw828qJ00dQ1RDRiVPxBTje0Er6AkwdikKws+05"
            "Nh2o4/jpFt765xvEbvoe1464muKx09OQ7QowNRsyAxhMw9UA+txptTw0p4mgOQzbhYvxC2xv"
            "WwdA0olz8swxNHUYZROn4InBleyFC8P21ufYdLCBC5fO9Y7z42/VELFGYhoBpowr4VIqxamO"
            "dhWYgty1WjKA9wCNQOSn0+qond1I2BxO0pFCjwiGCVuFtP3jAAC2l+SdM0dRlQjRG6WS/SF9"
            "AaYGRRF49shONh9Yxr/j53uf183ZxMyb5uD7Gq4HQdOieFwJSdvj3Y6jGnAj8EEGcBpw99eG"
            "Xx98/L6nGBW5moQjHwhAVXVuv+5WLH04xz4+1AdSUwuITpyKyAp3r3IRqVx/uNrZG5hfWo2i"
            "WPjp38H1ocCymDBqAoc+2EdX8mISOJLZSVzAN3ULQwvSfwfzfDC1EPOiNQBsPlQPQE+qk62H"
            "VgBQFatGAHEHLB0KQ7D9iAxrNtzS8g3Mm7EIVQli98t6jgu6ZhHQgyDTjdtnqxPCx/MHrqJs"
            "D0w9yM9Ka/CBrVmQWw4uRwhBZWwRugqWAdtbd7LxQD0X459lwa1nXrQaTQ1iD7KNe76PL75g"
            "yCvl2i7oapCK0hpqy9f13r9kd7H54DJ2tG6jMAy739rNhv0P94WbvZ550RpUJTQo3ECWS7HQ"
            "FzKtZEX0QRRFZfPBhwFIOD00t6zl8Id/4sPz79OZvNDr81D5BuaVLkJRv7x8/ueAACkXLN2i"
            "ouxBELD1Lw34wufTrrOc6+5ACLnyVUVlyZ3rmF/2cxQCeSmXsSvaVRUg6YKCycKZS6m7axNK"
            "eqgMHEDdnC1UxWpRlACpL5UBXyFgxnwBmuozMjyagGH1eaapOuFAGFURvank/wYoAEMDQ3PZ"
            "/eYuHvnDfJJO35rT812a9lay42gzquJgalkVwFcNaOqgqy57jr/AY39ciO0mABgZHs2ksWUU"
            "hUen3/RZ+2oNu9qfRVFszCtY8XkDGhpoistvj++mae8CHC8JQNCMUBldxt6FLVREGwia4V6f"
            "1a9Us6t9O4qwMS5Xw/83gBLO46W3d9G4rwI3DRcyC1hY9giVscVoCiyMLaEq1kjIjPSBfL79"
            "GRQcjDyU7AOoKhq6OvAUdR00VfDyiV2s3LcA13PScBEWxFaweHY9vg/nuuS+WlteT1XZCoJZ"
            "kGsy4cYbFFJXVdQshgygCai2l8AX0jl7QWsq6Irg5RPP8+i+KlxfntmDRoSq2CPUltfjuNCd"
            "BF2DniTYDtTObqAqtgLLyAr3q9XsOrYNFQ+tX/wMDXzhk3QSIGtVM4M6Dvh+T6o7YKgRJo+d"
            "RsDQcX05AwWH3598gZX7vlhzlhGmOtbEktkSrif5xVlFUWQy1zSITizF9TVOnj2K60vVD3+0"
            "n6LwGL55zW0IoSGAoAG2m+LZtl/R8ve9CIQLvJQBPAeMAVF8/HQLvlCYOm4GAV1DCDj7+Wkq"
            "d38H25VwAT3IopmPsqT8YWyvL1zGFAVSjrxfdkMprq/zzpk2PN8FBK0f7eeeW+ZSGCoioIPr"
            "2fzy9bU0t6xCyPj9GtiRAUwij31FQPHx04fxhELJhDIsU+XTzgvsOf40nu9iGSGqZzbJsA4C"
            "lw1pO3KJzJhYiudrvHv2GK7vYOoB7p9cybWFI3E9l1+8tprmltUZ12agDujM/iNsoCUb0vGh"
            "ZPxMRkeKKAqPJRyI8EDxfCqii/HE0HB9lPRAV2Hq+FKGWWMosAr54ZQaZt08C08IthxaSXPL"
            "mozL00A96ZbIQMedAuT5ZAHAgrJGGu5eRdCE890OBQEDx4OEk9uJLmO+kHWiqUG37TI6rBO3"
            "Yf2fm2g+3Ee5eqArc2OgnGIjW2QjgOITp1tJOB7Tb5hFOKDRnQDHzw8OpJKuL30LgyqOB+v2"
            "N7Ht8FrSOaMZaMiGGwwQZIMxDSmKT545SsJxmT5hFqoqQ5YvIEgVQ4bEWbe/iWdan0DITtc2"
            "YBnQ2d9nqI0nBRwFRgj84lMd7cRth9jNs9DTaSQfRl9AJCAP8E/sb2R720Y8mXaageUM0tC8"
            "3M6YTEMW+sKb/NeON+lJpSj/+h1cVSBzVySQ21UUkmFe/UojO9o24XopkMotBz4fDCBXEUYB"
            "a4BKSw9x760/4vbrJ5FIOTm6g2UYnPrXO+w79QIJWZptA1Yge9aDWj5RugpYBSxQFJVIoABf"
            "5F7hKShcsrsR8sTWjGwUfDa0V/5N9DHIxtJd5NGpzzITOAhsAs5f5l0gf0CQLbICrqxAVoBu"
            "hmj59rf/AKwAmU7Q+JDxAAAAAElFTkSuQmCC")
        self.configure_png = PyEmbeddedImage(
            "iVBORw0KGgoAAAANSUhEUgAAACgAAAAoCAYAAACM/rhtAAAABHNCSVQICAgIfAhkiAAACU9J"
            "REFUWIW1mHtwVNUdxz/37t7sOyEhmweQEEKIkUeCGAVLFApYqWgVi9RX1do6VCu2WqXFlgLS"
            "ispUHUcpY5HSdqyDhcQaqEbwQSqgxkJAIm9JQt4km+wm2ffd2z/ObrK7BFij/c6cmTvnnHvO"
            "9/zO+T11/H9gBAqAMYAe8APB4SykG8Y/ZmAKUAycOs+cbODPwONAKVAIWBAku4exZ0JQgJnA"
            "C0AD0AWkAUnA2DDp6cAM4HagHtDiWjWwDCHZhCAlOO8S4N5wywr3qQgpeYCJ4f4RgBxeNwtx"
            "vfEIAt8FdiVK8mJYCHzEudIYbtuCeKMJYagTxuMHiKsdEinGVHJGTsBuzcKkWJBlPYGgj15v"
            "Dy2uRhq6jqJpMb+MB+4BNgO+i22eyBXnAjuAydGdJWNmMCN/DtNyryRvZAFpliyMihlJ0qOq"
            "Pvr9TtqcjRxt+4LPGvax51QV7a6myO9+4K/ASqD16xKch3hreQA5aQXcVno/1028gfyMiZgU"
            "UFUIhCCkgaaBLIFOBiVsI5xuHweb9lJeu4Vt+zehhgKRtauAnyIUSodQRO9XITgf2IDQUmYX"
            "3sjSOcuZlnsVehncfgioDF5h9GrhPkkCowJmBVzeft46VM7zu1bQ6myIzNwBrALuA0zAj4FQ"
            "ZPBCdrAYeAkoArip5F5W3riOidmT8QTA7RMSi5CQJMFvoEmCo14HliRIUsBuTeLKvBLG26dT"
            "0/ARPe4uEDayDLgemAqcBA4lIsF/AosA5hbdytO3vIjdmkXfRZ+1gAYkG4Qoas8c4ETH5wTV"
            "IOPSCykrKGPvqU9YuuX7dLia43/9EmFPOyFWgqOAFQj7NRv4CWCckFHMs7dsYPSIsfT7EyMH"
            "kGKCRkcTqyof5tU9z1FVt423695A0Vm5LGcmV+SNR9HZ2XNyJ8FQzMKpCCewB2IleAeDhleH"
            "MLq8sLicm0oW4gkMXumFoAFWA7Q6m1hZ+Sg7v9gKaCg6I0tm/ZbbSu8kzZyDXtZhUPw88I+7"
            "eefwVkKaGr3MCaAE8MhRnd9G+NmREXJzihYyu3DWgIYmAkUWSrPjUAXvH60ANMamFfLS7a/z"
            "2LzHyErOQ0OHJwBGfRL3lz1Bsiktfpl8hIMgQrAIoRQxWDj1XqzGVIJq/MjQ0ABFD529HdQ0"
            "VKOGRADjV33YrdmYkgy4/SBp4gm0urxcllPM9LzZyFKMvnYBmdEELw2zHkCBfQpTRk8CpISl"
            "hyYkKEt+er19A93trjNsqP4jjY52MmyQYoHXPt3Equ0P4Q86WDBlMSbFHJn+JkJJXokmmIe4"
            "2gFMy51JmiWdUIiEYVJADYV498guDrd8MtAf0kK8d7SCLTV/p9XZx+rty3ly+0M43V1oSEzL"
            "LcOcZItMT0VES/0w6IvHD8pAKM44exEGvQ01AelJkjDEba4Ont+1gvLavxBUAzFz1FCQP1Wv"
            "YdOedbi8PQRDfq4puB69nILdJmO3ZXO2rwXETaYAPdEEI6HPt4DLANIsdnSSjHoRCSo60OuC"
            "VB+vZm3VLznWdhCNoU/V73PRjwtJkrl6wg1cN3kBIKPoIN2ahSRJaJqWgvAozmiCb4bb7yIE"
            "LYZkkDjPVkJqig76fQ42793Ay7tXE1RjDaUsySQbRwDgCbgB0MsKV0+Yz/L5z5FhG4U3ADYj"
            "2IwpSMhoqGbgZsQ1H4oPtwb4BIJ+0MR9x5PUy6CTgxxpPcLaqsfYd+rdcw5g0JuYVXg9v5j7"
            "JCFNY3/jx2iEmJR9BSU5U0EDb4ABIfiDvshOemA9wgndE0/QhYjRDOIBa0jSoBjlsNR8QSfl"
            "tRWsq1pGt/vsOeQyk0fzwxkPs+SaxzEpEoEgXJ47CRCk3IHBAEMGAkHodncRig0cvcDZeILt"
            "YZL2ZmcDAdWLTjYRRJgPnQ6+PHucV6qfYduBTecQ08sKxWOms+y6p5lzyUycHnD0gSQz6MOl"
            "WPel6KG7v5921xni7koFmuMJ1gEtgP3z5hr6fE5SzSZkCSTZz64j7/DyB3/gcMun55BLNdu5"
            "qeQuHp23msxkGx29Yjs5YsiGCEs0TZimTztq6fF0xQ/3AA1DEWwGSg417aOhq4mR1iysCrz2"
            "8WuUH/gbS65eSmN3K+s/XEO/vxeAidmX88CsZSy6fDHeALS7BLFEomFFBx8cq8QbVqIId+Aw"
            "0BtNsAgRk40DcPt72XW0kqm5U9FJetbv/j0Pzl7FQ/PuosMJb3y2EbVX5dpLb+aReaspHlPA"
            "2V4RwMrRHv480DQwJ0FLt4vdx3YQiLUAErATYpOmRcCa6FkVta9yc8mtlOZNZlx6ETvrtlGU"
            "NYaTHW2YFBOPzn2K+8p+jikJWnuE6ZETTWQl4Y9f+c9GGhwnIr3vAacR+U8lxMaDfYiIZsDl"
            "9ft66fP5uXbiAi7JnMI7dRV8Vl9Dc08DD8z+FXdOvwN/CFyer0AMUDXIsEJt0wme+vcjOIQl"
            "8AN3IlKMakRkHUOwFeFmphP1fI61H8BmGMXi0gWUFXyPnNR8bpl2N6W5xTjcwkR8FXIhDdIt"
            "0ONx8MS/fkZt077I0GZEPKoBjkhnfE5yGpgLZER3HjizF5sxlzlFV1KUlY9OstLjGcw9EiUm"
            "SWC3gcvrZGXlcrYfej0yfARYwhB1m3iCXQhDPQdRc1EB2Rf0UFNfjRoyU2AvxWKUCKiISEe6"
            "sLZqmhCJOUmQO9lRz5odv2Hr/o2RKY3Ag8D+of4fKqs7gkjWcxG++RRwhTfg5pPTH9DsbCQj"
            "eTRjR2ZjMwLaYD4cbWYlSbhEmxFGWoUHqTy4lbVvr+C9o+XR+y0FKs53wKEIBoGDCI16HfFg"
            "ZWCmGgpKX7Tu57/1+6h3NKCTDKRaskkz60g2gcUg8hGrQXxLErT0tFFVV8nLu59l894XOdVZ"
            "F7+fP7yfI34AEq9umYDFwK8J58kAuWkF5KTlU5g5mVEpuaSa0lH0Cm5fL539HZzuPMGXnceo"
            "7zpOj7vzQut/h7Dd+7qYgrCV3cRVrYyKRRthStfSLJmazThCU3RJiVS6WoAfIaLobwz5iOtI"
            "tNx2HFH0fD+u/y3gqottlkj5LR6LOf+JTwIbEbm1AxF0tiKipEWIEsdxYB2icNQ2jP0vikLg"
            "EeDDMJFoqdQABoY+eCaiYjGBxN9+4hPjkAQkIwz6JESJOB84BjwzzDWHxHAJRkNG1PUUhGH3"
            "fANrDuB/RaxoIIBxnUAAAAAASUVORK5CYII=")
        self.import_png = PyEmbeddedImage(
            "iVBORw0KGgoAAAANSUhEUgAAACgAAAAoCAYAAACM/rhtAAAABHNCSVQICAgIfAhkiAAABtVJ"
            "REFUWIXFmG1MVNkdxn+XNxURZgAHVu26ooCyvCXdxbSSLopZjUQjH9xtqkGzlphdEY1UbLLo"
            "atME2GBioSkubbUffCluTBSEVAnbNmqaSNIY0FXLmkYh3VTAlzAjzjDO0w/DzA4wA+Lg9klO"
            "cueec577O/97zzn/MxC8CoHHAcrBYM3DgjUAZgCmAHWzgzUPCdYAsE5QNxSsebCAbwEfTlC/"
            "EsgO8hmvpDnAYeARoEmKHTgGLPi+4N4HbgJavHixCgsLZRiGX7i8vDwtX77c8/s/wEevE2wG"
            "8DngCg0NVWlpqR4/fqzW1taA0Tt8+LAcDocqKys1Z84cz/3TgGW64ZKBvwNatmyZ2tvb5VFj"
            "Y2NAwP3793vbdXZ2atWqVZ66u8BPpgtuHfAtoG3btunRo0fy1csCSpLT6VRFRYXnkxgCPg4W"
            "bg8wHBERodraWvnTVAA9On/+vCwWi6ddLRA6VbAQ4DeAEhMT1dbW5vdBnocFAqyoqAjY7+bN"
            "m8rKyvK0vQDE+AMx/NyLBP4EbMrMzKSxsZGlS5d6K6urq7ly5Qrh4eEYhkFvby8dHR1+R5mc"
            "nExGRgYulwuXy8XChQupqakhIiICgIGBAYqKimhtbQX4B/AB0DtR5ExAG6DVq1err69v3Miv"
            "X78ecFmZrJw5c2acn91u1/bt2z1tvgZSA8HNHxmFNm3apGfPngV8PceOHZsyXGlpaUA/SSov"
            "L/e0/TcBdp8vDcNQcXGxhoeHJzSTpMLCwpeGy8jIkNVqndSzoqLC0+cOMGss4F+jo6MnjJyv"
            "+vr6lJqaOimc2WxWZ2fnpH4ul0ubN2/29PsWiIbRyYJzcHCQffv2BfoERiksLIzk5ORJ20XH"
            "RGO32ydtV15ezqlTp7wsI6Cj1DZyU0VFRXI6nQFH29HRoSXJSwRoVnhUwOiZIuPd1yGouro6"
            "oF9DQ4MApaWlKSUlRUAP7oRkNKDJZFJmZqYAbdmyxS9kR0eH4uLjBOjnub/UV3u7tcTy9ji4"
            "OTNNaim5rYYtrYqf/YYAHThwYJxfe3u7wsLCFBMTo1u3bnm2Q/+AiYmJun//vvLy8ryQvhOm"
            "v79fSUlJAvTput+qp1LqqZJOfnRNYSHhowB/teGP6qmSHlRKl0r/pQUmd79z5855/bq7u5WQ"
            "kCBATU1NkuTJfvwDms1mDQ0N6cmTJ8rNzR0XybKyMgHa9qNfqKdKunVIuvmZG+KT9w554dak"
            "faB7v/6u/n6l9Ofi6wozwvXmmz+Q3W7X4OCgsrOzBejIkSNe6LGAozJqwzCw2WzExMTQ1NTE"
            "ihUrOHnyJMXFxfT29tLwhwYsUfMpWfkZzxygkc/Y5oCSlZ+SPu8dTLPiOFBQi0vf1Q8+hxWL"
            "3+XDdz/mwYMejh8/zs6dO7lx4wY7duxg7969Y4Pl1bhDk2G4dz+z2UxzczMbNmzgxIkTXL16"
            "lcGng2zN20dCdBRPfU4bL1wwMzyMQ+t/zyPbQ+aZErA5Ru+jdif8NOcTvvznF+zZswe73U5+"
            "fj61tbUB4cZFcKzMZjNNTU3k5ubS3d0NwI+XrMHlswAYBoQYboCsBdnkL32foWEINdx1voCp"
            "Cakkxadht9tJSUnh9OnT3n35lQB9IfPz8wE4fq2ap0PPiQiFiDA3iOd12p0wNOy+dskdwZnh"
            "bp/ZM+DqN1/xYOAbTCYTZ8+exWKZWmLdFhsbq4GBAb9rlc1m09q1awUoL2WD7hx2qmnn18p5"
            "a5UWx77tt2TMy9HvftaihzXSX0q7FRvpzgEvXLgQcE2ccBZPBChJVqtV69atE6B3Fr6nuMhE"
            "AbK8Ea+EeZZxhZFZXb7mc2XOdx+eampqAvoHDShJDofDs9rLCEX19fUaGhrS8+fPx5WLFy8q"
            "Lj7WC7pr164JvYMGdDgc2r17twAlJCTo0qVLkz6wq6tLGRkZArRx40b19/e/HsCHDx96X29W"
            "VpZu3749KZxHfX19KigoEKD09HR1dXVNL2BnZ6fS0tK8URh7snsZDQ8Pq7S0VIBiY2PV0tIy"
            "PYDNzc0ym80CVFZWphcvXkwZzld1dXXeb7Kuri44wKNHj3rN6uvrgwLzVUtLi+Li4rxHAd+s"
            "adJkwWazyel0qqSkRIDmzp2ry5cvTxucR11dXUpPTxeggoIC7+TJycmZON26d++e1q9fL0DZ"
            "2dlTmgxT1djJc+fOHU+q5x8wKipKixYtEqCtW7fK5XK9NjhfHTx4UIZhKDEx0ZMfegF9sxnD"
            "arVitVqxWCwkJSVRVVWF0+kcO5BpVUhICJGRkaSkpHD37l0vi7+2f+MVDuOvqfyXkQj6kv4Q"
            "iMfPaep7loH7n69rwIv/M8vk+h8naWteoXs+wQAAAABJRU5ErkJggg==")
        self.export_png = PyEmbeddedImage(
            "iVBORw0KGgoAAAANSUhEUgAAACgAAAAoCAYAAACM/rhtAAAABHNCSVQICAgIfAhkiAAAB59J"
            "REFUWIXFmH1MFOYdxz/3AtwhB2JdeSnK2xU20YEVTZ02g6ZCg/Hl2vlaRVAWL5amirNzDI2u"
            "Zl196WRpNDIQQZHFRW2YxZDIgpmeYGFTShSsstpqrLbz5Xg9Du63P8678XIH14rzm/xy97z9"
            "ns/zPL/n91xOxehJAZwCbEDTKPodNeUAAtwCgp4xywB5AQuATuyAAvwNiHiGTAD4AkbgX/wP"
            "rL+ZgaPA9GcBtwC47AZssFmBPwNh/w+wYKAUED8/P4mLixOFQuEWTqfTyeTJkx3lW8BbTxPu"
            "deAGIElJSVJVVSXx8fHD7p5arZatW7fKvn37RKfTOeoPAgGjDbcF6FOr1bJt2zYRETEajZ4c"
            "r2i1Wrl69apcuXJFZs+e7ai/BCSOBthY4K+AREVFSXV1tYiIlJeXewTnsEmTJonVahWLxSI5"
            "OTmO+kfAiieBiwH+CUhqaqrcvn1bRERu3LghgYGB3wsQEKPRKA6VlpaKv7+/o237D4GbDdwG"
            "JDs7W3p6ekREpKurS1JSUr43nMOOHz/uhKyrqxO9Xu9oKwZ8PIUzAG2A7Nq1S/qrq6tLamtr"
            "pb6+Xurr66WhoUHmzZvnFqigoEAaGhqkvr5e6urqpKWlZYC/mzdvyqxZsxz9K7GH1LDKAKxa"
            "rVbKysrEE2VlZbkFvHDhwojjzWazLFy40DHGhD2VDZECeBuQgIAAqaqq8ghORCQjI8MtYE1N"
            "jUc+rFarZGZmOsZdxsUzGQJ0hoSEiMlk8hhO5Ml3sL/S0tL6xyQAysefaqAvLi6OmTNnjhQG"
            "TnV0dHDnzh237Y2NjR77On/+PBcvXnQUNYPbwwCzUqmUvLw8j1ZbVlYmUVFRI97cpKQkqa2t"
            "HdZXa2urhIaGCOB4OstcAjqcDgfZ19cn69atE0B8fHwkNDTUJZhSqZQpU6Y4n7zCwkKX/sxm"
            "syQmJgogb0xdI/7aQPeAcSHTJHL8TwSQLVu2uHS4du1aASQ+Pl6ampqc5cGmUCiktrZWKioq"
            "nEm5uLh4iL9FixYJIMumZ8vfc26JxsvXPeDcKculZuOXEhYY7XIni4qKBJCEhAS5d++eiAx/"
            "i8+ePSsiIufOnROdTicajUYaGxud/nJzcwWQ6RHJcu19q1S92yo+ao17wFdj58udnSJV734h"
            "YWPt8bVx40bnUQQFBYmfn580Nzc7J/E0zZSUlAggc+fOFRH7cwfIhEC9nHvvjnz1gcin2c1D"
            "ANUDORW090BMkJ6SzDOsPbKAPXv2oNVqCQ0N5e7du+Tm5hIbG+sc0dnZOXixTvX29jq/p6en"
            "c+DAASorK8nPzyf3t7n4eunYu+QYL4wNpqvXPv9gqQdXKIAOC0SOj6RoVSVrStPYsWMHWq0W"
            "X19fMjMzB/RfunQpMTExKBRDnUdHRw8oG41GTCYT69evByB/yQkSw6fS1g3eQ0jcADohe2BC"
            "YBhF6ZVklaTxxbefo9fr0ev1A/oaDAYMBoNr74M0Y8YMlEolNpuNnNd2YkgwYO4efozSXUN/"
            "yMJVlbz4/BSuX7/uXL1DDx8+dGkPHjzAYrE4+4kI27dvx2az8cbULLKTN9FuYUS5BRwCudIO"
            "mZ+fz4YNG2hvb2fz5s28GKsnQj+RSH240yL04URETyQxcRomkwmAvC15lJeXMz08md/N/5ie"
            "XrDJyIBuTn4oZNg4O+S6owb27t3L6dOnaWlpIVATRORzMfT1m02Bgj6x0tRUT+rrqSxftpyC"
            "ggLCx8Xwx8VH0Xj50G0dGc4jQAdkZw/8OCSMnDkf8svDc2hpaWFm1Bx2vnmIYP9Qem0D+6tV"
            "8Mmlv/CbExkUFBTgrdLw0aJjRIwP5kEHuLhTLjXsETsnVECABj5tPEPuJ6uwiY0liUYKV54i"
            "JCDUeVwO6xPo6YXF05ZyeHUNL4yNpKevmyN1f+Lbtna03vZEOSqAKiWM8YaD5w+w5nAqd823"
            "2JSymw8M+1Epvem2Ps7KMtBsAuZueDnqZY6sruGlibM5eekgmSWpfHX/3+g8/IE/LKC3GhQI"
            "209tYtspI2O8dXy87CTvJG+k28qAY3WnNgtMGDeR4lVVzI9P59LXJpYX/hzTjXP4a1ylZg8B"
            "td7wqOshb5e/SbFpNxHPxXJ4dQ3zfroQc7dnNxDsAN1W0Hj58tGiEt5Jfp9vzF+Tceg1yj87"
            "gp8GVMNQugTUaeDa3WusPJhMdfNJfhadQtmaGhLCEmgbIbG6k7XPHpe/Sslj9y+OolAo+fWJ"
            "lXxYtQ21CrxU4CoyBwAqFAp0GjjTXM1bRa/Q/M0lliQaKVhRwfO6YNp7fhicQzaB9m5YPG0Z"
            "JZnVhASEs69mOxuOraDL2oXGawwyCHIAoErpRbGplDWlKdzvuMd7qXv4g2E/XiofLL32zoon"
            "NHBcnpmUZ/2Dlya+wqnPy0g/mMaX/2nBWzXw9jjGhAFXxvjodB2WNn7kF8LvDYeYH59CuwVs"
            "Ns/zlqcSAa0XtFtsbK1Yx7GGA/j5BGDp7cLa13OUx/+E9U/U6g5LGwA6zVgqLpdS/tl+u6en"
            "KC+Vmj4RNGot7ZZHjmqVE8qxIOAB0AvQ+t1VWr+7+lTBhpECaO9fADtxMPYwe7pbNrKU2AHv"
            "P2MOz/RfpWa6DVq71T0AAAAASUVORK5CYII=")
        self.info_png = PyEmbeddedImage(
            "iVBORw0KGgoAAAANSUhEUgAAACgAAAAoCAYAAACM/rhtAAAABHNCSVQICAgIfAhkiAAACNpJ"
            "REFUWIW9mHlwlPUZxz/vsZvdzbW7hGgOEgJLw9UgGEhMCVhUwINhprWtOrQzOL2G2opaSrXD"
            "2AtlBKvTY0qrONTa1lKPUqlyigpBolwBIRJjLpIAuXaTTfZ8j/7x2yUhbC7Qfmcy2Xnf9/e8"
            "3/e5n0fi6uAGMoBsoADIAVJj90zABzQDdcBFoB3ouZoXSWN83gMUA4uB+cCUUZw5A7wN7AZO"
            "AQ1jeeFoCeYDdwHfAuYBOKwpjE/NxuUYh9OeQZrNhSwraEaE7qAXX6ADX6CTNn8rYS0Yl/Me"
            "sAXYC7R+FgQVYAnwEHArQJ7bw4zsGykpKKcody55rkLS7OkkqSBJoBsQ0cAX6KCh62NOnvuQ"
            "yoaDnGk9Smt3Y1zuduB3wL5rIZgG/AD4KZCWlZbHlwuXsfyGb1CcX06aHTQdQlGQJUiyCGGG"
            "KQgaJtitoMrQ1QeV9bt4/cQ/2X92B95AOwjf/BXwHBAZTkOJ4I4dfhRIKi1YxNqlT/Cd+Q8y"
            "KTOfUBR6ghCIgCJDUItyuP4gx5qO4w36cTqyUCQJbwB6w2BVYGa2h0VTlzPBVcCFnouc725M"
            "Ae4AVKAC0EZLMBXYAKwCuHfuKh6/6xnmTpxDXwT8IdBNYU6rCt5AOxt3P8bm9zaw/cRfOfTp"
            "frqDPmZkf5EUm52oAZohzikK3Jg/neL8cjr7fJy9eBKgHLAi/HJUBB9H+BzfX7iOtUvW405x"
            "4evrJyYhNGeYAZ4/+FteqHiK3nA3uqHhC3bwYeO7OB3XM6+gFN0A0xRuENWhLwK5bjdlk26m"
            "zd/FmfPHAL4EeIHKkQjeAzwJWFaUrmbtkvVYLVZ6Q/3E4rAo0OY/zxNvPkxPyDtIjIlmmJR7"
            "7sDlcBDRYudjAvrC4LTbKfPMp76jidq2jwBuAg4C5wZKkgf8ngKsARzlnmU8dMs6LIpKIDx0"
            "JIU1Df8V5AQi0SCBqB9ZvvKeLIE3CGk2Fz9f9jQzsosBnMAzgH0ogt8DijJTc3n4tnVkpLgJ"
            "DhlbwmxuRzpFuTclvD9h3GSy0q8nFO3X3GCSviDkOrNYs/hpklQbwFzg/kQEi4A7AfW+eT+i"
            "KHsOwYioWYkgSyKNOJNdrLr5MbKdEy+7PzN7HveXrcZhsRFJGJsCEoLkoqkLWFa0In55Nf1l"
            "85L1fgk8WpAxXX1uxatMHj+V3sjQpk1OAlkWDqwqcLKlgX3V/6bNf578cR5um7acCa5M/KGh"
            "PzIOE3BY4ez5Gu7+czGBiD8KfBfYCiIHjQfKAHXJ9LuZ4PYQ0ROTkyXQjRBPvrWemotVyJKC"
            "IitY1SQiWhjNiNLUVUNV82G+Omcl5ZPn4w8PT1ACghGYmvUFFky5nZ2nt6nAvQMJzgEKbaqN"
            "koKFWFURGEN/sU5l/T6qmt+/7LoiKUiSjG5qWBU7pQULSbKMTHAg09tn3sPO09skYBaQCzTL"
            "wHQguzBrDgUZ+ZgmGCPIUmT1imu6qaMZUUzTRJZkZFnBHMm+AxDR4IbcMpz2cQDJQAmIIPEA"
            "8pTxRbgd49FHYgdo+jDhDUiShDTGTk7XISPVhSdzBgjL3hD/kQmQ7czFZk1FN4YODsMEWbKx"
            "dsmz+ELtWBUb3r5Wfv/Or2nqqh0TocEwAZsqMTGjkCON71mACXGCKQBORwaKJKGPKEph7sQS"
            "0cGo0NrTyIuH/whcG0EQms9MyQZh2Yw4QQlAkdSECTURIrp40CTeWg2T7MYCU8JhTSbGyRpn"
            "GsMonO//gIh2yb91EATDAL1hP8YYou5zgWTSFegAoa1uEAS7ANr9rUS0EOpYx6jPEJpm0Oqr"
            "A9G8XogTbADM+o4a/KEuFGXk8vR5QJbAH47ySdtpgChQHSf4MeA7c+Eord0tow6UzxpWBWrb"
            "qmnprgPhdpVxgseBunZ/K1XNpzAMAzVBD3c1MBm9NSwq7K/ZjiECoQX4KE6wBqgC2HPmFbyB"
            "TqxDjVIxyLF5xKqCRRE1+HJIqLJCkipypTLCB6sKtPv9vHnqZRDRu4PYEBU/+h+g81DdLg7X"
            "H0aWBYnhoBs6hpk4B4rxU8c0QTeimObwKSzVBq+feJFz3loQK5Kt8XtxXZ0DisGceqGnlUWF"
            "d5JmTyaaoKyoskn1xdO8UPEbDn36DhW1+6hqriQY6R3wlEkoGuJMay1vn/0v4WiYSRmTUAY1"
            "EKYJKTZo8bbxs+3fjs822xDbh8sIRhELn9vPdzc67NZ0SicvwCQ2yQ3QjFUxOfDJHjbt+THH"
            "mg5wquXIIHJCew2dNXzQsJ+q5vdJsji4ZdpSrIqFqNE/4NssIjh+seMRDtftBegAVgKdcVkD"
            "vWMX8BfA2FLxFG9U/YuU2GZgIEzAolivVO0wsFkcgIRJPzmrAul22FLxJ1499nz80Q2ImLiE"
            "we77NLC/L9zDhp0/YW/1Tpx2QXKgaYwRfGowdEODmCV0Q6xExiXD3yq38ezexzBMHeAV4A+D"
            "zyYKhVnAS8DMPPcU1ixez1dmf42IDr0hUGSTxq463j37BuooNBmMBpiVU8rsvJuQZYU0m5hn"
            "Xjy0lY171tLV1wZi9XEf0DQagiBWbFuAmRkpWXyz5AFWlj3IdenJdAeFFhxjsHI4KlKN0wEN"
            "nR1sfncT//hwM73hboADiDXLR4nODpXxWoBDgCcQ6Z10tOkgJ5uPkGRJpSCjEHeKCJ5QFEKa"
            "aLku+9O5lAFsFkizgz9k8Nrxv7Nx9zq2V71ERA8BvAY8Apwe6uNGKmwTgQeABwE1MzWHWbnz"
            "WDx9OSWTbiXPlYPdKoQMrBiyBIYh9jB17Z9QUbeLfdVvcbL5A3zBDoAgsAnYzAiLzNFU3hRg"
            "AfBDYClAut3NdWk55Lk8eMZPI8eVR7rDjSonEdVDdPW1c85bR21bNc3eei76W+gL++Py4sHw"
            "PrFW71oJxpGF8M2vx4i6QaQcm8WBKluQJRnDNIjoYcLRAJpxqcq0IcrXy8AJxFJ9VLia3sWN"
            "GLSKgdmIsTUXsZFV6E/65xCOfyT2vy12fUy4luZKRmyikgBLjFzcHXXEWjcMBK7hHfwP7vRT"
            "Ih0IvMAAAAAASUVORK5CYII=")
        self.menu_png = PyEmbeddedImage(
            "iVBORw0KGgoAAAANSUhEUgAAACgAAAAoCAYAAACM/rhtAAAABHNCSVQICAgIfAhkiAAABVVJ"
            "REFUWIXNmF1sFFUUx38zs7v9oNvvlu623bbQUr6pbVERoYZXH0RjJI2KYGzEBxNJfJAYEmOU"
            "EGKiaKoixSiEaCoRTbAJDypGVIQIjZgIhBQsVKiWfne3O7Mz14fZ3bA7M+1uJbT/5CY755y5"
            "93/Pvffc/44E5AM+5h7GgOsAbYAxB9sXAC5Aira5BglMggKgtraWjo6O2WUkSbS3t9PZ2UmM"
            "lyvmzM3NpaWlZba4xdHV1ZXwLMd+GIZx18nYQdO0hOc4QUVR7joZO3g8noTn+BL39PTQ2tp6"
            "1wklo7u722Jrw9yQc63FywwABdklPLzCzKD4/4mYETyKwqkrJ/jzxrm4LU7Qlxdg18a9CMAQ"
            "4JIhy2MtkIaAoAoiOgsBuGXItIlNBbph9geQkwGvHN1hT9AQOuNhc0BFghsTtzh4ai8D4zcT"
            "Olzmb+aJ5ucAGRGdSO9YP4dOvcNw8FbaBJurWnik4UmM6IQ1XU3wu2zeIcsNH//8Fh0nd1t8"
            "R87up7poEesXPURINWPbv3+Nzt8+TJtcrL/6spUsL19h65ftjLqAyoIFSDaL5s3MpyinhFjZ"
            "NATUFC+aETmA+d5y8rMKcCrDthkMqfB4UxuLy+5hQh2LExUIfHkBFhQvJBwxY4MqbL5/Ow2V"
            "awlpE7aTcoagsrAOf34Fmg4ZNmxsCQpAEtBU1YyUNJ5ugBpJipVgdfW9lthUENFB0539tgRj"
            "A+sGyEmD2i2FEKZ9JgSNaWqaI0G3AteH+giq4wn2Eq+PguzchFm7FPhrsJdJLZQ2wbLcCryZ"
            "84ikswezPHDs96O8+tVWJtSxBF9VYS37nz5OoLAaTYdsD3x+5hBvdL1AaAYE6+ev5MDm4xTl"
            "lNr6bU+xS4bTV08wHh5BCCOhXb11iZ6BS7ij2kKR4dcr3xJUJyyxqbQLN7v5e7gPly0Tp1Os"
            "wZY12xkLDTEcGkSObi5DCOpKl3NfzXomo6poUoO2dTuIGBpjkyPx2OkQ23qNlQ+y1LeCcARc"
            "HmucLUFNh0BhNW9vOmg5FIpskoptblWHutJ63t10eNoNbwdFNhMiHN61JShFSWq69X5N7meq"
            "2FQw3Zwc66Bbhky3vc8iFhTIdKwHU8MQ5sXgRNS2W7cM/aM3+ejHXQyHBhN8i8tWsWXNS8iy"
            "GyHM2GuDvXSc3M14eDRtgo2BtbSufh6H8+pcZg78tIfDp9+z+L45f5glviY21G8gqJqxH/zw"
            "Ol+eO5A2uVh/qyrW0FDZYOu3FwsGLPU14lEyLL4Sr5/y/Ip4YdUNWF7ejEu22Q8poKqwjhLv"
            "fPR0CnVQhY0NT7HU18RYeBgpOg+BwJ9fhT/PF7+Pgyq0rt5GY+U6JtTReGyqCBTVUpxTlJ5Y"
            "AIgYsMS3BFlK3MC6Yb3cDQOW+ZdZYlNBRDebE6acbuzfS7Jtqvg7DccMumS4/M9lQupEgr00"
            "10+ptyQhi7IMF/svEtYm0ybgywtQlFPgKLlsCWZ74MjZz9j59bOEI4mD+vICfPLMd9QUL0SN"
            "ioVPf9nHm10vEjE0u+6mRE1xPQe3nqDEW2brt11iRYI/+s5YyAHcGOmld+gKrtvEwvm+0zMi"
            "B3Bl4CL9o/3pi4WtD7xMODLJUPBfJEkGBAhY7GuwiIVtLTtRZBcjocFobAoQAiSJ5qr1pljQ"
            "wGWtag5iwQBfnp/dj75vEQCyBMEksVBVWM2ex/bNSCxIUopXnSzJZHuiJ/e2aDkpIUKYfzXt"
            "kBybCoQwbyOAeRngVhw+Ho1ODtN1/hgCMWufPrJcCj0DFyz2bcz+hyK7dhTMDI4C1+5YGu4c"
            "+sHUmApgI7ZnHTqg/gdpcos5Yxs2LAAAAABJRU5ErkJggg==")
        self.web_png = PyEmbeddedImage(
            "iVBORw0KGgoAAAANSUhEUgAAACgAAAAoCAYAAACM/rhtAAAABHNCSVQICAgIfAhkiAAACo5J"
            "REFUWIXdmGl0VdUVgL87vDnzy0AmQkIihISEKSbUCmgBCZRBrKVoitouQFexWm1L1TotrdSp"
            "FKUsWMqgFUEggMoM0ogoBJYYQgYIIYGQhMwJeZneePvj5oYXCBCs/ule66x37z7nnfPdffbZ"
            "Z58j8L/LACAWGAQEAn5AG9ACnAfKgaofYJxbkiHAn4H/AI2AcoPSAnwNvACk/NhgtwObgM6b"
            "QF2vuIDPgfE/NFggsBxw3ghALxsUg2xU9LKxP7DrgPCbDSz0Ay4DWA0Mu7oiLjiRi83ncLod"
            "PJv5LhOGTMLpFkFRcLhaeDdnCQdPb8cgG3G6HXgUz9VdnAcWAPuvN7h0E7jZQDYQ5a28M2Eq"
            "r89+j6jA2zh4ejuzR/6Wv059FV9TMEEWK1YfK9FBkaQPmszOgg0oCLwyYzVGnZ6S2gLvrgKA"
            "OUA18N2tAs4CPgFMmiI2eChLZq1h8ZSXsRhCWbRxFn7GQJbP3YYsmXC4wO1Ri8MFVh8Tob6D"
            "2Za3hgCzlZVZ60iJnMCZ2gLq2y55M8wEKvuCvB5gGvCpBieJMplJc1j+qy2MGDgCEViy5xmO"
            "lO3nhWkrGRuXRqfz2k6cLhgWMYT8yu/YWfAxowdOZdKwO7gn6UGaO5opqc3Ho7i15lOBY8A5"
            "7z768sEAIBe4TVOE+kaw8/ESBvhZ6HBAVXMVme/GEx+aRPbCY3gUEY/S95cadXCqqoDZK1O4"
            "Mz6TNQ/tRBah0wnTlg+jrL7Yu/klYAzqlAMg9tHn37zhYoMTqbNVs2jDDCqaagj2gc0n3sPu"
            "6mLBnc9h1F0fDqDLCSOjk7ln2P0cOruLMzVnabO38/jG+yirLyYuuNfaCweWeiuunuJRwEoN"
            "PHHASDYvPIRe8mHrd2vYU5BNoDma1V+/RZAlhOcyl+H2SNyADwBZglDfWLblraW65TwfHFnO"
            "0fIvyEp/ghUPrOdExXEqW8q15knAYdQd6BrAd4BktVMdy36ZzW1h8WTEjSPcP449hVv4LP8j"
            "bF0tTE/JYnrqNGQJRAEErQCiCJIIehlMelAUqG2tZ3/xFoounaClo5HFU5by1MQXMeoNDA1L"
            "Z1veWlyeHkeOBj64GjCh27wywIzUecy/83e02cHlhlExqUweNodj5Tk0tteSX5VLTskBmtqb"
            "URQRUZARBAlBELG7HFzubKK0vpjdBVt5a9+z/OPAYjqd7QC8MnMtC8fPx2YHuwsGBgXT3GHj"
            "RMVhjWUQsAuolr0A7wWMADpJz0Njn8bVHVcVoMMOMUGDURSFcP9o0mPvZl9RNicqvlZ9RZQI"
            "Modi0JlwuLpo7qjH6VYtYpBNzEjJYljEGP6+50kqm8twuFTLAnS5YG7aIjYcX0G7vRXUiZgL"
            "HPcGnKY9jIy+g+SIZOxeoUOWoLSulJK6U9w/egH/mruKs3Vvc7Iyl/yqI5TVn6bOVk27vRWr"
            "JZSUyHRig4eQEjWW1Kh0BlnDuNypsCLnZb4p24fd+SJC98c7XBAXEsm4+KnsLtyoDTkZkDXA"
            "EGC4VnPX0JnoZdX8mugkKK1Td4GkiDF0OsFqsTIlaSrThk/F7QGnWw3SogA6GWQRPB5wuMFm"
            "B6NeYGj4CAqqjtPUYcPf5Iu7e5YE4K6hM7wBE4BYLczEoyYEAKRGZeDsiZ+qiAKUNagxa3DI"
            "MFxucHmgwwG2LvXX5VEt4lags1vf7lDBFQV0IiSEDqfD0UZdayWyV5BzuCE5Ig2dpNdUemCo"
            "ZsFoTetvCiIuOBFZBIOXA+gkqGq5AAhEBsQiS2C4WXy5Sow6iA9JAqClsxGz/kqdJEJ04GDC"
            "/KKobC7T1LEaQrCmMet9qGgqpd5mwO0FYJQFLjSWoJN0VLaU0eFo7llE/RWzXqDdbgPgVFUu"
            "A/yC6HCqgwiAKAgYZKP3X0K0h0V8vwT0xy6vaRbssVWA2cr0lCxEQewJAwCSKPB5/sfY7Je5"
            "f9R8JFHqVd8fkSWRi03l7C/eSlrMeFKi0nC6rzi7KAjsLtxEbWulpnJrgI2axsfgzzNT/omv"
            "kV4rzKCDgupvOVtXyPPTlmHWX6nvr1gMsPPUIfYXb+XekY+wYNxDtHVpcKqVcs/neAM2aIAX"
            "NU29rZrK5gsMssbg8FrJLg/4GALocNioa2sgxBJ8yz4IUGdTExWDzqyucruqlyVobGvyhgO4"
            "oC30c0ArgN3Vxdm6U+iu2qVFEUJ8BuBw2bF1XUbqIw8Sui1xvXOEIEBjWw0AgabgXlmQToSK"
            "plKa2us0lRM4rQ1TC/Tk4kfK9iNeBSAKEBkQq1qhtQpJVEOPRQ++RjUp0MnqNOlkMOnAx6BO"
            "q/axigIVzaWAmmN6u4gsQW75F95DlgHnvBfJbuAnAAfPfEpD2xLMenNPJ24PxIUkAlDRdBZf"
            "4ziKL9WQX5lLXmUu5+oLaWirpcPRhklvxmoJIy44kdSodFKjM4gJikQS4UzNSXyNAYT5RV3x"
            "cQHa7Qp7Czd7Ax4AnN578VbUw7WuuuUC+4u3MzftAWzdTuzyQFLEaIw6E7tObSDv4lfsPPUJ"
            "dpfawCAb8TX6YzH4UdvaxNnaAr4s2QGoicTEobP4eco8ztUXkRQ+hgCzpWcrNethb+Feimt6"
            "HUk2Qndq1S1FwB5gOsD7h18jM2k2etmIRwGjDK2dzZh0Fo52T0V67N1MSpxNSlQGEf4x+Bj8"
            "kCQdHo8LW1crta0Xya86yhent7O3KJu9RdkAJIaPxM8IDe2qvzpcCqsOveoNdwT4Bq5NWM8D"
            "jwBCU3s9kqhnYuJ4RBE+PPo+T2yaTZuaDvHmfRt4afqbjI65nTC/CIw6M6IoIQoCkihhMZiJ"
            "CBjAmJgxzEzNIjN5DkXVedS0XuR0TR4Cvtw+KB1fI6w+vJJN36705ngMKOkLsBL1ImgEQN7F"
            "bxgSNpZteet5fe9TRAclMC/jD+SWHyQhdDjjEibQ2qVOv0fpvQV4FFXvcKnvVp8QPjzyDqIo"
            "Ee43kO0n19HS0YpeDuAv236N0+3QGHYAL2svfUWEEOA4EAPqkdPtcTFhyAzenL2OQEsgd70d"
            "j93VyZ7fl2LSm24asC16OHB6P/P/PZnHxr/Akz9bzNNbFrAjfz16yYDDbdeaNqIeeXsOKH2d"
            "6uqBhwEHgNvjQhIlHkhbRIw1EL0Ec8Y8Rp2tmn1FWzHrbgynJaUfHlmKKIhMH55FqK+Zh8f+"
            "EZPO4g2nAAu94a4HCJADzNde3B43T23+BStyVuFW4MH0R/E1+rPqq1ex2Z19Bm1NTHo4Wn6M"
            "r0p3MyNlHqNiEnj/8Mcs+OienjNKtzyNes3SS2509XESuABkArLDbSenZAffXsglOSKVYJ9o"
            "dhVsIMQ3ioy40b2y757ORTXGLc6eR0NbDVkZT7Ai5w2W57zkDad0wy29tof+3W5NAt6j2ycB"
            "REEiNSqD/KpcAkxBbHn0BPEhkb18UUHdOdbnruX5z36DXjIgCEJP3OyWBuBR+rDcrQACRABv"
            "AA/2VRkdGE9y5GislgEYdSYcLjsNbTVUtpyjsPoEbk8f5lXvfv4EnO0nQ79kImow9/D9k9Av"
            "UW+zflQZA/wd9brsZtfBDtREZBnw01sdqL9TfD0RUQP7EFQftQL+gA01plWg7ghlqOnT/5/8"
            "F1JFRAPmWlJkAAAAAElFTkSuQmCC")

        self.catalog = {
            'info' : self.info_png,
            'web' : self.web_png,
            'configure' : self.configure_png,
            'menu' : self.menu_png,
            'export' : self.export_png,
            'import' : self.import_png,
            'close' : self.close_png,
            'apply' : self.apply_png,
            }

    def GetBitmap(self, id):
        return self.catalog[id].GetBitmap()

# COLOUR
class Colour(object):
    def __init__(self, col=(255,255,255), poser=False):
        self.r, self.g, self.b = self.ConvertColour(col, adj=False)
        if poser:
            self.r *= 255
            self.g *= 255
            self.b *= 255

    def GetColour(self, ctl):
        wxcol = ctl.GetColour()
        return self.ConvertColour(wxcol)

    def ConvertColour(self, wxcol, adj=True):
        if isinstance(wxcol, wx.Colour):
            r, g, b = wxcol.Get(includeAlpha=False)
        elif isinstance(wxcol, tuple):
            r, g, b = wxcol
        else:
            try:
                col = wx.Colour(0,0,0)
                col.SetFromName(wxcol)
                r, g, b = col.Get()
            except:
                tmp = wxcol[3:]
                r, g, b = eval(tmp)
        if adj:
            return (r/255.0, g/255.0, b/255.0)
        else:
            return (r,g,b)

    def RGBToPoserCol(self, rgb):
        return tuple([x/255.0 for x in rgb])

    def HTML(self):
        return self.RGBToHTMLColor(self.RGB())

    def HTMLColorToRGB(self, colorstring):
        """ convert #RRGGBB to an (R, G, B) tuple """
        colorstring = colorstring.strip()
        if colorstring[0] == '#': colorstring = colorstring[1:]
        if len(colorstring) != 6:
            raise ValueError("input #%s is not in #RRGGBB format" % colorstring)
        r, g, b = colorstring[:2], colorstring[2:4], colorstring[4:]
        r, g, b = [int(n, 16) for n in (r, g, b)]
        return (r, g, b)

    def POSERCOL(self):
        return self.RGBToPoserCol(self.RGB())

    def PoserColToHTML(self, col_tuple):
        rgb = tuple([int(x*255) for x in col_tuple])
        return self.RGBToHTMLColor(rgb)

    def PoserColToRGB(self, col_tuple):
        rgb = tuple([int(x*255) for x in col_tuple])
        return rgb

    def SetColour(self, rgb):
        self.r, self.g, self.b = self.ConvertColour(rgb, adj = False)

    def RGB(self):
        return (self.r, self.g, self.b)

    def RGBToHTMLColor(self, rgb_tuple):
        hexcolor = '#%02x%02x%02x' % rgb_tuple
        return hexcolor

# EZCONTROLS
class EZControl(object):
    def __init__(self, ui, parent, parms, width ):

        LABEL, TYPE, RANGE, INIT, TIP, INCREMENT = list(range(6))

        # Object parameters
        self.ui = ui
        self.parent = parent
        self.label = parms[LABEL]
        self.range = parms[RANGE]
        self.initValue = parms[INIT]
        self.tooltip = parms[TIP]
        self.labelwidth = width
        self.font = ui.GetFont()
        if parms[INCREMENT]:
            self.increment = parms[INCREMENT]
        else:
            self.increment = 1
        self.ProcessRange()
        self.BuildControl()

    def ProcessRange(self):
        try:
            self.minValue = self.range[0]
            self.maxValue = self.range[1]
        except:
            self.minValue = None
            self.maxValue = None

    def AddPresetCheckbox(self):
        self.save = wx.CheckBox( self.parent, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
        self.save.SetValue(True)
        self.sizer.Add( self.save, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

    def AddToSizer(self):
        self.sizer.Add( self.control, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

    def BindEvents(self):
        pass

    def BuildControl(self):

        # Sizer
        self.sizer = wx.BoxSizer(wx.HORIZONTAL)

        # Add checkbox for saving presets
        #self.AddPresetCheckbox()

        # The control's label
        self.ctllabel = wx.StaticText( self.parent, wx.ID_ANY, self.label, wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_RIGHT )
        self.ctllabel.SetMinSize( wx.Size( self.labelwidth,-1))
        self.sizer.Add( self.ctllabel, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        # The main control
        try:
            self.control = self.CreateMainControl()
            if self.control:
                self.control.SetToolTip(self.tooltip)
                self.AddToSizer()
                self.BindEvents()
        except:
            if DEBUG:
                print('label:', self.label)
                print('range:', self.range)
                print('initvalue:', self.initValue, '\n')
            self.control = None
            self.ui.output('Unable to build control %s' % self.label)

    def Increment(self):
        return self.category

    def ConsumeKey( self, event ):
        pass

    def CreateMainControl(self):
        # wxTextCtrl is the base control
        ctl = wx.TextCtrl( self.parent, wx.ID_ANY, str(self.initValue), wx.DefaultPosition, wx.DefaultSize, 0 )
        return ctl

    def Height(self):
        return self.control.GetSize()[1]

    def Max(self):
        return self.maxValue

    def Min(self):
        return self.minValue

    def Reset(self):
        if not self.initValue is None:
            self.SetValue(self.initValue)

    def Save(self):
        if self.save is None:
            return False
        else:
            return self.save.IsChecked()

    def SetValue(self, value):
        self.control.SetValue(str(value))

    def SetValueString(self, value):
        self.SetValue(value)

    def Sizer(self):
        return self.sizer

    def Successful(self):
        return (True, False)[self.control is None]

    def UpdateSave(self, category):
        if self.save:
            self.save.SetValue(category == 'All')

    def Value(self):
        return self.control.GetValue()

    def ValueString(self):
        return self.Value()
class EZCtlBool(EZControl):
    def CreateMainControl(self):
        ctl = wx.CheckBox( self.parent, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
        ctl.SetValue(self.initValue)
        return ctl

    def SetValue(self, value):
        if isinstance(value, str):
            self.control.SetValue((False, True)[value=='True'])
        else:
            self.control.SetValue((False, True)[value != False])

    def ValueString(self):
        return ('False', 'True')[self.Value()]
class EZCtlButton(EZControl):
    def AddPresetCheckbox(self):
        self.save = None
        self.sizer.AddSpacer(25)

    def CreateMainControl(self):
        # wxTextCtrl is the base control
        ctl = wx.Button( self.parent, wx.ID_ANY, self.label, wx.DefaultPosition, wx.DefaultSize, 0 )
        self.ctllabel.SetLabel('')
        ctl.Bind(wx.EVT_BUTTON, self.initValue)
        return ctl

    def SetValue(self, value):
        pass

    def SetValueString(self, value):
        pass

    def Value(self):
        return self.initValue

    def ValueString(self):
        return None
class EZCtlInt(EZControl):
    def CreateMainControl(self):
        ctl = wx.SpinCtrl( self.parent, wx.ID_ANY, "", wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, self.minValue, self.maxValue, self.initValue )
        ctl.Bind( wx.EVT_KEY_DOWN, self.ConsumeKey )
        ctl.Bind( wx.EVT_TEXT, self.ConsumeKey )
        return ctl
    def SetValue(self, value):
        self.control.SetValue(int(value))
class EZCtlColour(EZControl):
    def CreateMainControl(self):
        initValue = tuple([int(x) for x in self.initValue])
        ctl = wx.ColourPickerCtrl( self.parent, wx.ID_ANY, self.initValue, wx.DefaultPosition, wx.DefaultSize, wx.CLRP_DEFAULT_STYLE )
        return ctl

    def SetValue(self, col):
        colour = Colour(col).RGB()
        self.control.SetColour(colour)

    def SetValueString(self, col):
        self.SetValue(col)

    def Value(self):
        col = Colour(self.control.GetColour())
        return col.RGB()

    def ValueString(self):
        col = self.control.GetColour()
        if isinstance(col, wx.Colour): col = col.GetAsString()
        return col
class EZCtlColourP(EZCtlColour):
    def CreateMainControl(self):
        initValue = tuple([int(x*255) for x in self.initValue])
        ctl = wx.ColourPickerCtrl( self.parent, wx.ID_ANY, initValue, wx.DefaultPosition, wx.DefaultSize, wx.CLRP_DEFAULT_STYLE )
        return ctl

    def SetValue(self, col):
        colour = Colour(col, poser=True).RGB()
        self.control.SetColour(colour)

    def SetValueString(self, col):
        colour = Colour(col).RGB()
        self.control.SetColour(colour)

    def Value(self):
        col = Colour(self.control.GetColour())
        return col.POSERCOL()
class EZCtlFloat(EZControl):
    def CreateMainControl(self):
        ctl = FS.FloatSpin(self.parent, wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize, style=0, value=self.initValue, min_val=self.minValue, max_val=self.maxValue, increment=self.increment, digits=3)
        ctl.GetTextCtrl().SetToolTip(self.tooltip)
        return ctl

    def SetValue(self, value):
        self.control.SetValue(float(value))
class EZCtlChoice(EZControl):
    def CreateMainControl(self):
        ctl = wx.Choice( self.parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, self.range, 0 )
        ctl.SetSelection( 0 )
        return ctl

    def Reset(self):
        self.control.SetSelection(0)

    def SetValue(self, value):
        self.control.SetSelection(value)

    def SetValueString(self, value):
        self.control.SetSelection(int(value))

    def Value(self):
        return self.control.GetSelection()
class EZCtlSelect(EZControl):
    def CreateMainControl(self):
        ctl = wx.Choice( self.parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, self.range, 0 )
        start = ctl.FindString(self.initValue)
        ctl.SetSelection( max(0, start ))
        return ctl

    def ProcessRange(self):
        self.minValue = None
        self.maxValue = None

    def Reset(self):
        start = self.control.FindString(self.initValue)
        self.control.SetSelection( max(0, start ))

    def SetValue(self, value):
        start = self.control.FindString(value)
        self.control.SetSelection( max(0, start ))

    def SetValueString(self, value):
        self.SetValue(value)

    def Value(self):
        return self.control.GetString(self.control.GetSelection())
class EZCtlISelect(EZCtlSelect):

    def CreateMainControl(self):
        ctl = wx.Choice( self.parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, self.range, 0 )
        ctl.SetSelection(self.initValue)
        return ctl

    def Reset(self):
        start = self.control.SetSelection(self.initValue)

    def SetValue(self, value):
        self.control.SetSelection(value)

    def Value(self):
        return self.control.GetSelection()

    def SetValueString(self, value):
        self.control.SetSelection( int(value))
class EZCtlFile(EZControl):
    def CreateMainControl(self):

        try:
            start = ('', self.initValue)[os.path.exists(self.initValue)]
        except:
            start = ''

        ctl = FB.FileBrowseButton( self.parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, labelText='', startDirectory=start)

        return ctl

    def AddToSizer(self):
        self.sizer.Add( self.control, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

    def SetValue(self, value):
        self.control.SetValue(value)

    def Value(self):
        return self.control.GetValue()
class EZCtlDir(EZCtlFile):
    def CreateMainControl(self):

        try:
            start = ('', self.initValue)[os.path.exists(self.initValue)]
        except:
            start = ''

        ctl = FB.DirBrowseButton( self.parent, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, labelText='', startDirectory=start)
        ctl.SetValue(start)

        return ctl
class EZCtlLabel(EZControl):
    def AddPresetCheckbox(self):
        self.save = None

    def BuildControl(self):

        # Sizer
        self.sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.AddPresetCheckbox()

        # The control's label
        self.ctllabel = wx.StaticText( self.parent, wx.ID_ANY, self.label.upper(), wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_LEFT )
        self.ctllabel.SetMinSize( wx.Size( -1,-1 ) )
        font = self.font
        font.SetWeight(wx.FONTWEIGHT_BOLD)
        self.ctllabel.SetFont(font)
        self.sizer.Add( self.ctllabel, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.control = None

    def Height(self):
        return self.ctllabel.GetSize()[1]

    def SetValue(self, value):
        self.ctllabel.SetLabel(str(value))

    def Successful(self):
        return (True, False)[self.ctllabel is None]

    def Value(self):
        self.ctllabel.GetLabel()
class EZCtlText (EZControl):
    def AddToSizer(self):
        self.sizer.Add( self.control, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL|wx.EXPAND, 5 )
class EZCtlPresets(EZCtlSelect):
    def AddPresetCheckbox(self):
        self.save = None
        self.sizer.AddSpacer(25)

    def BindEvents(self):
        self.control.Bind( wx.EVT_CHOICE, self.Update )

    def Update(self, event):
        self.ui.Filter(self.Value())

# EZCTLCFG
class EZCtlCfg(object):
    def __init__(self, name, label, kind, minmax, default, increment=None, tooltip='', category=None):
        self.name = name
        self.label = label
        self.kind = kind
        self.minmax = minmax
        self.default = default
        self.increment = increment
        self.tooltip = tooltip
        self.category = category
        self.value = self.default

    def AsTuple(self):
        return (self.label, self.kind, self.minmax, self.value, self.tooltip, self.increment)

    def Category(self):
        return self.category

    def Default(self):
        return self.default

    def Kind(self):
        return self.kind

    def MinMax(self):
        return self.minmax

    def Name(self):
        return self.name

    def SetValue(self, value):
        self.value = value

        # Check for out of range value in list type controls
        if self.kind in (EZCHOICE, EZISELECT):
            num_items = len(self.MinMax()) - 1
            if value > num_items:
                self.value = 0

    def Value(self):
        return self.value

# EZDIALOG
class DlgUI ( wx.Dialog ):

    def __init__( self, parent ):
        wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = wx.EmptyString, pos = wx.DefaultPosition, size = wx.Size( 400,400 ), style = wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER )

        self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )

        box_Dialog = wx.BoxSizer( wx.VERTICAL )

        self.pnl_User = wx.ScrolledWindow( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.HSCROLL|wx.VSCROLL )
        self.pnl_User.SetScrollRate( 5, 5 )
        box_Dialog.Add( self.pnl_User, 1, wx.EXPAND |wx.ALL, 5 )

        self.m_staticline6 = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
        box_Dialog.Add( self.m_staticline6, 0, wx.EXPAND |wx.ALL, 5 )

        self.pnl_Controls = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
        box_Controls = wx.BoxSizer( wx.HORIZONTAL )

        self.cmd_Defaults = wx.Button( self.pnl_Controls, wx.ID_ANY, "Defaults", wx.DefaultPosition, wx.DefaultSize, 0 )
        box_Controls.Add( self.cmd_Defaults, 0, wx.ALL, 5 )

        self.cmd_Reset = wx.Button( self.pnl_Controls, wx.ID_ANY, "Reset", wx.DefaultPosition, wx.DefaultSize, 0 )
        box_Controls.Add( self.cmd_Reset, 0, wx.ALL, 5 )

        self.cmd_Cancel = wx.Button( self.pnl_Controls, wx.ID_CANCEL, "Cancel", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.cmd_Cancel.SetDefault()
        box_Controls.Add( self.cmd_Cancel, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.cmd_OK = wx.Button( self.pnl_Controls, wx.ID_OK, "OK", wx.DefaultPosition, wx.DefaultSize, 0 )
        box_Controls.Add( self.cmd_OK, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        self.pnl_Controls.SetSizer( box_Controls )
        self.pnl_Controls.Layout()
        box_Controls.Fit( self.pnl_Controls )
        box_Dialog.Add( self.pnl_Controls, 0, wx.EXPAND |wx.ALL, 5 )


        self.SetSizer( box_Dialog )
        self.Layout()

        self.Centre( wx.BOTH )

        # Connect Events
        self.cmd_Defaults.Bind( wx.EVT_BUTTON, self.Defaults )
        self.cmd_Reset.Bind( wx.EVT_BUTTON, self.Reset )
        self.cmd_Cancel.Bind( wx.EVT_BUTTON, self.Cancel )
        self.cmd_OK.Bind( wx.EVT_BUTTON, self.OK )

    def __del__( self ):
        pass


    # Virtual event handlers, overide them in your derived class
    def Defaults( self, event ):
        event.Skip()

    def Reset( self, event ):
        event.Skip()

    def Cancel( self, event ):
        event.Skip()

    def OK( self, event ):
        event.Skip()
class EZDialog(DlgUI):

    def __init__(self, title="EZDialog", data=None, output=None, pos=None, size=None):
        DlgUI.__init__(self, None)

        if pos: self.SetPosition(pos)
        if size: self.SetSize(size)

        self.title = title
        self.data = data
        self.output = output or self.Output
        self.controls = {}
        self.categories = []
        self.labelwidth = 150
        self.panel = self.pnl_User
        self.sizer = wx.BoxSizer( wx.VERTICAL )
        self.pnl_User.SetSizer(self.sizer)

        # Map control type to classes
        self.ControlTypes = (
            EZCtlBool,
            EZCtlButton,
            EZCtlInt,
            EZCtlColour,
            EZCtlColourP,
            EZCtlFloat,
            EZCtlChoice,
            EZCtlSelect,
            EZCtlISelect,
            EZCtlFile,
            EZCtlLabel,
            EZCtlText,
            EZCtlPresets,
            EZCtlDir
            )

        self.SetTitle(self.title)

        if data: self.BuildControls(data)

    def BuildControls(self, data, filter=None):

        try:
            controls = {}

            # Remove all previous controls
            self.panel.DestroyChildren()
            self.panel.Hide()

            # Variables used in calculating the scoll area height
            height = 0
            margin = 5

            # Get list of control categories
            catlist = []
            for key, row in list(data.items()):
                # Get the category
                try:
                    cat = row.Category()
                except:
                    cat = None

                # Add to the list if it's not already in it
                if cat and not cat in catlist: catlist.append(cat)

            # And keep the list
            self.categories = sorted(catlist)

            # Add a category selector if needed
            if len(self.categories) > 0:

                # Prepare a list of choices
                lst = self.categories
                lst.insert(0, 'All')

                # Add the control
                ctl = EZCtlPresets(self, self.panel, ('Show category', 'select', lst, filter or 'All', 'Select the category of controls to be saved', None), self.labelwidth)
                self.sizer.Add(ctl.Sizer(), 0, wx.EXPAND, margin)
                height += (ctl.Height() + margin * 2)

                # Add a horizontal divider
                ctl = wx.StaticLine( self.panel, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
                self.sizer.Add( ctl, 0, wx.EXPAND, 5 )
                height += (ctl.GetSize()[1])

            # Add the controls to the UI
            for key, row in list(data.items()):

                # Check that the control should be shown
                if filter is None or filter.lower() == 'all' or row.Category() == filter:

                    # Get the control type
                    ctlType = self.ControlTypes[row.Kind()]

                    if ctlType:

                        # Create the control
                        ctl = ctlType(self, self.panel, row.AsTuple(), self.labelwidth)

                        if ctl.Successful():

                            # Add control to UI
                            self.sizer.Add(ctl.Sizer(), 0, wx.EXPAND, margin)
                            height += (ctl.Height() + margin * 2)

                            # Add control to our dictionary
                            controls[row.Name()] = ctl

            # Update the layout
            self.panel.Show()

            # Adjust the scroll bars to suit the new layout
            self.panel.SetScrollbars( 10, 10, 0, height / 10)

            # Keep a record of all controls created
            self.controls = controls


        except:
            # Remove all previous controls
            self.panel.DestroyChildren()
            self.output('Error building controls')

    def Cancel( self, event ):
        self.EndModal(0)

    def Defaults(self, event):
        for name, ezctl in list(self.controls.items()):
            ctl = self.data.get(name, None)
            if ctl:
                value = ctl.Default()
                if not value is None: ezctl.SetValue(value)

    def Filter(self, val):

        # Update all data values
        for k, v in list(self.data.items()):
            ctl = self.controls.get(k, None)
            if ctl:
                v.SetValue(ctl.Value())

        # Reinitialise
        self.controls = {}

        # Rebuild the UI
        self.BuildControls(self.data, filter=val)

    def GetLabelWidth(self):
        return self.labelwidth

    def OK(self, event):
        self.EndModal(1)

    def Output(self, msg):
        ShowAlert(msg)

    def Position(self):
        return self.GetPosition()

    def Reset(self, event):
        for ctl in list(self.controls.values()):
            ctl.Reset()

    def SetLabelWidth(self, width):
        self.labelwidth = width

    def Size(self):
        return self.GetSize()

    def Values(self):
        return {x:y.Value() for x, y in list(self.controls.items())}

# CONFIGMANAGER
class ConfigManager(object):
    def __init__(self, path=None, defaults=None, default_section=None, listctl=None, msgfunc=None):

        # Parameters
        self.file = path
        self.defaults = defaults or {}
        self.default_section = default_section or 'DEFAULT'
        self.list = listctl
        self.message = msgfunc or self.Message

        # Set up the Parser
        self.config = configparser.RawConfigParser(defaults)
        self.config.optionxform = str

        # Read data from disk fle
        if self.file: self.ReadFile()

        # Create the default section if it does not exist
        if default_section and not self.config.has_section(default_section):
            self.config.add_section(default_section)

        # Reset the defaults
        if self.defaults:
            for k, v in list(self.defaults.items()):
                self.config.set('DEFAULT', k, v)

    def ConfigFile(self):
        return self.config

    def Dump(self):
        print('-- DEFAULTS --')
        for k, v in list(self.config.defaults().items()):
            print('\t%s: %s' % (k,v))

        for sec in self.config.sections():
            print('SECTION: %s' % sec)
            for opt in self.config.items(sec):
                print('\t%s: %s' % opt)

    def Get(self, section, item):
        if self.config.has_section(section):
            if self.config.has_option(section, item):
                value = self.config.get(section, item)
                if isinstance(value, str):
                    if value.lower().startswith('<bound'):
                        return None
                    elif value == '':
                        return value
                    else:
                        try:
                            return literal_eval(value)
                        except:
                            return value
                else:
                    return value
            else:
                if DEBUG == True: print('Missing configuration option %s in section %s' % (item, section))
                return None
        else:
            if DEBUG == True: print('Missing configuration section %s' % section)
            return None

    def GetD(self, item):
        return self.Get(self.default_section, item)

    def Items(self, section):
        if self.config.has_section(section):
            return [(k, self.Get(section, k)) for k, v in self.config.items(section)]
        else:
            return []

    def Message(self, msg):
        # Display a message to the user
        wx.MessageDialog(None, msg, caption='Configuration Manager', style=wx.OK|wx.STAY_ON_TOP).ShowModal()

    def Options(self, section):
        return self.config.options(section)

    def ReadFile(self):

        # Start from scratch, deleting all existing settings
        self.Reset()

        # Get the data from the file
        self.config.read(self.file)

        # Update the list control, if there is one
        if self.list:
            self.list.Clear()
            self.list.AppendItems(self.config.sections())

    def Reset(self):
        # Remove all existing settings
        presets = self.config.sections()
        for p in presets:
            self.config.remove_section(p)

    def Sections(self):
        return self.config.sections()

    def Set(self, section, item, value):
        if not self.config.has_section(section): self.config.add_section(section)
        self.config.set(section, item, value)

    def SetD(self, item, value):
        self.Set(self.default_section, item, value)

    def SetDMany(self, data, first_time_only=False):

        # Check if we should only write values the first time
        if first_time_only and len(self.config.options(self.default_section)) > 0:
            return

        # Add the settings in data to the default section
        for k, v in list(data.items()):
            self.SetD(k, v)

    def SetMany(self, section, data):

        # Ensure the section exists
        if not self.config.has_section(section): self.config.add_section(section)

        # Add the settings in data to the section
        for k, v in list(data.items()):
            self.config.set(section, k, v)

    def SetFile(self, path):
        self.file = path
        self.ReadFile()

    def SetListControl(self, ctl):
        self.list = ctl

    def WriteFile(self):
        try:
            cfgfile = open(self.file, 'w')
            self.config.write(cfgfile)
            cfgfile.close()
        except:
            self.Message('Unable to open configuration file for writing')

# PACKAGEMANAGER
class PackageManager(object):

    def __init__(self, path=None):

        # Initialise
        self.path = None
        self.SetPath(path)

    def GetPath(self):
        return self.path

    def IsValidFolder(self, path):
        return os.path.isdir(path)

    def SetPath(self, path):
        if self.IsValidFolder(path): self.path = path

# PARSER
class EZParser(configparser.RawConfigParser):

    def __init__(self, defaults=None, dict_type=dict, allow_no_value=False, nocolon=True):
        configparser.RawConfigParser.__init__(self)

        if nocolon:

            # Create a new regular expression
            self.OPTCRE    = re.compile(
            r'(?P<option>[^=\s][^=]*)'
            r'\s*(?P<vi>[=])\s*'
            r'(?P<value>.*)$'
            )

            # Assign it for use
            self._optcre = self.OPTCRE

# LIB
class Lib(object):

    def __init__(self):

        # Conversion from EZSkin 2 format to EZSkin 3 format
        self.convert = {
            "cornea" : ('cornea', '9'),
            "eyelash" : ('eyelash', '3'),
            "eyetrans" : ('invisible', '1'),
            "gums" : ('gums', '3'),
            "hair" : ('hair', '4'),
            "ignore" : ('ignore', '1'),
            "innermouth" : ('innermouth', '5'),
            "invisible" : ('invisible', '6'),
            "iris" : ('iris', '7'),
            "lacrimal" : ('lacrimal', '8'),
            "lips" : ('lips', '1'),
            "maybe" : ('skin', '1'),
            "nails" : ('nails', '1'),
            "nostrils" : ('skin', '2'),
            "nostril" : ('skin', '2'),
            "pupil" : ('pupil', '10'),
            "sclera" : ('sclera', '11'),
            "sclerasimple" : ('sclera', '11'),
            "skin" : ('skin', '1'),
            "skintrans" : ('skin', '1'),
            "specular" : ('specular', '15'),
            "tear" : ('tear', '14'),
            "teeth" : ('teeth', '13'),
            "ericsteeth" : ('teeth', '13'),
            "tongue" : ('tongue', '12'),
        }

        # Read the figure type defnitions configuraton file
        # self.config = ConfigParser.RawConfigParser()
        self.config = EZParser()
        self.presetsfile = os.path.join(poser.PrefsLocation(), 'ezskin3', 'ezskin3fig' + os.extsep + 'defs')
        self.config.read(self.presetsfile)
        if not self.config.has_section('OBJLOOKUP'): self.config.add_section('OBJLOOKUP')

        # Convert string values to tuples
        for sec in self.config.sections():

            # The OBJ file values are not tuples, so ignore
            if sec == 'OBJLOOKUP': continue

            # Convert each value to a tuple
            for k, v in self.config.items(sec):
                try:
                    self.config.set(sec, k, eval(v))
                except:
                    pass

    def AddObj(self, obj, ftype):
        # Associates a geometry with a figure type

        if self.config.has_section(ftype):
            self.config.set('OBJLOOKUP', obj, ftype)
            return True
        else:
            return False

    def Dump(self):
        # Prints library contents to stdout for debugging

        print('-- DEFAULTS --')
        for k, v in list(self.config.defaults().items()):
            print('\t%s: %s' % (k,v))

        for sec in self.config.sections():
            print('SECTION: %s' % sec)
            for opt in self.config.items(sec):
                print('\t%s: %s' % opt)
                print(type(opt))
                print('\n')

    def GetAllTemplates(self):
        # Returns a list of all figure types

        lst = self.config.sections()
        try:
            lst.remove('OBJLOOKUP')
        except:
            pass
        return lst

    def GetFigureDefs(self, figs):
        # Returns a RawConfigParser object with a subset of the definitions for all types in 'figs'

        cfg = configparser.RawConfigParser()
        cfg.optionxform = str
        cfg.add_section('OBJLOOKUP')

        for  k, v in self.config.items('OBJLOOKUP'):
            if v in figs:
                if not cfg.has_section(v): cfg.add_section(v)
                for mat, shader in self.config.items(v):
                    cfg.set(v, mat, shader)
                cfg.set('OBJLOOKUP', k, v)

        return cfg

    def GetObjName(self, figure, prefix=None):
        # Returns the geometry name in lowercase without an extension

        if figure:
            file = figure.GeomFileName()
            if file:
                name = os.path.splitext(os.path.basename(file.lower()))[0]
                if prefix:
                    if name.startswith(prefix): name = prefix
                return name
            else:
                return None
        else:
            return None

    def GetTemplate(self, figure, material, figtype=None, prefix=None):
        # Returns the shader associated with a specific material of a Figure object

        if not figtype:
            obj = self.GetObjName(figure, prefix)
            figtype = self.GetType(obj)

        if self.config.has_section(figtype):
            try:
                shader = self.config.get(figtype, material)
            except:
                shader = None
        else:
            shader = None

        return shader

    def GetType(self, obj):
        # Uses the name of an object file to determine which figure type to work with

        if obj:
            try:
                result = self.config.get('OBJLOOKUP', obj)
            except:
                result = None
            finally:
                return result
        else:
            return None

    def GetTypeStr(self, obj):
        # Returns a string identifying the figure type, or '(Unsupported)'
        try:
            text = self.config.get('OBJLOOKUP', obj)
        except:
            text = None
        return text or '(Unsupported)'

    def ImportDefinitions(self, cfg, figs, legacy=False):
        # Appends a file of definitions to the library

        for  key, v in cfg.items('OBJLOOKUP'):

            if legacy == True:
                k = os.path.splitext(key)[0].lower()
            else:
                k = key

            if v in figs:
                if not self.config.has_section(v): self.config.add_section(v)
                for mat, shader in cfg.items(v):
                    if legacy:
                        shader = self.convert[shader]
                    else:
                        shader = eval(shader)
                    self.config.set(v, mat, shader)
                self.config.set('OBJLOOKUP', k, v)

    def Supports(self, figure, prefix=None):
        # Determines whether or not a figure is supported by EZSkin

        obj = self.GetObjName(figure, prefix)

        if obj:
            ftype = self.GetType(obj)
            if ftype:
                return True
            else:
                return False
        else:
            return False

    def UpdateType(self, figtype, data):
        # Updates a definition with new data

        if not self.config.has_section(figtype):
            self.config.add_section(figtype)

        for k, v in list(data.items()):
            self.config.set(figtype, k, v)

    def Write(self):
        # Writes the definitions to disk

        cfgfile = open(self.presetsfile, 'w')
        self.config.write(cfgfile)
        cfgfile.close()

# LISTSELECTOR
class ListSelector ( wx.Dialog ):

    def __init__( self, parent ):
        wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = "Preset editor", pos = wx.DefaultPosition, size = wx.Size( 257,432 ), style = wx.CAPTION|wx.DEFAULT_DIALOG_STYLE )

        self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )

        box_Configure = wx.BoxSizer( wx.VERTICAL )

        ctl_PresetListChoices = []
        self.ctl_PresetList = wx.CheckListBox( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, ctl_PresetListChoices, wx.LB_ALWAYS_SB|wx.LB_MULTIPLE|wx.LB_SORT )
        box_Configure.Add( self.ctl_PresetList, 1, wx.ALL|wx.EXPAND, 5 )

        box_Select = wx.BoxSizer( wx.HORIZONTAL )

        self.ctl_SelectAll = wx.Button( self, wx.ID_ANY, "Select All", wx.DefaultPosition, wx.DefaultSize, 0 )
        box_Select.Add( self.ctl_SelectAll, 0, wx.ALL, 5 )

        self.ctl_SelectNone = wx.Button( self, wx.ID_ANY, "Select None", wx.DefaultPosition, wx.DefaultSize, 0 )
        box_Select.Add( self.ctl_SelectNone, 0, wx.ALL, 5 )


        box_Configure.Add( box_Select, 0, wx.EXPAND, 5 )

        box_Buttons = wx.BoxSizer( wx.HORIZONTAL )

        self.ctl_SelectPresets = wx.Button( self, wx.ID_OK, "Select", wx.DefaultPosition, wx.DefaultSize, 0 )
        box_Buttons.Add( self.ctl_SelectPresets, 0, wx.ALL, 5 )

        self.ctl_Cancel = wx.Button( self, wx.ID_CANCEL, "Cancel", wx.DefaultPosition, wx.DefaultSize, 0 )
        box_Buttons.Add( self.ctl_Cancel, 0, wx.ALL, 5 )


        box_Configure.Add( box_Buttons, 0, wx.EXPAND, 5 )


        self.SetSizer( box_Configure )
        self.Layout()

        self.Centre( wx.BOTH )

        # Connect Events
        self.ctl_SelectAll.Bind( wx.EVT_BUTTON, self.SelectAll )
        self.ctl_SelectNone.Bind( wx.EVT_BUTTON, self.SelectNone )
        self.ctl_SelectPresets.Bind( wx.EVT_BUTTON, self.SelectPresets )
        self.ctl_Cancel.Bind( wx.EVT_BUTTON, self.Cancel )

    def __del__( self ):
        # Disconnect Events
        self.ctl_SelectAll.Unbind( wx.EVT_BUTTON, None )
        self.ctl_SelectNone.Unbind( wx.EVT_BUTTON, None )
        self.ctl_SelectPresets.Unbind( wx.EVT_BUTTON, None )
        self.ctl_Cancel.Unbind( wx.EVT_BUTTON, None )

    # Virtual event handlers, overide them in your derived class
    def SelectAll( self, event ):
        event.Skip()

    def SelectNone( self, event ):
        event.Skip()

    def SelectPresets( self, event ):
        event.Skip()

    def Cancel( self, event ):
        event.Skip()
class Selector(ListSelector):
    def __init__(self, names, title=None, buttontext=None):
        ListSelector.__init__(self, None)

        if title:
            self.SetTitle(title)

        if buttontext:
            self.ctl_SelectPresets.SetLabel(buttontext)

        for name in names:
            self.ctl_PresetList.Append(name)

    def __del__( self ):
        pass

    def GetSelected(self):
        return self.ctl_PresetList.GetCheckedStrings()

    def SelectAll( self, event ):
        self.ctl_PresetList.SetCheckedItems(list(range(self.ctl_PresetList.GetCount())))

    def SelectNone( self, event ):
        self.ctl_PresetList.SetCheckedItems(())

    def SelectPresets( self, event ):
        self.EndModal(1)

    def Cancel( self, event ):
        self.EndModal(0)

# EDITOR
class Editor ( wx.Dialog ):

    def __init__( self, parent ):
        wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = "EZSkin Figure Editor", pos = wx.DefaultPosition, size = wx.Size( 480,600 ), style = wx.CAPTION|wx.RESIZE_BORDER|wx.STAY_ON_TOP )

        self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )

        siz_Dialog = wx.BoxSizer( wx.VERTICAL )

        self.pnl_Editor = wx.Panel( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
        siz_Editor = wx.BoxSizer( wx.VERTICAL )

        siz_FigObjFile = wx.BoxSizer( wx.HORIZONTAL )

        self.lbl_ObjFile = wx.StaticText( self.pnl_Editor, wx.ID_ANY, "Figure's object file", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_ObjFile.Wrap( -1 )
        siz_FigObjFile.Add( self.lbl_ObjFile, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_ObjFile = wx.TextCtrl( self.pnl_Editor, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_READONLY )
        self.ctl_ObjFile.SetToolTip( "The name of the geometry file" )

        siz_FigObjFile.Add( self.ctl_ObjFile, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        siz_Editor.Add( siz_FigObjFile, 0, wx.EXPAND, 5 )

        siz_UseDef = wx.BoxSizer( wx.HORIZONTAL )

        self.lbl_UseDef = wx.StaticText( self.pnl_Editor, wx.ID_ANY, "Use figure definition", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_UseDef.Wrap( -1 )
        siz_UseDef.Add( self.lbl_UseDef, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        ctl_FigDefChoiceChoices = [ "Custom" ]
        self.ctl_FigDefChoice = wx.Choice( self.pnl_Editor, wx.ID_ANY, wx.DefaultPosition, wx.Size( 200,-1 ), ctl_FigDefChoiceChoices, 0 )
        self.ctl_FigDefChoice.SetSelection( 0 )
        self.ctl_FigDefChoice.SetToolTip( "A list of all defined figure types." )

        siz_UseDef.Add( self.ctl_FigDefChoice, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_AddObj = wx.Button( self.pnl_Editor, wx.ID_ANY, "<< Add Figure", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.ctl_AddObj.SetToolTip( "Set the current figure to use the selected figure type definition" )

        siz_UseDef.Add( self.ctl_AddObj, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        siz_Editor.Add( siz_UseDef, 0, wx.EXPAND, 5 )

        siz_Shaders = wx.BoxSizer( wx.HORIZONTAL )

        self.ctl_Shaders = wx.ListCtrl( self.pnl_Editor, wx.ID_ANY, wx.DefaultPosition, wx.Size( 450,-1 ), wx.LC_HRULES|wx.LC_REPORT|wx.LC_VRULES )
        siz_Shaders.Add( self.ctl_Shaders, 0, wx.ALL|wx.EXPAND, 5 )


        siz_Editor.Add( siz_Shaders, 1, wx.EXPAND, 5 )

        siz_Select = wx.BoxSizer( wx.HORIZONTAL )

        ctl_SelectShaderChoices = []
        self.ctl_SelectShader = wx.Choice( self.pnl_Editor, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, ctl_SelectShaderChoices, wx.CB_SORT )
        self.ctl_SelectShader.SetSelection( 0 )
        self.ctl_SelectShader.SetToolTip( "The shaders available to EZSkin" )

        siz_Select.Add( self.ctl_SelectShader, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_Group = wx.SpinCtrl( self.pnl_Editor, wx.ID_ANY, "1", wx.DefaultPosition, wx.Size( 60,-1 ), wx.SP_ARROW_KEYS, 1, 1000, 1 )
        self.ctl_Group.SetToolTip( "Scatter group" )

        siz_Select.Add( self.ctl_Group, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_ApplyShader = wx.Button( self.pnl_Editor, wx.ID_ANY, "Apply shader", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.ctl_ApplyShader.SetToolTip( "Set the highlighted material to use the selected shader" )

        siz_Select.Add( self.ctl_ApplyShader, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        siz_Editor.Add( siz_Select, 0, wx.EXPAND, 5 )

        box_UpdateDefs = wx.BoxSizer( wx.HORIZONTAL )

        self.ctl_SaveDef = wx.Button( self.pnl_Editor, wx.ID_ANY, "Save definition as ->", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
        self.ctl_SaveDef.SetToolTip( "Create a NEW figure type definition, with the specified name" )

        box_UpdateDefs.Add( self.ctl_SaveDef, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.txt_FigDef = wx.TextCtrl( self.pnl_Editor, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
        self.txt_FigDef.SetMaxLength( 0 )
        self.txt_FigDef.SetToolTip( "The name for the figure type definition" )

        box_UpdateDefs.Add( self.txt_FigDef, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_UpdateDef = wx.Button( self.pnl_Editor, wx.ID_ANY, "Update definition", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
        self.ctl_UpdateDef.SetToolTip( "Update the EXISTING figure type definition with these settings" )

        box_UpdateDefs.Add( self.ctl_UpdateDef, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_RefreshEditor = wx.Button( self.pnl_Editor, wx.ID_ANY, "Reset", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
        self.ctl_RefreshEditor.SetToolTip( "Remove all unsaved changes" )

        box_UpdateDefs.Add( self.ctl_RefreshEditor, 0, wx.ALL, 5 )


        siz_Editor.Add( box_UpdateDefs, 0, wx.EXPAND, 5 )

        siz_Status = wx.BoxSizer( wx.HORIZONTAL )

        self.ctl_Message = wx.TextCtrl( self.pnl_Editor, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_READONLY )
        siz_Status.Add( self.ctl_Message, 1, wx.ALL, 5 )

        siz_DlgButtons = wx.StdDialogButtonSizer()
        self.siz_DlgButtonsOK = wx.Button( self.pnl_Editor, wx.ID_OK )
        siz_DlgButtons.AddButton( self.siz_DlgButtonsOK )
        siz_DlgButtons.Realize();

        siz_Status.Add( siz_DlgButtons, 0, wx.EXPAND, 5 )


        siz_Editor.Add( siz_Status, 0, wx.EXPAND, 5 )


        self.pnl_Editor.SetSizer( siz_Editor )
        self.pnl_Editor.Layout()
        siz_Editor.Fit( self.pnl_Editor )
        siz_Dialog.Add( self.pnl_Editor, 1, wx.EXPAND |wx.ALL, 5 )


        self.SetSizer( siz_Dialog )
        self.Layout()

        self.Centre( wx.BOTH )

        # Connect Events
        self.ctl_FigDefChoice.Bind( wx.EVT_CHOICE, self.ChangeFigureDef )
        self.ctl_AddObj.Bind( wx.EVT_BUTTON, self.AddFigureToDef )
        self.ctl_ApplyShader.Bind( wx.EVT_BUTTON, self.ApplyShader )
        self.ctl_SaveDef.Bind( wx.EVT_BUTTON, self.SaveDefAs )
        self.ctl_UpdateDef.Bind( wx.EVT_BUTTON, self.UpdateFigureDef )
        self.ctl_RefreshEditor.Bind( wx.EVT_BUTTON, self.RefreshEditor )

    def __del__( self ):
        pass


    # Virtual event handlers, overide them in your derived class
    def ChangeFigureDef( self, event ):
        event.Skip()

    def AddFigureToDef( self, event ):
        event.Skip()

    def ApplyShader( self, event ):
        event.Skip()

    def SaveDefAs( self, event ):
        event.Skip()

    def UpdateFigureDef( self, event ):
        event.Skip()

    def RefreshEditor( self, event ):
        event.Skip()
class EZEditor(Editor):

    def __init__(self, parent, library, figure, shaders=None):

        # Save the parameters
        Editor.__init__(self, parent)
        self.ezskin = parent
        self.lib = library
        self.fig = figure
        self.shaders = shaders or {}

        # Start the show
        self.InitialiseEditor()

    def AddFigureToDef( self, event, figtype=None ):
        # Associate the actor with the specified type definition, or as in the drop down box

        if figtype == None:
            idx = self.ctl_FigDefChoice.GetSelection()
            if  idx == 0:
                self.Message('Cannot apply figure to Custom figtype')
                return
            else:
                figtype = self.ctl_FigDefChoice.GetString(idx)

        if self.lib.AddObj(self.lib.GetObjName(self.fig, None), figtype):
            self.Message('%s defined as type %s' % (self.fig.Name(), figtype))
        else:
            self.Message('Unable to add figure to selected definition')

        self.InitialiseEditor()

    def ApplyShader(self, event):
        # Associate the selected shader with the highlighted material(s)

        if self.ctl_Shaders.GetSelectedItemCount() == 0:
            self.Message('No items selected')
        else:
            itm = self.ctl_Shaders.GetFirstSelected()
            while itm != -1:
                self.ctl_Shaders.SetItem(itm, 1, label=self.ctl_SelectShader.GetStringSelection())
                self.ctl_Shaders.SetItem(itm, 2, label=str(self.ctl_Group.GetValue()))
                itm = self.ctl_Shaders.GetNextSelected(itm)

    def GetCustom(self):
        # Returns the custom shaders if 'Custom' is selected, or None
        if self.ctl_FigDefChoice.GetSelection() == 0:    # 'Custom'
            return ListDictionary(self.ctl_Shaders)
        else:
            return None

    def InitialiseEditor(self):

        # For Genesis compatibility at a later date
        prefix = None

        # Get our geometry name
        obj = self.lib.GetObjName(self.fig, None) or 'embedded'

        # We can't save defintions for props with embedded geometry
        if obj == 'embedded':
            self.ctl_SaveDef.Disable()

        # Just for convenience ...
        ctl = self.ctl_Shaders

        # Display the figure's geometry file name
        self.ctl_ObjFile.SetValue('%s, which is of type "%s"' % (obj, self.lib.GetTypeStr(obj)))

        # Get known figure types
        figtypes = sorted(self.lib.GetAllTemplates())
        figtypes.insert(0, 'Custom')

        # Configure Figure Definition control
        self.ctl_FigDefChoice.SetItems(figtypes)
        figtype = self.lib.GetType(obj)

        if not self.shaders == {}:
            self.ctl_FigDefChoice.SetSelection(0)
        else:
            if figtype:
                idx = self.ctl_FigDefChoice.FindString(figtype)
                if idx == wx.NOT_FOUND:
                    self.ctl_FigDefChoice.SetSelection(0)
                else:
                    self.ctl_FigDefChoice.SetSelection(idx)
            else:
                self.ctl_FigDefChoice.SetSelection(0)

        # Get list of material names
        materials = []
        for m in self.fig.Materials(): materials.append(m.Name())
        materials.sort()

        # And the list of shaders
        shaders = self.ezskin.builder.Shaders()
        shaders.sort()
        self.ctl_SelectShader.SetItems(shaders)
        self.ctl_SelectShader.SetSelection(0)

        # Populate list with data
        ctl.ClearAll()
        ctl.InsertColumn(0, 'Material', width=200)
        ctl.InsertColumn(1, 'Shader', width=150)
        ctl.InsertColumn(2, 'Group', width=80)

        for mat in materials:
            s = self.shaders.get(mat, self.lib.GetTemplate(self.fig, mat, figtype, prefix=prefix) or ('ignore', 1))
            ctl.Append([mat, s[0], s[1]])

        # Clear 'Save definition as' text box
        self.txt_FigDef.Clear()

    def Message(self, msg):
        # Display a message in the mock 'status bar'
        self.ctl_Message.SetValue(msg)

    def RefreshEditor( self, event ):
        # Refresh allcontrols, losing any unsaved changes
        self.shaders = {}
        self.InitialiseEditor()

    def SaveDefAs( self, event ):
        # Create a new type definition and associate the actor with it

        new = self.txt_FigDef.GetValue()

        if new != "":
            self.UpdateFigureDef(None, new)
            self.AddFigureToDef(None, new)
            self.InitialiseEditor()
        else:
            self.Message('You need to enter a definition name')

    def UpdateFigureDef( self, event, new=None ):
        # Change the type definition in the drop down box to match the current settings

        # New definition, or modifying an old one?
        if new:
            figtype = new
            msg = 'saved'
        else:
            figtype = self.ctl_FigDefChoice.GetStringSelection()
            msg = 'updated'

        # Change the definition
        if figtype != 'Custom':
            data = ListDictionary(self.ctl_Shaders)
            self.lib.UpdateType(figtype, data)
            self.Message('Definition %s' % msg)
        else:
            self.Message("Custom def. cannot be updated - use 'Save def. as' instead")

# SHADERMAKER
class Shadermaker(object):

    def __init__(self, api):

        self.api = api
        self.parameters = None     # The method to interrogate the value of UI controls

        self.shaders = {
            "cornea" : (self.MakeEyetrans, EYETYPE),
            "eyelash" : (self.MakeEyelash, OTHERTYPE),
            "eyetrans" : (self.MakeEyetrans, EYETYPE),
            "gums" : (self.MakeGums, MOUTHTYPE),
            "hair" : (self.MakeHair, OTHERTYPE),
            "ignore" : (self.Nix, SKINTYPE),
            "innermouth" : (self.MakeInnermouth, MOUTHTYPE),
            "invisible" : (self.MakeInvisible, OTHERTYPE),
            "iris" : (self.MakeIris, EYETYPE),
            "lacrimal" : (self.MakeLacrimal, EYETYPE),
            "lips" : (self.MakeLips, MOUTHTYPE),
            "nails" : (self.MakeNails, NAILTYPE),
            "pupil" : (self.MakePupil, EYETYPE),
            "sclera" : (self.MakeSclera, EYETYPE),
            "skin" : (self.MakeSkin, SKINTYPE),
            "specular" : (self.MakeSpecular, SKINTYPE),
            "tear" : (self.MakeEyetrans, EYETYPE),
            "teeth" : (self.MakeTeeth, TEETHTYPE),
            "tongue" : (self.MakeTongue, MOUTHTYPE),
            }

    def BuildShader(self, material, data, mat):

        # Parse the data
        shader = data[0]
        scattergroup = int(data[1])

        # Don't waste time if we don't have to do anything
        if shader == 'ignore': return 'ignored'

        # Stop if no PoserSurface is found
        if mat.Root() is None: return 'skipped - No Firefly PoserSurface found'

        # Configure imagemap nodes
        p = self.parameters
        if p('gc'): mat.GC_AllMaps()
        if p('filter'): mat.ApplyTextureFilters(p('filtering') + 1)

        # ... and build the shader
        if shader in self.Shaders():

            # Get the method which will build the shader
            method = self.shaders[shader][METHOD]

            # Check to see if we need to use a built-in shader
            if method is None and self.api.Build() >= 68:
                builder = self.api.GetBuilder(self.Renderer())
                if builder:
                    return builder.BuildShader(material, data, mat)
                else:
                    return 'failed'

            else:
                # Build the shader
                if method is not None:
                    method(mat, p)
                    mat.RemoveOrphans()
                    mat.UpdatePreview()

                return 'processed'

        else:
            return "no shader found"

    def MakeBaseShader(self, mat, p, bumpvalue=None):

        # Initialise
        bvalue = bumpvalue or 0.005

        n000 = mat.Root()

        # Prepare for new shader
        mat.ClearPoserSurface(keepTrans=True, keepBump=True)
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))

        # Build nodes
        n001 = mat.GetMap('color')
        n002 = mat.AddNode('blender', name='Blender', loc=(500, 437), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(0.000, 0.000, 0.000), Blending=1.000, )
        n003 = mat.AddNode('blinn', name='EZSpecular', loc=(500, 551), collapsed=False, preview=False, Specular_Color=(1.000, 1.000, 1.000), Eccentricity=p('spec_eccentricity'), SpecularRollOff=p('spec_rolloff'), Reflectivity=p('specularity'), Normals_Forward=0.000, )
        n004 = mat.AddNode('scatter', name='EZScatter', loc=(800, 447), collapsed=False, preview=False, Material=p('sss_type') + 1, Use_Material_Color=0.000, Texture_Detail=1.000, scale=p('sss_scale'), maxError=1.0 - p('sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        n005 = mat.AddNode('blender', name='Specular blend', loc=(800, 724), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n006 = mat.AddNode('math_functions', name='Proc. Specular Map', loc=(1050, 567), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.200, )
        n007 = mat.AddNode('math_functions', name='PSMap Add', loc=(1050, 615), collapsed=True, preview=False, Math_Argument=1.000, Value_1=0.500, Value_2=0.500, )
        n008 = mat.AddNode('turbulence', name='PSMap Turbulence', loc=(1050, 663), collapsed=True, preview=False, x_Index=0.060, y_Index=0.060, z_Index=0.060, Octaves=4.000, Bottom=0.000, Bias=0.500, Gain=0.500, Noise_Type=2.000, )
        n009 = mat.AddNode('math_functions', name='Subtract', loc=(1050, 475), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n010 = mat.AddNode('comp', name='Comp', loc=(1050, 427), collapsed=True, preview=False, Component=0.000, Color=(1.000, 1.000, 1.000), Point=(0.000, 0.000, 0.000), )
        n011 = mat.AddNode('hsv', name='HSV', loc=(1050, 311), collapsed=False, preview=False, Color=p('HSV_colour'), Hue=p('HSV_H'), Saturation=p('HSV_S'), Value=p('HSV_V'), )
        n012 = mat.AddNode('blender', name='EZBump', loc=(250, 281), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n013 = mat.AddNode('math_functions', name='PBMultiply', loc=(500, 375), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
        n014 = mat.AddNode('math_functions', name='PBSubtract', loc=(500, 327), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n015 = mat.AddNode('turbulence', name='Procedural bump', loc=(500, 279), collapsed=True, preview=False, x_Index=0.100, y_Index=0.100, z_Index=0.100, Octaves=3.000, Bottom=0.000, Bias=0.800, Gain=0.500, Noise_Type=2.000, )
        n016 = mat.AddNode('reflect', name='EZReflectColour', loc=(250, 66), collapsed=False, preview=False, Background=(0.000, 0.000, 0.000), Quality=p('reflect_quality'), Diffusion=p('reflect_softness'), RayBias=p('reflect_raybias'), )
        n017 = mat.AddNode('math_functions', name='EZReflectValue', loc=(250, 182), collapsed=False, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
        n018 = mat.AddNode('math_functions', name='Fresnel blend adj', loc=(500, 165), collapsed=False, preview=False, Math_Argument=1.0, Value_1=0.000, Value_2=p(('fresnel_mult', 'rough_mult')[p('rough_fresnel')]), )
        n019 = mat.AddNode('fresnel_blend', name='Fresnel blend', loc=(500, 66), collapsed=False, preview=False, Outer_Color=(1.000, 1.000, 1.000), Inner_Color=(0.000, 0.000, 0.000), Index_of_Refraction=p(('fresnel_ior', 'rough_ior')[p('rough_fresnel')]), )
        n020 = mat.AddNode('math_functions', name='Specular softness factor', loc=(500, 684), collapsed=False, preview=False, Math_Argument=2.000, Value_1=4.500, Value_2=1.000, )
        n021 = mat.AddNode('scatter', name='BBMaxScatter', loc=(800, 280), collapsed=False, preview=False, Material=p('sss_type') + 1, Use_Material_Color=0.000, Texture_Detail=1.000, scale=p('bbSSSscale'), maxError=1.0 - p('sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        n022 = mat.AddNode('hsv', name='HSV2', loc=(800, 164), collapsed=False, preview=False, Color=(1.000, 1.000, 1.000), Hue=1.000, Saturation=1.000, Value=p('bbHSVscale'), )
        n023 = mat.AddNode('color_math', name='EZDiffuse', loc=(800, 65), collapsed=False, preview=False, Math_Argument=16.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )

        # Colour maps

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(1050, 63), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(1050, 63), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Specular
        n024 = mat.GetMap('specular')
        if n024:
            mat.ConnectToPS(n024, 'Highlight_Value')
            mat.ConnectNodes(n024, n005, 'Input_2')
            if not mat.SameAsKdTexture(n024):
                mat.ConfigureNode(n024, loc=(1050, 711), name="Specular Map", showPreview=True, showInputs=False)

        # Bump
        n025 = mat.GetMap('bump')
        if n025:
            mat.ConfigureNode(n025, loc=(250, 428), name='Bump map', showPreview=True, showInputs=False)
            n026 = mat.AddNode('math_functions', loc=(250, 380), name='Bump map mod', Math_Argument=3.0, Value_2=1.0)
            mat.ConnectNodes(n025, n026, "Value_1")
            mat.ConnectNodes(n026, n012, 'Input_2')
        else:
            mat.SetPSValue('Bump', bvalue)

        # Connect nodes
        mat.ConnectNodes(n001, n010, 'Color')
        mat.ConnectNodes(n001, n011, 'Color')
        mat.ConnectNodes(n002, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n003, n000, 'AlternateSpecular')
        mat.ConnectNodes(n003, n002, 'Blending')
        mat.ConnectNodes(n004, n023, 'Value_2')
        mat.ConnectNodes(n005, n003, 'Reflectivity')
        mat.ConnectNodes(n005, n017, 'Value_2')
        mat.ConnectNodes(n005, n020, 'Value_2')
        mat.ConnectNodes(n006, n005, 'Input_1')
        mat.ConnectNodes(n007, n006, 'Value_2')
        mat.ConnectNodes(n008, n007, 'Value_2')
        mat.ConnectNodes(n009, n004, 'Texture_Detail')
        mat.ConnectNodes(n009, n021, 'Texture_Detail')
        mat.ConnectNodes(n010, n006, 'Value_1')
        mat.ConnectNodes(n010, n009, 'Value_2')
        mat.ConnectNodes(n011, n004, 'Color')
        mat.ConnectNodes(n011, n021, 'Color')
        mat.ConnectNodes(n012, n000, 'Bump')
        mat.ConnectNodes(n013, n012, 'Input_1')
        mat.ConnectNodes(n014, n013, 'Value_1')
        mat.ConnectNodes(n015, n014, 'Value_1')
        mat.ConnectNodes(n016, n000, 'Reflection_Color')
        mat.ConnectNodes(n017, n000, 'Reflection_Value')
        mat.ConnectNodes(n018, n017, 'Value_1')
        mat.ConnectNodes(n019, n018, 'Value_2')
        mat.ConnectNodes(n020, n016, 'Diffusion')
        mat.ConnectNodes(n021, n022, 'Color')
        mat.ConnectNodes(n022, n023, 'Value_1')
        mat.ConnectNodes(n023, n002, 'Input_1')

        # APPLY USER SETTINGS

        # BB MAX SCATTER
        if not p('maxscatter'):
            mat.ConnectNodes(n004, n002, 'Input_1')

        # FRESNEL REFLECTION
        if p('add_fresnel'):
            mat.SetPSValue('Reflection_Color', (1.000, 1.000, 1.000))
            mat.SetPSValue('Reflection_Value', 1.000)
        else:
            mat.PSDisconnect('Reflection_Color')
            mat.PSDisconnect('Reflection_Value')

        # MODULATE SPECULAR SOFTNESS
        if p('modulate_spec'):
            n016.InputByInternalName('Diffusion').SetFloat(1.0)
        else:
            n016.InputByInternalName('Diffusion').Disconnect()

        # BUMP

        b = p('bump_opt')                                        # Get the user's selections ...
        basebmp = bvalue * p('bump_mod')
        bump = mat.GetMap('bump') is not None                     # Is there a bump map?
        bmpBlend = (1, 0)[(not bump and b == 1) or (b == 2)]     # Determine the blending value

        bv = mat.PSValue('Bump')                                # The bump value
        if bv == 0: bv = bvalue                                    # Avoid division by zero

        procbmpMult = basebmp / bv                                 # Procedural bump modifier

        # Determine blender's Bump map colour chip value
        if not bump and b == 0:
            r, g, b = (0,0,0)
        else:
            r, g, b = (1,1,1)

        # Configure procedural nodes
        n012.InputByInternalName('Input_2').SetColor(r, g, b)
        n012.InputByInternalName('Blending').SetFloat(bmpBlend)
        n013.InputByInternalName('Value_2').SetFloat(procbmpMult)

        # Configure bump map nodes
        if n025:
            n026.InputByInternalName('Value_2').SetFloat((1.0, p('bump_value') / bv)[p('bump_usecustom')])

        # SPECULAR

        # Is there a specular map?
        spec = n024 is not None

        # Get the user's preference
        s = p('spec_opt')

        # Get the blending value
        bl = (1, 0)[(not spec and s == 1) or (s == 2)]

        # Determine whether or not there will be any specular mapping
        if not spec and s == 0:
            # Delete the blender node
            n005.Delete()
        else:
            # Update the blender node
            n005.InputByInternalName('Blending').SetFloat(bl)

    def MakeEyelash(self, mat, p):

        mat.ClearPoserSurface(keepTrans=True)

        n01 = mat.GetMap('transparency')

        if n01:
            mat.ConfigureNode(n01, name='Transparency Map', loc=(290, 200), showInputs=False, showPreview=True)
            mat.ConnectToPS(n01, 'Transparency_Max', value=1.0)
            mat.ConnectToPS(n01, 'Transparency_Min', value=1.0)

    def MakeEyetrans(self, mat, p):

        # Initialise
        blur = p('eye_blur')
        eccentricity = 0.03 + 0.065 * blur
        reflectivity = 90 * (0.5 ** blur)
        reflect_quality = p('eye_reflect_quality')
        ior = p('eye_ior')

        #Configure PoserSurface
        mat.ClearPoserSurface()
        mat.SetPSValue('Diffuse_Color', (1.000, 1.000, 1.000))
        mat.SetPSValue('Highlight_Color', (1.000, 1.000, 1.000))
        mat.SetPSValue('Transparency_Max', 1.000)
        mat.SetPSValue('Transparency_Min', 0.000)
        mat.SetPSValue('Transparency_Falloff', 0.000)
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))

        # Build nodes
        n000 = mat.Root()
        n001 = mat.AddNode('fresnel_blend', name='Fresnel_Blend', loc=(250, 128), collapsed=False, preview=False, Outer_Color=(1.000, 1.000, 1.000), Inner_Color=(0.000, 0.000, 0.000), Index_of_Refraction=ior, )
        n002 = mat.AddNode('color_math', name='Color_Add', loc=(250, 315), collapsed=False, preview=False, Math_Argument=1.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )
        n003 = mat.AddNode('reflect', name='Reflect', loc=(550, 293), collapsed=False, preview=False, Background=(0.000, 0.000, 0.000), Quality=reflect_quality, Diffusion=blur, RayBias=0.300, )
        n004 = mat.AddNode('blinn', name='Blinn', loc=(550, 409), collapsed=False, preview=False, Specular_Color=(1.000, 1.000, 1.000), Eccentricity=eccentricity, SpecularRollOff=1.000, Reflectivity=reflectivity, Normals_Forward=0.000, )

        # Connect nodes
        mat.ConnectNodes(n001, n000, 'Transparency_Max')
        mat.ConnectNodes(n002, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n003, n002, 'Value_1')
        mat.ConnectNodes(n004, n002, 'Value_2')

    def MakeGums(self, mat, p):
        self.MakeMouth(mat, p, kind='gums')

    def MakeHair(self, mat, p):

        mat.ClearPoserSurface(keepTrans=True)

        # Color_math
        n01 = mat.AddNode('color_math', loc=(250,340), collapsed=False, Value_2=(1.0, 1.0, 1.0)); mat.ConnectToPS(n01, "AlternateDiffuse", value=(1, 1, 1))
        mat.ConnectToPS(n01, 'AlternateDiffuse', (1,1,1))

        # Blender
        n03 = mat.AddNode('blender', loc=(490, 240), collapsed=False, Input_1=(1,1,1), Input_2=(0,0,0), Blending=1.0)
        mat.ConnectNodes(n03, n01, 'Value_1')

        # Diffuse
        if p('hair_useSSS'):
            n02 = mat.AddNode('scatter', loc=(730, 70), collapsed=False, Material=9.0, Texture_Detail=0.8, Use_Material_Color=0, scale=p('hair_sss_scale'), maxError=1.0 - p('hair_sss_quality'), Scatter_Group=mat.Scattergroup())
        else:
            n02 = mat.AddNode('diffuse', loc=(730, 70), collapsed=False, Diffuse_Color=(1,1,1), Diffuse_Value=(0.9))
        mat.ConnectNodes(n02, n03, 'Input_1')

        # Anisotropic
        sc = p('hair_colour')
        sv = p('hair_specular')
        n04 = mat.AddNode('aniso', loc=(490, 360), collapsed=False, Specular_Color=sc, Specular_Value=sv, u_Roughness=(1.0, 0.1)[p('hair_uvdir')], v_Roughness=(0.1, 1.0)[p('hair_uvdir')])

        mat.ConnectNodes(n04, n03, 'Blending')
        mat.ConnectNodes(n04, n01, 'Value_2')

        # HSV
        n05 = mat.AddNode('hsv', loc=(750, 237), collapsed=False, Color=(1,1,1), Hue=1.0, Saturation=0.4, Value=1.0)
        mat.ConnectNodes(n05, n04, 'Specular_Color')

        # Diffuse Color map
        n06 = mat.GetMap("color")
        if n06:
            mat.ConfigureNode(n06, loc=(970, 30), name="Color Map", showPreview=True, showInputs=False)
            mat.ConnectToPS(n06, "Diffuse_Color")
            mat.ConnectNodes(n06, n05, "Color")
            mat.ConnectNodes(n06, n02, ("Diffuse_Color", "Color")[p('hair_useSSS')])
        else:
            mat.SetInputValue(n02.InputByInternalName(('Diffuse_Color', 'Color')[p('hair_useSSS')]), (mat.GetPSBackup('Diffuse_Color')))

        if p('hair_keepKd'):
            mat.SetInputValue(n02.InputByInternalName(('Diffuse_Color', 'Color')[p('hair_useSSS')]), mat.GetPSBackup('Diffuse_Color'))

        # Transparency map
        n07 = mat.AddNode('edge_blend', loc=(1000, 300), collapsed=False, Inner_Color=(1,1,1), Outer_Color=(0,0,0), Attenuation=1.0)
        mat.ConnectNodes(n07, n04, 'Specular_Value')

        n08 = mat.GetMap('transparency')
        if n08:
            mat.ConfigureNode(n08, name='Transparency Map', loc=(970, 385), showInputs=False, showPreview=True)
            mat.ConnectToPS(n08, 'Transparency_Max', value=1.0)
            mat.ConnectNodes(n08, n07, 'Inner_Color')
            mat.ConnectNodes(n08, n07, 'Outer_Color')

        # Specular map
        n09 = mat.GetMap('specular')
        if n09:
            mat.ConfigureNode(n09, loc=(730, 535), name="Specular Map", showPreview=True, showInputs=False)
            mat.ConnectToPS(n09, 'Highlight_Value')
            mat.ConnectNodes(n09, n04, 'Specular_Value')

        # Specular Color map
        n10 = mat.GetMap('specular_color')
        if n10:
            mat.ConfigureNode(n10, loc=(730, 300), name="Specular Color Map", showPreview=True, showInputs=False)
            mat.ConnectToPS(n10, 'Highlight_Color')
            mat.ConnectNodes(n10, n04, 'Specular_Color')

    def MakeInnermouth(self, mat, p):

        #Configure PoserSurface
        n000 = mat.Root()

        # Prepare for new shader
        mat.ClearPoserSurface(keepBump=True)
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))
        mat.SetPSValue('Reflection_Color', (1.000, 1.000, 1.000))
        mat.SetPSValue('Reflection_Value', 1.0)

        # Build nodes
        n001 = mat.GetMap('color')
        n002 = mat.AddNode('scatter', name='Scatter', loc=(650, 414), collapsed=False, preview=False, Material=p('mouth_sss_type') + 1, Use_Material_Color=0.000, Texture_Detail=0.700, scale=p('mouth_sss_scale'), maxError=1.0 - p('mouth_sss_quality'), Color=p('mouth_colour'), Scatter_Group=mat.Scattergroup() )
        n003 = mat.AddNode('reflect', name='Reflect', loc=(300, 109), collapsed=False, preview=False, Background=(0.000, 0.000, 0.000), Quality=p('mouth_reflect_quality'), Diffusion=p('mouth_reflect_softness'), RayBias=p('mouth_reflect_raybias'), )
        n004 = mat.AddNode('ks_microfacet', name='EZSpecular', loc=(300, 426), collapsed=False, preview=False, Specular_Color=(1.000, 1.000, 1.000), roughness=p('mouth_spec_roughness'), Ks=p('mouth_spec_ks'), )
        n005 = mat.AddNode('fresnel_blend', name='Fresnel Blend', loc=(650, 315), collapsed=False, preview=False, Outer_Color=(1.000, 1.000, 1.000), Inner_Color=(0.000, 0.000, 0.000), Index_of_Refraction=p('mouth_fresnel_ior'), )
        n006 = mat.AddNode('turbulence', name='Turbulence', loc=(300, 225), collapsed=True, preview=False, x_Index=0.030, y_Index=0.030, z_Index=0.030, Octaves=8.000, Bottom=0.000, Bias=0.500, Gain=0.150, Noise_Type=1.000, )

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(650, 67), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(1050, 63), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Connect nodes
        mat.ConnectNodes(n001, n000, 'Diffuse_Color')
        mat.ConnectNodes(n003, n000, 'Reflection_Color')
        mat.ConnectNodes(n005, n000, 'Reflection_Value')
        mat.ConnectNodes(n006, n000, 'Bump')
        mat.ConnectNodes(n002, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n004, n000, 'AlternateSpecular')
        mat.ConnectNodes(n005, n004, 'Ks')
        mat.ConnectNodes(n001, n002, 'Color')

    def MakeInvisible(self, mat, p):
        mat.ClearPoserSurface()
        mat.SetInputValue(mat.Root().InputByInternalName('Transparency_Max'), 1.0)
        mat.SetInputValue(mat.Root().InputByInternalName('Transparency_Min'), 0.0)

    def MakeIris(self, mat, p, kind='iris'):

        # Initialise
        pbv = 0.002        # Procedural bump value

        #Configure PoserSurface
        mat.ClearPoserSurface(keepBump=True)
        mat.SetPSValue('Diffuse_Color', (1.000, 1.000, 1.000))
        mat.SetPSValue('Highlight_Color', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))

        # Build nodes
        n000 = mat.Root()
        n001 = mat.GetMap('color')
        n002 = mat.AddNode('hsv', name='HSV2', loc=(500, 590), collapsed=False, preview=False, Color=(1.000, 1.000, 1.000), Hue=1.000, Saturation=1.000, Value=0.800, )
        n003 = mat.AddNode('scatter', name='Scatter', loc=(500, 423), collapsed=False, preview=False, Material=9.000, Use_Material_Color=0.000, Texture_Detail=1.000, scale=1.000, maxError=1.0 - p('eye_sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        if kind == 'iris':
            n004 = mat.AddNode('hsv', name='HSV', loc=(750, 314), collapsed=False, preview=False, Color=p('iris_HSV_colour'), Hue=p('iris_HSV_H'), Saturation=p('iris_HSV_S'), Value=p('iris_HSV_V'), )
        else:
            n004 = mat.AddNode('hsv', name='HSV', loc=(750, 314), collapsed=False, preview=False, Color=(1.000, 1.000, 1.000), Hue=1.000, Saturation=1.000, Value=1.000, )
        n012 = mat.AddNode('blender', name='EZBump', loc=(250, 281), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n013 = mat.AddNode('math_functions', name='PBMultiply', loc=(500, 341), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
        n014 = mat.AddNode('math_functions', name='Sub Proc Bump', loc=(500, 248), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n015 = mat.AddNode('turbulence', name='Turbulence', loc=(500, 64), collapsed=False, preview=False, x_Index=0.070, y_Index=0.070, z_Index=0.070, Octaves=8.000, Bottom=0.000, Bias=0.700, Gain=0.600, Noise_Type=2.000, )

        # Colour maps

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(750, 66), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(750, 66), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Bump
        n005 = mat.GetMap('bump')
        if n005:
            mat.ConfigureNode(n005, loc=(250, 478), name='Bump map', showPreview=True, showInputs=False)
            n006 = mat.AddNode('math_functions', loc=(250, 379), name='Bump map mod', collapsed=False, Math_Argument=2.0, Value_1=1.0, Value_2=0.5)
            mat.ConnectNodes(n005, n006, "Value_1")
            mat.ConnectNodes(n006, n012, 'Input_2')
        else:
            mat.SetPSValue('Bump', pbv)

        # Connect nodes
        mat.ConnectNodes(n001, n004, 'Color')
        mat.ConnectNodes(n002, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n003, n002, 'Color')
        mat.ConnectNodes(n004, n003, 'Color')
        mat.ConnectNodes(n012, n000, 'Bump')
        mat.ConnectNodes(n013, n012, 'Input_1')
        mat.ConnectNodes(n014, n013, 'Value_1')
        mat.ConnectNodes(n015, n014, 'Value_1')

        # APPLY USER SETTINGS

        # BUMP

        b = p('eye_bump_opt')                                    # Get the user's selections ...
        basebmp = pbv * p('bump_mod')
        bump = mat.GetMap('bump') is not None                     # Is there a bump map?
        bmpBlend = (1, 0)[(not bump and b == 1) or (b == 2)]     # Determine the blending value

        bv = mat.PSValue('Bump')                                # The bump value
        if bv == 0: bv = pbv                                    # Avoid division by zero

        procbmpMult = basebmp / bv                                 # Procedural bump modifier

        # Determine blender's Bump map colour chip value
        if not bump and b == 0:
            r, g, b = (0,0,0)
        else:
            r, g, b = (1,1,1)

        # Configure procedural nodes
        n012.InputByInternalName('Input_2').SetColor(r, g, b)
        n012.InputByInternalName('Blending').SetFloat(bmpBlend)
        n013.InputByInternalName('Value_2').SetFloat(procbmpMult)

    def MakeLacrimal(self, mat, p):

        self.MakeBaseShader(mat, p)
        nodes = mat.AllNodes()

        # Configure scatter node
        scatter = nodes.get('EZScatter', None)
        if scatter:
            scatter.InputByInternalName('Texture_Detail').SetFloat(1.0)

        # Configure Blinn node
        blinn = nodes.get('EZSpecular', None)
        if blinn:
            blinn.InputByInternalName('Eccentricity').SetFloat(0.02)

    def MakeLips(self, mat, p, bumpvalue=None):

        # Check for skin shader preference
        if p('lips_use_skin'):
            self.MakeBaseShader(mat, p, bumpvalue)
            return

        # Initialise
        bvalue = bumpvalue or 0.005

        n000 = mat.Root()

        # Prepare for new shader
        mat.ClearPoserSurface(keepTrans=True, keepBump=True)
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))

        # Build nodes
        n001 = mat.GetMap('color')
        n002 = mat.AddNode('blender', name='Blender', loc=(580, 437), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(0.000, 0.000, 0.000), Blending=1.000, )
        n003 = mat.AddNode('blinn', name='EZSpecular', loc=(580, 551), collapsed=False, preview=False, Specular_Color=(1.000, 1.000, 1.000), Eccentricity=p('lips_spec_eccentricity'), SpecularRollOff=p('lips_spec_rolloff'), Reflectivity=p('lips_specularity'), Normals_Forward=0.000, )
        n004 = mat.AddNode('scatter', name='EZScatter', loc=(860, 447), collapsed=False, preview=False, Material=p('sss_type') + 1, Use_Material_Color=0.000, Texture_Detail=1.000, scale=p('sss_scale'), maxError=1.0 - p('sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        n005 = mat.AddNode('blender', name='Specular blend', loc=(860, 724), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n006 = mat.AddNode('math_functions', name='Proc. Specular Map', loc=(1140, 567), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.200, )
        n007 = mat.AddNode('math_functions', name='PSMap Add', loc=(1140, 615), collapsed=True, preview=False, Math_Argument=1.000, Value_1=0.500, Value_2=0.500, )
        n008 = mat.AddNode('turbulence', name='PSMap Turbulence', loc=(1140, 663), collapsed=True, preview=False, x_Index=0.060, y_Index=0.060, z_Index=0.060, Octaves=4.000, Bottom=0.000, Bias=0.500, Gain=0.500, Noise_Type=2.000, )
        n009 = mat.AddNode('math_functions', name='Subtract', loc=(1140, 475), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n010 = mat.AddNode('comp', name='Comp', loc=(1140, 427), collapsed=True, preview=False, Component=0.000, Color=(1.000, 1.000, 1.000), Point=(0.000, 0.000, 0.000), )
        n011 = mat.AddNode('hsv', name='HSV', loc=(1140, 311), collapsed=False, preview=False, Color=p('lips_colour'), Hue=1.000, Saturation=1.000, Value=p('lips_V'), )
        n012 = mat.AddNode('blender', name='EZBump', loc=(300, 281), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n013 = mat.AddNode('math_functions', name='PBMultiply', loc=(580, 375), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
        n014 = mat.AddNode('math_functions', name='PBSubtract', loc=(580, 327), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n015 = mat.AddNode('turbulence', name='Procedural bump', loc=(580, 279), collapsed=True, preview=False, x_Index=0.100, y_Index=10.000, z_Index=0.100, Octaves=3.000, Bottom=0.000, Bias=0.800, Gain=0.500, Noise_Type=2.000, )
        n016 = mat.AddNode('reflect', name='EZReflectColour', loc=(300, 66), collapsed=False, preview=False, Background=(0.000, 0.000, 0.000), Quality=p('reflect_quality'), Diffusion=p('reflect_softness'), RayBias=p('reflect_raybias'), )
        n017 = mat.AddNode('math_functions', name='EZReflectValue', loc=(300, 182), collapsed=False, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
        n018 = mat.AddNode('math_functions', name='Fresnel blend adj', loc=(580, 165), collapsed=False, preview=False, Math_Argument=1.0, Value_1=0.000, Value_2=p(('fresnel_mult', 'rough_mult')[p('rough_fresnel')]), )
        n019 = mat.AddNode('fresnel_blend', name='Fresnel blend', loc=(580, 66), collapsed=False, preview=False, Outer_Color=(1.000, 1.000, 1.000), Inner_Color=(0.000, 0.000, 0.000), Index_of_Refraction=p(('fresnel_ior', 'rough_ior')[p('rough_fresnel')]), )
        n020 = mat.AddNode('math_functions', name='Specular softness factor', loc=(580, 684), collapsed=False, preview=False, Math_Argument=2.000, Value_1=4.500, Value_2=1.000, )
        n021 = mat.AddNode('scatter', name='BBMaxScatter', loc=(860, 280), collapsed=False, preview=False, Material=p('sss_type') + 1, Use_Material_Color=0.000, Texture_Detail=1.000, scale=p('bbSSSscale'), maxError=1.0 - p('sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        n022 = mat.AddNode('hsv', name='HSV2', loc=(860, 164), collapsed=False, preview=False, Color=(1.000, 1.000, 1.000), Hue=1.000, Saturation=1.000, Value=p('bbHSVscale'), )
        n023 = mat.AddNode('color_math', name='EZDiffuse', loc=(860, 65), collapsed=False, preview=False, Math_Argument=16.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )

        # Colour maps

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(1140, 63), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(1140, 63), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Specular
        n024 = mat.GetMap('specular')
        if n024:
            mat.ConnectToPS(n024, 'Highlight_Value')
            mat.ConnectNodes(n024, n005, 'Input_2')
            if not mat.SameAsKdTexture(n024):
                mat.ConfigureNode(n024, loc=(1140, 711), name="Specular Map", showPreview=True, showInputs=False)

        # Bump
        n025 = mat.GetMap('bump')
        if n025:
            mat.ConfigureNode(n025, loc=(300, 428), name='Bump map', showPreview=True, showInputs=False)
            n026 = mat.AddNode('math_functions', loc=(300, 380), name='Bump map mod', Math_Argument=3.0, Value_2=1.0)
            mat.ConnectNodes(n025, n026, "Value_1")
            mat.ConnectNodes(n026, n012, 'Input_2')
        else:
            mat.SetPSValue('Bump', bvalue)

        # Connect nodes
        mat.ConnectNodes(n001, n010, 'Color')
        mat.ConnectNodes(n001, n011, 'Color')
        mat.ConnectNodes(n002, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n003, n000, 'AlternateSpecular')
        mat.ConnectNodes(n003, n002, 'Blending')
        mat.ConnectNodes(n004, n023, 'Value_2')
        mat.ConnectNodes(n005, n003, 'Reflectivity')
        mat.ConnectNodes(n005, n017, 'Value_2')
        mat.ConnectNodes(n005, n020, 'Value_2')
        mat.ConnectNodes(n006, n005, 'Input_1')
        mat.ConnectNodes(n007, n006, 'Value_2')
        mat.ConnectNodes(n008, n007, 'Value_2')
        mat.ConnectNodes(n009, n004, 'Texture_Detail')
        mat.ConnectNodes(n009, n021, 'Texture_Detail')
        mat.ConnectNodes(n010, n006, 'Value_1')
        mat.ConnectNodes(n010, n009, 'Value_2')
        mat.ConnectNodes(n011, n004, 'Color')
        mat.ConnectNodes(n011, n021, 'Color')
        mat.ConnectNodes(n012, n000, 'Bump')
        mat.ConnectNodes(n013, n012, 'Input_1')
        mat.ConnectNodes(n014, n013, 'Value_1')
        mat.ConnectNodes(n015, n014, 'Value_1')
        mat.ConnectNodes(n016, n000, 'Reflection_Color')
        mat.ConnectNodes(n017, n000, 'Reflection_Value')
        mat.ConnectNodes(n018, n017, 'Value_1')
        mat.ConnectNodes(n019, n018, 'Value_2')
        mat.ConnectNodes(n020, n016, 'Diffusion')
        mat.ConnectNodes(n021, n022, 'Color')
        mat.ConnectNodes(n022, n023, 'Value_1')
        mat.ConnectNodes(n023, n002, 'Input_1')

        # APPLY USER SETTINGS

        # BB MAX SCATTER
        if not p('maxscatter'):
            mat.ConnectNodes(n004, n002, 'Input_1')

        # FRESNEL REFLECTION
        if p('add_fresnel'):
            mat.SetPSValue('Reflection_Color', (1.000, 1.000, 1.000))
            mat.SetPSValue('Reflection_Value', 1.000)
        else:
            mat.PSDisconnect('Reflection_Color')
            mat.PSDisconnect('Reflection_Value')

        # MODULATE SPECULAR SOFTNESS
        if p('modulate_spec'):
            n016.InputByInternalName('Diffusion').SetFloat(1.0)
        else:
            n016.InputByInternalName('Diffusion').Disconnect()

        # BUMP

        b = p('bump_opt')                                        # Get the user's selections ...
        basebmp = bvalue * p('bump_mod')
        bump = mat.GetMap('bump') is not None                     # Is there a bump map?
        bmpBlend = (1, 0)[(not bump and b == 1) or (b == 2)]     # Determine the blending value

        bv = mat.PSValue('Bump')                                # The bump value
        if bv == 0: bv = bvalue                                    # Avoid division by zero

        procbmpMult = basebmp / bv                                 # Procedural bump modifier

        # Determine blender's Bump map colour chip value
        if not bump and b == 0:
            r, g, b = (0,0,0)
        else:
            r, g, b = (1,1,1)

        # Configure procedural nodes
        n012.InputByInternalName('Input_2').SetColor(r, g, b)
        n012.InputByInternalName('Blending').SetFloat(bmpBlend)
        n013.InputByInternalName('Value_2').SetFloat(procbmpMult)

        # Configure bump map nodes
        if n025:
            n026.InputByInternalName('Value_2').SetFloat((1.0, p('bump_value') / bv)[p('bump_usecustom')])

        # SPECULAR

        # Is there a specular map?
        spec = n024 is not None

        # Get the user's preference
        s = p('spec_opt')

        # Get the blending value
        bl = (1, 0)[(not spec and s == 1) or (s == 2)]

        # Determine whether or not there will be any specular mapping
        if not spec and s == 0:
            # Delete the blender node
            n005.Delete()
        else:
            # Update the blender node
            n005.InputByInternalName('Blending').SetFloat(bl)

    def MakeSkin(self, mat, p):

        # Build the base shader
        self.MakeBaseShader(mat, p)

    def MakeMouth(self, mat, p, kind):

        # Initialise
        if kind == 'gums':
            ks = p('mouth_spec_ks_gum')
            bmp = p('mouth_gum_bump')
            refl = 1.0
        elif kind == 'tongue':
            ks = p('mouth_spec_ks_tongue')
            bmp = p('mouth_tongue_bump')
            refl = 0.5
        else:
            raise Exception('Unknown mouth type')

        # Configure PoserSurface
        mat.ClearPoserSurface()
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))
        mat.SetPSValue('Reflection_Color', (1.000, 1.000, 1.000))
        mat.SetPSValue('Reflection_Value', refl)
        mat.SetPSValue('Bump', bmp)

        # Build nodes
        n000 = mat.Root()
        n001 = mat.GetMap('color')
        n002 = mat.AddNode('scatter', name='Scatter', loc=(500, 110), collapsed=False, preview=False, Material=9.000, Use_Material_Color=0.000, Texture_Detail=0.300, scale=p('mouth_sss_scale'), maxError=1.0 - p('mouth_sss_quality'), Color=p('mouth_colour'), Scatter_Group=mat.Scattergroup(), )
        n003 = mat.AddNode('reflect', name='Reflect', loc=(250, 110), collapsed=False, preview=False, Background=(0.000, 0.000, 0.000), Quality=p('mouth_reflect_quality'), Diffusion=p('mouth_reflect_softness'), RayBias=p('mouth_reflect_raybias'), )
        n004 = mat.AddNode('ks_microfacet', name='ks_microfacet', loc=(250, 488), collapsed=False, preview=False, Specular_Color=(1.000, 1.000, 1.000), roughness=p('mouth_spec_roughness'), Ks=ks, )
        n005 = mat.AddNode('fresnel_blend', name='Fresnel_Blend', loc=(500, 461), collapsed=False, preview=False, Outer_Color=(1.000, 1.000, 1.000), Inner_Color=(0.000, 0.000, 0.000), Index_of_Refraction=p('mouth_fresnel_ior'), )
        n006 = mat.AddNode('blender', name='Blender', loc=(250, 290), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )

        if kind == 'gums':
            n007 = mat.AddNode('math_functions', name='Math_Functions', loc=(500, 373), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
            n008 = mat.AddNode('math_functions', name='Math_Functions_2', loc=(500, 325), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
            n009 = mat.AddNode('turbulence', name='Procedural bump', loc=(500, 277), collapsed=True, preview=False, x_Index=0.100, y_Index=0.100, z_Index=0.100, Octaves=3.000, Bottom=0.000, Bias=0.800, Gain=0.500, Noise_Type=2.000, )
        else:
            n009 = mat.AddNode('turbulence', name='Procedural bump', loc=(500, 277), collapsed=False, preview=False, x_Index=0.030, y_Index=0.030, z_Index=0.030, Octaves=8.000, Bottom=0.000, Bias=0.500, Gain=0.150, Noise_Type=2.000, )

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(752, 11), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(752, 11), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Connect nodes
        mat.ConnectNodes(n001, n000, 'Diffuse_Color')
        mat.ConnectNodes(n001, n002, 'Color')
        mat.ConnectNodes(n002, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n003, n000, 'Reflection_Color')
        mat.ConnectNodes(n004, n000, 'AlternateSpecular')
        mat.ConnectNodes(n005, n000, 'Reflection_Value')
        mat.ConnectNodes(n005, n004, 'Ks')
        mat.ConnectNodes(n006, n000, 'Bump')

        if kind == 'gums':
            mat.ConnectNodes(n007, n006, 'Input_1')
            mat.ConnectNodes(n008, n007, 'Value_1')
            mat.ConnectNodes(n009, n008, 'Value_1')
        else:
            mat.ConnectNodes(n009, n006, 'Input_1')

    def MakeNails(self, mat, p):

        # Initialise
        bvalue = 0.005

        n000 = mat.Root()

        # Prepare for new shader
        mat.ClearPoserSurface(keepTrans=True, keepBump=True)
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))

        # Build nodes
        n001 = mat.GetMap('color')
        n002 = mat.AddNode('blender', name='Blender', loc=(500, 437), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(0.000, 0.000, 0.000), Blending=1.000, )
        n003 = mat.AddNode('glossy', loc=(500, 536), collapsed=False, Ks=p('nail_spec_strength'), Roughness=p('nail_spec_size'))
        n004 = mat.AddNode('scatter', name='EZScatter', loc=(800, 447), collapsed=False, preview=False, Material=p('nail_sss_type') + 1, Use_Material_Color=0.000, Texture_Detail=1.000, scale=p('nail_sss_scale'), maxError=1.0 - p('nail_sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        n009 = mat.AddNode('math_functions', name='Subtract', loc=(1050, 475), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n010 = mat.AddNode('comp', name='Comp', loc=(1050, 427), collapsed=True, preview=False, Component=0.000, Color=(1.000, 1.000, 1.000), Point=(0.000, 0.000, 0.000), )
        n011 = mat.AddNode('hsv', name='HSV', loc=(1050, 311), collapsed=False, preview=False, )
        n012 = mat.AddNode('blender', name='EZBump', loc=(250, 281), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n013 = mat.AddNode('math_functions', name='PBMultiply', loc=(500, 375), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
        n014 = mat.AddNode('math_functions', name='PBSubtract', loc=(500, 327), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n015 = mat.AddNode('turbulence', name='Procedural bump', loc=(500, 279), collapsed=True, preview=False, x_Index=0.100, y_Index=0.100, z_Index=0.100, Octaves=3.000, Bottom=0.000, Bias=0.800, Gain=0.500, Noise_Type=2.000, )
        n016 = mat.AddNode('reflect', name='EZReflectColour', loc=(250, 66), collapsed=False, preview=False, Background=(0.000, 0.000, 0.000), Quality=p('nail_reflect_quality'), Diffusion=p('nail_reflect_softness'), RayBias=0.3, )
        n017 = mat.AddNode('math_functions', name='EZReflectValue', loc=(250, 182), collapsed=False, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
        n018 = mat.AddNode('math_functions', name='Fresnel blend adj', loc=(500, 165), collapsed=False, preview=False, Math_Argument=1.0, Value_1=0.000, Value_2=1.0, )
        n019 = mat.AddNode('fresnel_blend', name='Fresnel blend', loc=(500, 66), collapsed=False, preview=False, Outer_Color=(1.000, 1.000, 1.000), Inner_Color=(0.000, 0.000, 0.000), Index_of_Refraction=p('nail_ior'), )
        n020 = mat.AddNode('math_functions', name='Specular softness factor', loc=(500, 684), collapsed=False, preview=False, Math_Argument=2.000, Value_1=4.500, Value_2=1.000, )
        n021 = mat.AddNode('scatter', name='BBMaxScatter', loc=(800, 280), collapsed=False, preview=False, Material=p('nail_sss_type') + 1, Use_Material_Color=0.000, Texture_Detail=1.000, scale=p('nail_bbSSSscale'), maxError=1.0 - p('nail_sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        n022 = mat.AddNode('hsv', name='HSV2', loc=(800, 164), collapsed=False, preview=False, Color=(1.000, 1.000, 1.000), Hue=1.000, Saturation=1.000, Value=p('bbHSVscale'), )
        n023 = mat.AddNode('color_math', name='EZDiffuse', loc=(800, 65), collapsed=False, preview=False, Math_Argument=16.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )
        n027 = mat.AddNode('simple_color', name='Diffuse Color', loc=(1050, 63), collapsed=True, preview=True, Color=p('nail_colour'), )

        # Colour maps

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(1050, 63), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(1050, 63), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Specular
        n024 = mat.GetMap('specular')
        if n024:
            mat.ConnectToPS(n024, 'Highlight_Value')
            mat.ConnectNodes(n024, n003, 'Ks')
            if not mat.SameAsKdTexture(n024):
                mat.ConfigureNode(n024, loc=(1050, 711), name="Specular Map", showPreview=True, showInputs=False)

        # Bump
        n025 = mat.GetMap('bump')
        if n025:
            mat.ConfigureNode(n025, loc=(250, 428), name='Bump map', showPreview=True, showInputs=False)
            n026 = mat.AddNode('math_functions', loc=(250, 380), name='Bump map mod', Math_Argument=3.0, Value_2=1.0)
            mat.ConnectNodes(n025, n026, "Value_1")
            mat.ConnectNodes(n026, n012, 'Input_2')
        else:
            mat.SetPSValue('Bump', bvalue)

        # Connect nodes
        if p('nail_usecolour'):
            mat.ConnectNodes(n027, n010, 'Color')
            mat.ConnectNodes(n027, n011, 'Color')
        else:
            mat.ConnectNodes(n001, n010, 'Color')
            mat.ConnectNodes(n001, n011, 'Color')

        mat.ConnectNodes(n002, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n003, n000, 'AlternateSpecular')
        mat.ConnectNodes(n003, n002, 'Blending')
        mat.ConnectNodes(n004, n023, 'Value_2')
        mat.ConnectNodes(n009, n021, 'Texture_Detail')
        mat.ConnectNodes(n010, n009, 'Value_2')
        mat.ConnectNodes(n011, n004, 'Color')
        mat.ConnectNodes(n011, n021, 'Color')
        mat.ConnectNodes(n012, n000, 'Bump')
        mat.ConnectNodes(n013, n012, 'Input_1')
        mat.ConnectNodes(n014, n013, 'Value_1')
        mat.ConnectNodes(n015, n014, 'Value_1')
        mat.ConnectNodes(n016, n000, 'Reflection_Color')
        mat.ConnectNodes(n017, n000, 'Reflection_Value')
        mat.ConnectNodes(n018, n017, 'Value_1')
        mat.ConnectNodes(n019, n018, 'Value_2')
        mat.ConnectNodes(n020, n016, 'Diffusion')
        mat.ConnectNodes(n021, n022, 'Color')
        mat.ConnectNodes(n022, n023, 'Value_1')
        mat.ConnectNodes(n023, n002, 'Input_1')

        # APPLY USER SETTINGS

        # BB MAX SCATTER
        if not p('nail_maxscatter'):
            mat.ConnectNodes(n004, n002, 'Input_1')

        # FRESNEL REFLECTION
        mat.SetPSValue('Reflection_Color', (1.000, 1.000, 1.000))
        mat.SetPSValue('Reflection_Value', 1.000)

        # MODULATE SPECULAR SOFTNESS
        if p('modulate_spec'):
            n016.InputByInternalName('Diffusion').SetFloat(1.0)
        else:
            n016.InputByInternalName('Diffusion').Disconnect()

        # BUMP

        b = 0                                                    # Bump map or nothing
        basebmp = bvalue * p('bump_mod')
        bump = mat.GetMap('bump') is not None                     # Is there a bump map?
        bmpBlend = (1, 0)[(not bump and b == 1) or (b == 2)]     # Determine the blending value

        bv = mat.PSValue('Bump')                                # The bump value
        if bv == 0: bv = bvalue                                    # Avoid division by zero

        procbmpMult = basebmp / bv                                 # Procedural bump modifier

        # Determine blender's Bump map colour chip value
        if not bump and b == 0:
            r, g, b = (0,0,0)
        else:
            r, g, b = (1,1,1)

        # Configure procedural nodes
        n012.InputByInternalName('Input_2').SetColor(r, g, b)
        n012.InputByInternalName('Blending').SetFloat(bmpBlend)
        n013.InputByInternalName('Value_2').SetFloat(procbmpMult)

        # Configure bump map nodes
        if n025:
            n026.InputByInternalName('Value_2').SetFloat((1.0, p('bump_value') / bv)[p('bump_usecustom')])

    def MakePupil(self, mat, p):
        # Shader shared with Iris :-)
        self.MakeIris(mat, p, kind='pupil')

    def MakeSclera(self, mat, p):

        # Initialise
        pbv = 0.002        # Procedural bump value
        blur = p('eye_blur')
        eccentricity = 0.03 + 0.065 * blur
        reflectivity = 90 * (0.5 ** blur)
        reflect_quality = p('eye_reflect_quality')
        ior = p('eye_ior')

        #Configure PoserSurface
        mat.ClearPoserSurface(keepBump=True)
        mat.SetPSValue('Diffuse_Color', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))

        # Build nodes
        n000 = mat.Root()
        n001 = mat.AddNode('blender', name='Bump blender', loc=(250, 280), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n002 = mat.AddNode('blender', name='Main blender', loc=(250, 548), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=1.000, )
        n003 = mat.AddNode('hsv', name='HSV2', loc=(550, 483), collapsed=False, preview=False, Color=(1.000, 1.000, 1.000), Hue=1.000, Saturation=1.000, Value=0.800, )
        n004 = mat.AddNode('fresnel_blend', name='Fresnel_Blend', loc=(250, 647), collapsed=False, preview=False, Outer_Color=(1.000, 1.000, 1.000), Inner_Color=(0.000, 0.000, 0.000), Index_of_Refraction=ior, )
        n005 = mat.AddNode('color_math', name='Color_add', loc=(550, 599), collapsed=False, preview=False, Math_Argument=1.000, Value_1=(1.000, 1.000, 1.000), Value_2=(1.000, 1.000, 1.000), )
        n006 = mat.AddNode('reflect', name='Reflect', loc=(800, 558), collapsed=False, preview=False, Background=(0.000, 0.000, 0.000), Quality=reflect_quality, Diffusion=blur, RayBias=0.300, )
        n007 = mat.AddNode('blinn', name='Blinn', loc=(800, 674), collapsed=False, preview=False, Specular_Color=(1.000, 1.000, 1.000), Eccentricity=eccentricity, SpecularRollOff=1.000, Reflectivity=reflectivity, Normals_Forward=0.000, )
        n008 = mat.GetMap('color')
        n009 = mat.AddNode('hsv', name='HSV', loc=(800, 275), collapsed=False, preview=False, Color=p('sclera_HSV_colour'), Hue=p('sclera_HSV_H'), Saturation=p('sclera_HSV_S'), Value=p('sclera_HSV_V'), )
        n010 = mat.AddNode('scatter', name='Scatter', loc=(800, 391), collapsed=False, preview=False, Material=9.000, Use_Material_Color=0.000, Texture_Detail=1.000, scale=1.000, maxError=1.0 - p('eye_sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        n011 = mat.AddNode('turbulence', name='Turbulence', loc=(550, 30), collapsed=False, preview=False, x_Index=0.070, y_Index=0.070, z_Index=0.070, Octaves=8.000, Bottom=0.000, Bias=0.700, Gain=0.600, Noise_Type=2.000, )
        n012 = mat.AddNode('math_functions', name='PBSubtract', loc=(550, 214), collapsed=False, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n013 = mat.AddNode('math_functions', name='PBMultiply', loc=(550, 313), collapsed=False, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )

        # Diffuse
        if n008:
            mat.ConfigureNode(n008, loc=(800, 27), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n008, 'Diffuse_Color')
        else:
            n008 = mat.AddNode('simple_color', name='Diffuse Color', loc=(800, 27), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Bump
        n025 = mat.GetMap('bump')
        if n025:
            mat.ConfigureNode(n025, loc=(250, 478), name='Bump map', showPreview=False, showInputs=False)
            n026 = mat.AddNode('math_functions', loc=(250, 379), name='Bump map mod', collapsed=False, Math_Argument=2.0, Value_1=1.0, Value_2=0.5)
            mat.ConnectNodes(n025, n026, "Value_1")
            mat.ConnectNodes(n026, n001, 'Input_2')
        else:
            mat.SetPSValue('Bump', pbv)

        # Connect nodes
        mat.ConnectNodes(n001, n000, 'Bump')
        mat.ConnectNodes(n002, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n012, n013, 'Value_1')
        mat.ConnectNodes(n010, n003, 'Color')
        mat.ConnectNodes(n011, n012, 'Value_1')
        mat.ConnectNodes(n006, n005, 'Value_1')
        mat.ConnectNodes(n007, n005, 'Value_2')
        mat.ConnectNodes(n008, n009, 'Color')
        mat.ConnectNodes(n009, n010, 'Color')
        mat.ConnectNodes(n003, n002, 'Input_1')
        mat.ConnectNodes(n005, n002, 'Input_2')
        mat.ConnectNodes(n004, n002, 'Blending')
        mat.ConnectNodes(n013, n001, 'Input_1')

        # APPLY USER SETTINGS

        # BUMP

        b = p('eye_bump_opt')                                    # Get the user's selections ...
        basebmp = pbv
        bump = mat.GetMap('bump') is not None                     # Is there a bump map?
        bmpBlend = (1, 0)[(not bump and b == 1) or (b == 2)]     # Determine the blending value

        bv = mat.PSValue('Bump')                                # The bump value
        if bv == 0: bv = pbv                                    # Avoid division by zero

        procbmpMult = basebmp / bv                                 # Procedural bump modifier

        # Determine blender's Bump map colour chip value
        if not bump and b == 0:
            r, g, b = (0,0,0)
        else:
            r, g, b = (1,1,1)

        # Configure procedural nodes
        n001.InputByInternalName('Input_2').SetColor(r, g, b)
        n001.InputByInternalName('Blending').SetFloat(bmpBlend)
        n013.InputByInternalName('Value_2').SetFloat(procbmpMult)

    def MakeSkin(self, mat, p):

        # Build the base shader
        self.MakeBaseShader(mat, p)

    def MakeSpecular(self, mat, p):

        mat.ClearPoserSurface()

        mat.SetPSValue('Transparency_Max', 1.0)
        n01 = mat.AddNode('aniso', loc=(300,400), collapsed=False)
        mat.ConnectToPS(n01, 'AlternateSpecular', value=(1.0, 1.0, 1.0))

    def MakeTeeth(self, mat, p):

        # Prepare for new shader
        mat.ClearPoserSurface()
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))

        # Build nodes
        n000 = mat.Root()
        n001 = mat.GetMap('color')
        n002 = mat.AddNode('scatter', name='EZScatter', loc=(600, 110), collapsed=False, preview=False, Material=p('teeth_sss_type') + 1, Use_Material_Color=0.000, Texture_Detail=0.700, scale=p('teeth_sss_scale'), maxError=1.0 - p('teeth_sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        n003 = mat.AddNode('hsv', name='hHSV', loc=(850, 268), collapsed=False, preview=False, Color=p('teeth_HSV_colour'), Hue=p('teeth_HSV_H'), Saturation=p('teeth_HSV_S'), Value=p('teeth_HSV_V'), )
        n004 = mat.AddNode('reflect', name='Reflect', loc=(250, 110), collapsed=False, preview=False, Background=(0.000, 0.000, 0.000), Quality=p('teeth_reflect_quality'), Diffusion=p('teeth_reflect_softness'), RayBias=p('teeth_reflect_raybias'), )
        n005 = mat.AddNode('blinn', name='EZSpecular', loc=(250, 462), collapsed=False, preview=False, Specular_Color=(1.000, 1.000, 1.000), Eccentricity=0.020, SpecularRollOff=1.000, Reflectivity=15.000, Normals_Forward=0.000, )
        n006 = mat.AddNode('fresnel_blend', name='Fresnel blend', loc=(600, 277), collapsed=False, preview=False, Outer_Color=(1.000, 1.000, 1.000), Inner_Color=(0.000, 0.000, 0.000), Index_of_Refraction=p('teeth_fresnel_ior'), )
        n007 = mat.AddNode('turbulence', name='Turbulence', loc=(250, 278), collapsed=False, preview=False, x_Index=0.100, y_Index=0.500, z_Index=0.100, Octaves=8.000, Bottom=0.000, Bias=0.500, Gain=0.500, Noise_Type=1.000, )

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(850, 20), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(850, 20), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Connect nodes
        mat.ConnectNodes(n001, n003, 'Color')
        mat.ConnectNodes(n002, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n003, n002, 'Color')
        mat.ConnectNodes(n004, n000, 'Reflection_Color')
        mat.ConnectNodes(n005, n000, 'AlternateSpecular')
        mat.ConnectNodes(n006, n000, 'Reflection_Value')
        mat.ConnectNodes(n006, n005, 'Reflectivity')
        mat.ConnectNodes(n007, n000, 'Bump')

    def MakeTongue(self, mat, p):
        self.MakeMouth(mat, p, kind='tongue')

    def Nix(self, mat, p):
        pass

    def Renderer(self):
        return 'Firefly'

    def Shaders(self):
        return list(self.shaders.keys())

    def ShaderType(self, shader):
        return self.shaders.get(shader, (None, 0))[SHADERTYPE]
class ShadermakerCycles(Shadermaker):

    def _PrepareRoot(self, mat, p):

        # Backup PS values
        mat.MakeBackup(mat.Root())

        # Make new root
        if p('make_root', 'settings'):
            n000 = mat.MakeRoot(renderer='superfly', roottype=poser.kNodeTypeCodePOSERSURFACE)
            mat.SetPSRoot(n000)
        else:
            n000 = mat.Root()

        # Copy values
        for inp in ('Bump', 'Displacement', 'Gradient_Bump', 'Gradient_Mode', 'Transparency_Max', 'Transparency_Min', 'Transparency_Falloff'):
            mat.SetPSValue(inp, mat.GetPSBackup(inp))

        # Transparency
        tmap = mat.GetMap('transparency')
        if tmap:
            tmap.ConnectToInput(n000.InputByInternalName('Transparency_Max'))
            mat.ConfigureNode(tmap, loc=(300,170), showPreview=False, showInputs=False)

        return n000

    def MakeEyelash(self, mat, p):

        n000 = self._PrepareRoot(mat, p)
        mat.ClearPoserSurface(keepTrans=True, use_root=n000)

        n01 = mat.GetMap('transparency')

        if n01:
            mat.ConfigureNode(n01, name='Transparency Map', loc=(290, 200), showInputs=False, showPreview=True)
            mat.ConnectToPS(n01, 'Transparency_Max', value=1.0)
            mat.ConnectToPS(n01, 'Transparency_Min', value=1.0)

    def MakeEyetrans(self, mat, p):

        n000 = mat.AddNode('PhysicalSurface', name='Root', loc=(20, 20), collapsed=False, preview=False, Color=(0.980, 0.980, 0.980), Transparency=1.000, Roughness=0.050, Specular=(0.973, 0.973, 0.973), Metallic=0.100, Emission=(1.000, 1.000, 1.000), EmissionStrength=0.000, Bump=0.000, Displacement=0.000, Volume=(0.300, 0.000, 1.000), )
        mat.SetRenderer('superfly', n000)

    def MakeGums(self, mat, p):
        # Gums and Tongue share a similar shader
        self.MakeMouth(mat, p, kind='gums')

    def MakeHair(self, mat, p):

        n000 = self._PrepareRoot(mat, p)

        # Prepare for new shader
        mat.ClearPoserSurface(keepTrans=True, keepBump=True, use_root=n000, backup=False)
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))

        # Diffuse
        if p('hair_useSSS'):
            n02 = mat.AddNode('scatter', loc=(300, 107), collapsed=False, Material=9.0, Texture_Detail=0.8, Use_Material_Color=0, scale=p('hair_sss_scale'), maxError=1.0 - p('hair_sss_quality'), Scatter_Group=mat.Scattergroup())
        else:
            n02 = mat.AddNode('diffuse', loc=(300, 175), collapsed=False, Diffuse_Color=(1,1,1), Diffuse_Value=(0.9))
        mat.ConnectToPS(n02, 'AlternateDiffuse')

        # Anisotropic
        sv = p('hair_specular')
        n04 = mat.AddNode('aniso', loc=(300, 390), collapsed=False, Specular_Color=(1,1,1), Specular_Value=sv, u_Roughness=(1.0, 0.1)[p('hair_uvdir')], v_Roughness=(0.1, 1.0)[p('hair_uvdir')])
        mat.ConnectToPS(n04, 'AlternateSpecular')

        # HSV
        n05 = mat.AddNode('hsv2', loc=(300, 274), collapsed=False, Color=p('hair_colour'), Hue=1.0, Saturation=1.0, Value=p('hair_V'))
        mat.ConnectNodes(n05, n04, 'Specular_Color')

        # Diffuse Color map
        n06 = mat.GetMap("color")
        if n06:
            mat.ConfigureNode(n06, loc=(580, 29), name="Color Map", showPreview=True, showInputs=False)
            mat.ConnectToPS(n06, "Diffuse_Color")

            # Convert to monochrome?
            if p('hair_bw'):
                n12 = mat.AddNode('math_functions', name='Math_Functions', loc=(580, 277), collapsed=False, preview=False, Math_Argument=1.000, Value_1=1.000, Value_2=0.000, )
                mat.ConnectNodes(n06, n12, 'Value_1')
                mat.ConnectNodes(n12, n05, "Color")
            else:
                mat.ConnectNodes(n06, n05, "Color")

            mat.ConnectNodes(n05, n02, ("Diffuse_Color", "Color")[p('hair_useSSS')])
        else:
            mat.SetInputValue(n02.InputByInternalName(('Diffuse_Color', 'Color')[p('hair_useSSS')]), (mat.GetPSBackup('Diffuse_Color')))

        if p('hair_keepKd'):
            mat.SetInputValue(n02.InputByInternalName(('Diffuse_Color', 'Color')[p('hair_useSSS')]), mat.GetPSBackup('Diffuse_Color'))

        # Transparency map
        n07 = mat.AddNode('edge_blend', loc=(580, 525), collapsed=False, Inner_Color=(1,1,1), Outer_Color=(0,0,0), Attenuation=1.0)
        mat.ConnectNodes(n07, n04, 'Specular_Value')

        n08 = mat.GetMap('transparency')
        if n08:
            mat.ConfigureNode(n08, name='Transparency Map', loc=(580, 376), showInputs=False, showPreview=True)
            mat.ConnectToPS(n08, 'Transparency_Max', value=1.0)
            mat.ConnectNodes(n08, n07, 'Inner_Color')
            mat.ConnectNodes(n08, n07, 'Outer_Color')

            # Translucence
            mat.SetPSValue('Translucence_Color', (1.0, 1.0, 1.0))
            mat.SetPSValue('Translucence_Value', p('hair_translucence'))
            mat.ConnectToPS(n08, 'Translucence_Value')
            mat.ConnectToPS(n05, 'Translucence_Color')

        # Specular map
        n09 = mat.GetMap('specular')
        if n09:
            mat.ConfigureNode(n09, loc=(580, 624), name="Specular Map", showPreview=True, showInputs=False)
            mat.ConnectToPS(n09, 'Highlight_Value')
            mat.ConnectNodes(n09, n04, 'Specular_Value')

        # Specular Color map
        n10 = mat.GetMap('specular_color')
        if n10:
            mat.ConfigureNode(n10, loc=(580, 525), name="Specular Color Map", showPreview=True, showInputs=False)
            mat.ConnectToPS(n10, 'Highlight_Color')
            mat.ConnectNodes(n10, n04, 'Specular_Color')

        # Reflection
        if p('hair_useReflection'):
            n11 = mat.AddNode('turbulence', name='Turbulence', loc=(300, 59), collapsed=True, preview=False, x_Index=0.010, y_Index=0.010, z_Index=4.000, Octaves=8.000, Bottom=0.000, Bias=0.500, Gain=0.500, Noise_Type=1.000, Offset_X=0.000, Offset_Y=0.000, Offset_Z=0.000, )
            mat.SetPSValue('Reflection_Color', (1.0, 1.0, 1.0))
            mat.SetPSValue('Reflection_Value', 0.1)
            mat.ConnectToPS(n05, 'Reflection_Color')
            mat.ConnectToPS(n11, 'Reflection_Value')

    def MakeInnermouth(self, mat, p):

        # Initialise
        n000 = self._PrepareRoot(mat, p)

        # Prepare for new shader
        mat.ClearPoserSurface(keepTrans=True, keepBump=True, use_root=n000, backup=False)
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))

        # Build nodes
        n001 = mat.GetMap('color')
        n002 = mat.AddNode('scatter', name='Scatter', loc=(650, 414), collapsed=False, preview=False, Material=p('mouth_sss_type') + 1, Use_Material_Color=0.000, Texture_Detail=0.700, scale=p('mouth_sss_scale'), maxError=1.0 - p('mouth_sss_quality'), Color=p('mouth_colour'), Scatter_Group=mat.Scattergroup() )
        n004 = mat.AddNode('ks_microfacet', name='EZSpecular', loc=(300, 426), collapsed=False, preview=False, Specular_Color=(1.000, 1.000, 1.000), roughness=p('mouth_spec_roughness'), Ks=p('mouth_spec_ks'), )
        n006 = mat.AddNode('turbulence', name='Turbulence', loc=(300, 225), collapsed=True, preview=False, x_Index=0.030, y_Index=0.030, z_Index=0.030, Octaves=8.000, Bottom=0.000, Bias=0.500, Gain=0.150, Noise_Type=1.000, )

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(650, 67), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(1050, 63), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Connect nodes
        mat.ConnectNodes(n001, n000, 'Diffuse_Color')
        mat.ConnectNodes(n006, n000, 'Bump')
        mat.ConnectNodes(n002, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n004, n000, 'AlternateSpecular')
        mat.ConnectNodes(n001, n002, 'Color')

    def MakeIris(self, mat, p, kind='iris'):

        # Initialise
        pbv = 0.002        # Procedural bump value
        n000 = self._PrepareRoot(mat, p)

        # Prepare for new shader
        mat.ClearPoserSurface(keepTrans=True, keepBump=True, use_root=n000, backup=False)
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))

        n001 = mat.GetMap('color')
        n003 = mat.AddNode('scatter', name='Scatter', loc=(500, 423), collapsed=False, preview=False, Material=9.000, Use_Material_Color=0.000, Texture_Detail=1.000, scale=1.000, maxError=1.0 - p('eye_sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        if kind == 'iris':
            n004 = mat.AddNode('hsv2', name='HSV', loc=(750, 314), collapsed=False, preview=False, Color=p('iris_HSV_colour'), Hue=p('iris_HSV_H'), Saturation=p('iris_HSV_S'), Value=p('iris_HSV_V'), )
        else:
            n004 = mat.AddNode('hsv2', name='HSV', loc=(750, 314), collapsed=False, preview=False, Color=(1.000, 1.000, 1.000), Hue=1.000, Saturation=1.000, Value=1.000, )
        n012 = mat.AddNode('blender', name='EZBump', loc=(250, 281), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n013 = mat.AddNode('math_functions', name='PBMultiply', loc=(500, 341), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
        n014 = mat.AddNode('math_functions', name='Sub Proc Bump', loc=(500, 248), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n015 = mat.AddNode('turbulence', name='Turbulence', loc=(500, 64), collapsed=False, preview=False, x_Index=0.070, y_Index=0.070, z_Index=0.070, Octaves=8.000, Bottom=0.000, Bias=0.700, Gain=0.600, Noise_Type=2.000, )

        # Colour maps

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(750, 66), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(750, 66), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Bump
        n005 = mat.GetMap('bump')
        if n005:
            mat.ConfigureNode(n005, loc=(250, 478), name='Bump map', showPreview=True, showInputs=False)
            n006 = mat.AddNode('math_functions', loc=(250, 379), name='Bump map mod', collapsed=False, Math_Argument=2.0, Value_1=1.0, Value_2=0.5)
            mat.ConnectNodes(n005, n006, "Value_1")
            mat.ConnectNodes(n006, n012, 'Input_2')
        else:
            mat.SetPSValue('Bump', pbv)

        # Connect nodes
        mat.ConnectNodes(n001, n004, 'Color')
        mat.ConnectNodes(n003, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n004, n003, 'Color')
        mat.ConnectNodes(n012, n000, 'Bump')
        mat.ConnectNodes(n013, n012, 'Input_1')
        mat.ConnectNodes(n014, n013, 'Value_1')
        mat.ConnectNodes(n015, n014, 'Value_1')

        # APPLY USER SETTINGS

        # BUMP

        b = p('eye_bump_opt')                                    # Get the user's selections ...
        basebmp = pbv * p('bump_mod')
        bump = mat.GetMap('bump') is not None                     # Is there a bump map?
        bmpBlend = (1, 0)[(not bump and b == 1) or (b == 2)]     # Determine the blending value

        bv = mat.PSValue('Bump')                                # The bump value
        if bv == 0: bv = pbv                                    # Avoid division by zero

        procbmpMult = basebmp / bv                                 # Procedural bump modifier

        # Determine blender's Bump map colour chip value
        if not bump and b == 0:
            r, g, b = (0,0,0)
        else:
            r, g, b = (1,1,1)

        # Configure procedural nodes
        n012.InputByInternalName('Input_2').SetColor(r, g, b)
        n012.InputByInternalName('Blending').SetFloat(bmpBlend)
        n013.InputByInternalName('Value_2').SetFloat(procbmpMult)

    def MakeInvisible(self, mat, p):

        n000 = self._PrepareRoot(mat, p)
        mat.ClearPoserSurface(use_root=n000)
        mat.SetInputValue(mat.Root().InputByInternalName('Transparency_Max'), 1.0)
        mat.SetInputValue(mat.Root().InputByInternalName('Transparency_Min'), 0.0)

    def MakeLacrimal(self, mat, p):
        self.MakeSkin(mat, p)

        nodes = mat.AllNodes()

        # Configure scatter node
        scatter = nodes.get('EZScatter', None)
        if scatter:
            scatter.InputByInternalName('Texture_Detail').SetFloat(1.0)

        # Configure Blinn node
        blinn = nodes.get('EZBlinn', None)
        if blinn:
            blinn.InputByInternalName('Eccentricity').SetFloat(0.02)

    def MakeLips(self, mat, p):

        # Initialise
        bvalue = 0.005
        n000 = self._PrepareRoot(mat, p)

        # Prepare for new shader
        mat.ClearPoserSurface(keepTrans=True, keepBump=True, use_root=n000, backup=False)
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))

        # Build nodes
        n001 = mat.GetMap('color')
        n003 = mat.AddNode('blinn', name='EZBlinn', loc=(530, 551), collapsed=False, preview=False, Specular_Color=(1.000, 1.000, 1.000), Eccentricity=p('sf_lips_spec_eccentricity'), SpecularRollOff=p('sf_lips_spec_rolloff'), Reflectivity=p('sf_lips_spec_reflectivity'), Normals_Forward=0.000, )
        n004 = mat.AddNode('scatter', name='EZScatter', loc=(810, 447), collapsed=False, preview=False, Material=p('sss_type') + 1, Use_Material_Color=0.000, Texture_Detail=1.000, scale=p('sss_scale'), maxError=1.0 - p('sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        n005 = mat.AddNode('blender', name='Specular blend', loc=(810, 724), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n006 = mat.AddNode('math_functions', name='Proc. Specular Map', loc=(1090, 567), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.200, )
        n007 = mat.AddNode('math_functions', name='PSMap Add', loc=(1090, 615), collapsed=True, preview=False, Math_Argument=1.000, Value_1=0.500, Value_2=0.500, )
        n008 = mat.AddNode('turbulence', name='PSMap Turbulence', loc=(1090, 663), collapsed=True, preview=False, x_Index=0.060, y_Index=0.060, z_Index=0.060, Octaves=4.000, Bottom=0.000, Bias=0.500, Gain=0.500, Noise_Type=2.000, )
        n009 = mat.AddNode('math_functions', name='Subtract', loc=(1090, 475), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n010 = mat.AddNode('comp', name='Comp', loc=(1090, 427), collapsed=True, preview=False, Component=0.000, Color=(1.000, 1.000, 1.000), Point=(0.000, 0.000, 0.000), )
        n011 = mat.AddNode('hsv2', name='HSV', loc=(1090, 311), collapsed=False, preview=False, Color=p('lips_colour'), Hue=1.000, Saturation=1.000, Value=p('lips_V'), )
        n012 = mat.AddNode('blender', name='EZBump', loc=(250, 281), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n013 = mat.AddNode('math_functions', name='PBMultiply', loc=(530, 375), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
        n014 = mat.AddNode('math_functions', name='PBSubtract', loc=(530, 327), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n015 = mat.AddNode('turbulence', name='Procedural bump', loc=(530, 279), collapsed=True, preview=False, x_Index=0.100, y_Index=10.000, z_Index=0.100, Octaves=3.000, Bottom=0.000, Bias=0.800, Gain=0.500, Noise_Type=2.000, )

        # Colour maps

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(1090, 63), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(1090, 63), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Specular
        n024 = mat.GetMap('specular')
        if n024:
            mat.ConnectToPS(n024, 'Highlight_Value')
            mat.ConnectNodes(n024, n005, 'Input_2')
            if not mat.SameAsKdTexture(n024):
                mat.ConfigureNode(n024, loc=(1090, 711), name="Specular Map", showPreview=True, showInputs=False)

        # Bump
        n025 = mat.GetMap('bump')
        if n025:
            mat.ConfigureNode(n025, loc=(250, 428), name='Bump map', showPreview=True, showInputs=False)
            n026 = mat.AddNode('math_functions', loc=(250, 380), name='Bump map mod', Math_Argument=3.0, Value_1=p('bump_mod'), Value_2=1.0)
            mat.ConnectNodes(n025, n026, "Value_1")
            mat.ConnectNodes(n026, n012, 'Input_2')
        else:
            mat.SetPSValue('Bump', bvalue)

        # Normal
        n027 = mat.GetMap('normal')
        if n027:
            mat.ConfigureNode(n027, loc=(250, 676), name='Normal map', showPreview=True, showInputs=False)
            mat.ConnectNodes(n027, n000, 'Gradient_Bump')

        # Connect nodes
        mat.ConnectNodes(n001, n010, 'Color')
        mat.ConnectNodes(n001, n011, 'Color')
        mat.ConnectNodes(n003, n000, 'AlternateSpecular')
        mat.ConnectNodes(n004, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n005, n003, 'Specular_Color')
        mat.ConnectNodes(n006, n005, 'Input_1')
        mat.ConnectNodes(n007, n006, 'Value_2')
        mat.ConnectNodes(n008, n007, 'Value_2')
        mat.ConnectNodes(n009, n004, 'Texture_Detail')
        mat.ConnectNodes(n010, n006, 'Value_1')
        mat.ConnectNodes(n010, n009, 'Value_2')
        mat.ConnectNodes(n011, n004, 'Color')
        mat.ConnectNodes(n012, n000, 'Bump')
        mat.ConnectNodes(n013, n012, 'Input_1')
        mat.ConnectNodes(n014, n013, 'Value_1')
        mat.ConnectNodes(n015, n014, 'Value_1')

        # APPLY USER SETTINGS

        # BUMP

        b = p('bump_opt')                                        # Get the user's selections ...
        basebmp = bvalue * p('bump_mod')
        bump = mat.GetMap('bump') is not None                     # Is there a bump map?
        bmpBlend = (1, 0)[(not bump and b == 1) or (b == 2)]     # Determine the blending value

        bv = mat.PSValue('Bump')                                # The bump value
        if bv == 0: bv = bvalue                                    # Avoid division by zero

        procbmpMult = basebmp / bv                                 # Procedural bump modifier

        # Determine blender's Bump map colour chip value
        if not bump and b == 0:
            r, g, b = (0,0,0)
        else:
            r, g, b = (1,1,1)

        # Configure procedural nodes
        n012.InputByInternalName('Input_2').SetColor(r, g, b)
        n012.InputByInternalName('Blending').SetFloat(bmpBlend)
        n013.InputByInternalName('Value_2').SetFloat(procbmpMult)

        # Configure bump map nodes
        if n025:
            n026.InputByInternalName('Value_2').SetFloat((1.0, p('bump_value') / bv)[p('bump_usecustom')])

        # SPECULAR

        # Is there a specular map?
        spec = n024 is not None

        # Get the user's preference
        s = p('spec_opt')

        # Get the blending value
        bl = (1, 0)[(not spec and s == 1) or (s == 2)]

        # Determine whether or not there will be any specular mapping
        if not spec and s == 0:
            # Delete the blender node
            n005.Delete()
        else:
            # Update the blender node
            n005.InputByInternalName('Blending').SetFloat(bl)

    def MakeMouth(self, mat, p, kind):

        # Initialise
        if kind == 'gums':
            ks = p('mouth_spec_ks_gum')
            bmp = p('mouth_gum_bump')
        elif kind == 'tongue':
            ks = p('mouth_spec_ks_tongue')
            bmp = p('mouth_tongue_bump')
        else:
            raise Exception('Unknown mouth type')

        # Make the root node
        n000 = self._PrepareRoot(mat, p)

        # Prepare for new shader
        mat.ClearPoserSurface(keepTrans=True, keepBump=True, use_root=n000, backup=False)
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))
        mat.SetPSValue('Bump', bmp)

        # Build nodes
        n001 = mat.GetMap('color')
        n002 = mat.AddNode('scatter', name='Scatter', loc=(500, 110), collapsed=False, preview=False, Material=9.000, Use_Material_Color=0.000, Texture_Detail=0.300, scale=p('mouth_sss_scale'), maxError=1.0 - p('mouth_sss_quality'), Color=p('mouth_colour'), Scatter_Group=mat.Scattergroup(), )

        if kind == 'tongue':
            n004 = mat.AddNode('aniso', name='Anisotropic', loc=(789, 469), collapsed=False, preview=True, Specular_Color=(1.000, 1.000, 1.000), Specular_Value=ks, u_Roughness=p('mouth_spec_roughness'), v_Roughness=p('mouth_spec_roughness'), Rotation=0.000, Normals_Forward=0.000, )
            n005 = None
        else:
            n004 = mat.AddNode('ks_microfacet', name='ks_microfacet', loc=(250, 488), collapsed=False, preview=False, Specular_Color=(1.000, 1.000, 1.000), roughness=p('mouth_spec_roughness'), Ks=ks, )
            n005 = mat.AddNode('fresnel_blend', name='Fresnel_Blend', loc=(250, 587), collapsed=False, preview=False, Outer_Color=(1.000, 1.000, 1.000), Inner_Color=(0.000, 0.000, 0.000), Index_of_Refraction=p('mouth_fresnel_ior'), )
        n006 = mat.AddNode('blender', name='Blender', loc=(250, 290), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )

        if kind == 'gums':
            n007 = mat.AddNode('math_functions', name='Math_Functions', loc=(500, 373), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
            n008 = mat.AddNode('math_functions', name='Math_Functions_2', loc=(500, 325), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
            n009 = mat.AddNode('turbulence', name='Procedural bump', loc=(500, 277), collapsed=True, preview=False, x_Index=0.100, y_Index=0.100, z_Index=0.100, Octaves=3.000, Bottom=0.000, Bias=0.800, Gain=0.500, Noise_Type=2.000, )
        else:
            n009 = mat.AddNode('turbulence', name='Procedural bump', loc=(500, 277), collapsed=False, preview=False, x_Index=0.030, y_Index=0.030, z_Index=0.030, Octaves=8.000, Bottom=0.000, Bias=0.500, Gain=0.150, Noise_Type=2.000, )

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(752, 11), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(752, 11), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Connect nodes
        mat.ConnectNodes(n001, n000, 'Diffuse_Color')
        mat.ConnectNodes(n001, n002, 'Color')
        mat.ConnectNodes(n002, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n004, n000, 'AlternateSpecular')
        if n005: mat.ConnectNodes(n005, n004, 'Ks')
        mat.ConnectNodes(n006, n000, 'Bump')

        if kind == 'gums':
            mat.ConnectNodes(n007, n006, 'Input_1')
            mat.ConnectNodes(n008, n007, 'Value_1')
            mat.ConnectNodes(n009, n008, 'Value_1')
        else:
            mat.ConnectNodes(n009, n006, 'Input_1')

    def MakeNails(self, mat, p):

        # Initialise
        bvalue = 0.005
        n000 = self._PrepareRoot(mat, p)

        # Prepare for new shader
        mat.ClearPoserSurface(keepTrans=True, keepBump=True, use_root=n000, backup=False)
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))

        # Build nodes
        n001 = mat.GetMap('color')
        n002 = mat.AddNode('glossy', name='Glossy', loc=(530, 551), collapsed=False, preview=False, Specular_Color=(1.000, 1.000, 1.000), Ks=p('nail_spec_strength'), Roughness=p('nail_spec_size'), Sharpness=0.100, Normals_Forward=0.000, )
        n004 = mat.AddNode('scatter', name='EZScatter', loc=(810, 447), collapsed=False, preview=False, Material=p('nail_sss_type') + 1, Use_Material_Color=0.000, Texture_Detail=1.000, scale=p('nail_sss_scale'), maxError=1.0 - p('nail_sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        n005 = mat.AddNode('blender', name='Specular blend', loc=(810, 724), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n006 = mat.AddNode('math_functions', name='Proc. Specular Map', loc=(1090, 567), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.200, )
        n007 = mat.AddNode('math_functions', name='PSMap Add', loc=(1090, 615), collapsed=True, preview=False, Math_Argument=1.000, Value_1=0.500, Value_2=0.500, )
        n008 = mat.AddNode('turbulence', name='PSMap Turbulence', loc=(1090, 663), collapsed=True, preview=False, x_Index=0.060, y_Index=0.060, z_Index=0.060, Octaves=4.000, Bottom=0.000, Bias=0.500, Gain=0.500, Noise_Type=2.000, )
        n009 = mat.AddNode('math_functions', name='Subtract', loc=(1090, 475), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n010 = mat.AddNode('comp', name='Comp', loc=(1090, 427), collapsed=True, preview=False, Component=0.000, Color=(1.000, 1.000, 1.000), Point=(0.000, 0.000, 0.000), )
        n011 = mat.AddNode('hsv2', name='HSV', loc=(1090, 311), collapsed=False, preview=False, Color=p('HSV_colour'), Hue=p('HSV_H'), Saturation=p('HSV_S'), Value=p('HSV_V'), )
        n012 = mat.AddNode('blender', name='EZBump', loc=(250, 281), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n013 = mat.AddNode('math_functions', name='PBMultiply', loc=(530, 375), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
        n014 = mat.AddNode('math_functions', name='PBSubtract', loc=(530, 327), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n015 = mat.AddNode('turbulence', name='Procedural bump', loc=(530, 279), collapsed=True, preview=False, x_Index=0.100, y_Index=0.100, z_Index=0.100, Octaves=3.000, Bottom=0.000, Bias=0.800, Gain=0.500, Noise_Type=2.000, )
        n016 = mat.AddNode('simple_color', name='Diffuse Color', loc=(1090, 63), collapsed=True, preview=True, Color=p('nail_colour'), )

        # Colour maps

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(1090, 63), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(1090, 63), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Specular
        n024 = mat.GetMap('specular')
        if n024:
            mat.ConnectToPS(n024, 'Highlight_Value')
            mat.ConnectNodes(n024, n005, 'Input_2')
            if not mat.SameAsKdTexture(n024):
                mat.ConfigureNode(n024, loc=(1090, 711), name="Specular Map", showPreview=True, showInputs=False)

        # Bump
        n025 = mat.GetMap('bump')
        if n025:
            mat.ConfigureNode(n025, loc=(250, 428), name='Bump map', showPreview=True, showInputs=False)
            n026 = mat.AddNode('math_functions', loc=(250, 380), name='Bump map mod', Math_Argument=3.0, Value_1 = p('bump_mod'), Value_2=1.0)
            mat.ConnectNodes(n025, n026, "Value_1")
            mat.ConnectNodes(n026, n012, 'Input_2')
        else:
            mat.SetPSValue('Bump', bvalue)

        # Normal
        n027 = mat.GetMap('normal')
        if n027:
            mat.ConfigureNode(n027, loc=(250, 676), name='Normal map', showPreview=True, showInputs=False)
            mat.ConnectNodes(n027, n000, 'Gradient_Bump')

        # Connect nodes
        if p('nail_usecolour'):
            mat.ConnectNodes(n016, n010, 'Color')
            mat.ConnectNodes(n016, n011, 'Color')
        else:
            mat.ConnectNodes(n001, n010, 'Color')
            mat.ConnectNodes(n001, n011, 'Color')

        # Connect nodes
        mat.ConnectNodes(n002, n000, 'AlternateSpecular')
        mat.ConnectNodes(n004, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n005, n002, 'Ks')
        mat.ConnectNodes(n006, n005, 'Input_1')
        mat.ConnectNodes(n007, n006, 'Value_2')
        mat.ConnectNodes(n008, n007, 'Value_2')
        mat.ConnectNodes(n009, n004, 'Texture_Detail')
        mat.ConnectNodes(n010, n006, 'Value_1')
        mat.ConnectNodes(n010, n009, 'Value_2')
        mat.ConnectNodes(n011, n004, 'Color')
        mat.ConnectNodes(n012, n000, 'Bump')
        mat.ConnectNodes(n013, n012, 'Input_1')
        mat.ConnectNodes(n014, n013, 'Value_1')
        mat.ConnectNodes(n015, n014, 'Value_1')

        # APPLY USER SETTINGS

        # FRESNEL REFLECTION
        mat.SetPSValue('Reflection_Color', (1.000, 1.000, 1.000))
        mat.SetPSValue('Reflection_Value', 1.000)

        # BUMP

        b = p('bump_opt')                                        # Get the user's selections ...
        basebmp = bvalue * p('bump_mod')
        bump = mat.GetMap('bump') is not None                     # Is there a bump map?
        bmpBlend = (1, 0)[(not bump and b == 1) or (b == 2)]     # Determine the blending value

        bv = mat.PSValue('Bump')                                # The bump value
        if bv == 0: bv = bvalue                                    # Avoid division by zero

        procbmpMult = basebmp / bv                                 # Procedural bump modifier

        # Determine blender's Bump map colour chip value
        if not bump and b == 0:
            r, g, b = (0,0,0)
        else:
            r, g, b = (1,1,1)

        # Configure procedural nodes
        n012.InputByInternalName('Input_2').SetColor(r, g, b)
        n012.InputByInternalName('Blending').SetFloat(bmpBlend)
        n013.InputByInternalName('Value_2').SetFloat(procbmpMult)

        # Configure bump map nodes
        if n025:
            n026.InputByInternalName('Value_2').SetFloat((1.0, p('bump_value') / bv)[p('bump_usecustom')])

        # SPECULAR

        # Is there a specular map?
        spec = n024 is not None

        # Get the user's preference
        s = p('spec_opt')

        # Get the blending value
        bl = (1, 0)[(not spec and s == 1) or (s == 2)]

        # Determine whether or not there will be any specular mapping
        if not spec and s == 0:
            # Delete the blender node
            n005.Delete()
        else:
            # Update the blender node
            n005.InputByInternalName('Blending').SetFloat(bl)

    def MakePupil(self, mat, p):
        # Shader shared with Iris :-)
        self.MakeIris(mat, p, kind='pupil')

    def MakeSclera(self, mat, p):

        # Initialise
        pbv = 0.002        # Procedural bump value
        n000 = self._PrepareRoot(mat, p)

        # Prepare for new shader
        mat.ClearPoserSurface(keepTrans=True, keepBump=True, use_root=n000, backup=False)
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))

        # Build nodes
        n001 = mat.GetMap('color')
        n002 = mat.AddNode('scatter', name='Scatter', loc=(300, 459), collapsed=False, preview=False, Material=p('sf_sclera_sss_type') + 1, Use_Material_Color=0.000, Texture_Detail=1.000, scale=p('sf_sclera_sss_scale'), maxError=1.0 - p('sf_sclera_sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        n003 = mat.AddNode('specular', name='Specular', loc=(300, 626), collapsed=False, preview=False, Specular_Color=(1.000, 1.000, 1.000), Specular_Value=p('sf_sclera_specularity'), Roughness=p('sf_sclera_roughness'), Normals_Forward=0.000, )
        n004 = mat.AddNode('hsv2', name='hsv', loc=(580, 486), collapsed=False, preview=False, Color=p('sclera_HSV_colour'), Hue=p('sclera_HSV_H'), Saturation=p('sclera_HSV_S'), Value=p('sclera_HSV_V'), )
        n012 = mat.AddNode('blender', name='EZBump', loc=(300, 112), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n013 = mat.AddNode('math_functions', name='PBMultiply', loc=(580, 160), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
        n014 = mat.AddNode('math_functions', name='PBSubtract', loc=(580, 112), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n015 = mat.AddNode('turbulence', name='Procedural bump', loc=(580, 64), collapsed=True, preview=False, x_Index=0.100, y_Index=0.100, z_Index=0.100, Octaves=3.000, Bottom=0.000, Bias=0.800, Gain=0.500, Noise_Type=2.000, )

        # Colour maps

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(580, 238), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(580, 238), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Connect nodes
        mat.ConnectNodes(n002, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n003, n000, 'AlternateSpecular')
        mat.ConnectNodes(n004, n003, 'Specular_Color')
        mat.ConnectNodes(n001, n004, 'Color')
        mat.ConnectNodes(n004, n002, 'Color')
        mat.ConnectNodes(n012, n000, 'Bump')
        mat.ConnectNodes(n013, n012, 'Input_1')
        mat.ConnectNodes(n014, n013, 'Value_1')
        mat.ConnectNodes(n015, n014, 'Value_1')

        # APPLY USER SETTINGS

        # BUMP

        n005 = mat.GetMap('bump')
        if n005:
            mat.ConfigureNode(n005, loc=(300, 211), name='Bump map', showPreview=True, showInputs=False)
            mat.ConnectNodes(n005, n012, "Input_2")
        else:
            mat.SetPSValue('Bump', pbv)

        b = p('eye_bump_opt')                                    # Get the user's selections ...
        basebmp = pbv * p('bump_mod')
        bump = mat.GetMap('bump') is not None                     # Is there a bump map?
        bmpBlend = (1, 0)[(not bump and b == 1) or (b == 2)]     # Determine the blending value

        bv = mat.PSValue('Bump')                                # The bump value
        if bv == 0: bv = pbv                                    # Avoid division by zero

        procbmpMult = basebmp / bv                                 # Procedural bump modifier

        # Determine blender's Bump map colour chip value
        if not bump and b == 0:
            r, g, b = (0,0,0)
        else:
            r, g, b = (1,1,1)

        # Configure procedural nodes
        n012.InputByInternalName('Input_2').SetColor(r, g, b)
        n012.InputByInternalName('Blending').SetFloat(bmpBlend)
        n013.InputByInternalName('Value_2').SetFloat(procbmpMult)

    def MakeSkin(self, mat, p):

        # Initialise
        bvalue = 0.005
        n000 = self._PrepareRoot(mat, p)

        # Prepare for new shader
        mat.ClearPoserSurface(keepTrans=True, keepBump=True, use_root=n000, backup=False)
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))

        # Build nodes
        n001 = mat.GetMap('color')
        n003 = mat.AddNode('blinn', name='EZBlinn', loc=(530, 551), collapsed=False, preview=False, Specular_Color=(1.000, 1.000, 1.000), Eccentricity=p('sf_spec_eccentricity'), SpecularRollOff=p('sf_spec_rolloff'), Reflectivity=p('sf_spec_reflectivity'), Normals_Forward=0.000, )
        n004 = mat.AddNode('scatter', name='EZScatter', loc=(810, 447), collapsed=False, preview=False, Material=p('sss_type') + 1, Use_Material_Color=0.000, Texture_Detail=1.000, scale=p('sss_scale'), maxError=1.0 - p('sss_quality'), Color=(1.000, 1.000, 1.000), Scatter_Group=mat.Scattergroup(), )
        n005 = mat.AddNode('blender', name='Specular blend', loc=(810, 724), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n006 = mat.AddNode('math_functions', name='Proc. Specular Map', loc=(1090, 567), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.200, )
        n007 = mat.AddNode('math_functions', name='PSMap Add', loc=(1090, 615), collapsed=True, preview=False, Math_Argument=1.000, Value_1=0.500, Value_2=0.500, )
        n008 = mat.AddNode('turbulence', name='PSMap Turbulence', loc=(1090, 663), collapsed=True, preview=False, x_Index=0.060, y_Index=0.060, z_Index=0.060, Octaves=4.000, Bottom=0.000, Bias=0.500, Gain=0.500, Noise_Type=2.000, )
        n009 = mat.AddNode('math_functions', name='Subtract', loc=(1090, 475), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n010 = mat.AddNode('comp', name='Comp', loc=(1090, 427), collapsed=True, preview=False, Component=0.000, Color=(1.000, 1.000, 1.000), Point=(0.000, 0.000, 0.000), )
        n011 = mat.AddNode('hsv2', name='HSV', loc=(1090, 311), collapsed=False, preview=False, Color=p('HSV_colour'), Hue=p('HSV_H'), Saturation=p('HSV_S'), Value=p('HSV_V'), )
        n012 = mat.AddNode('blender', name='EZBump', loc=(250, 281), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n013 = mat.AddNode('math_functions', name='PBMultiply', loc=(530, 375), collapsed=True, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
        n014 = mat.AddNode('math_functions', name='PBSubtract', loc=(530, 327), collapsed=True, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=0.500, )
        n015 = mat.AddNode('turbulence', name='Procedural bump', loc=(530, 279), collapsed=True, preview=False, x_Index=0.100, y_Index=0.100, z_Index=0.100, Octaves=3.000, Bottom=0.000, Bias=0.800, Gain=0.500, Noise_Type=2.000, )

        # Colour maps

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(1090, 63), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(1090, 63), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Specular
        n024 = mat.GetMap('specular')
        if n024:
            mat.ConnectToPS(n024, 'Highlight_Value')
            mat.ConnectNodes(n024, n005, 'Input_2')
            if not mat.SameAsKdTexture(n024):
                mat.ConfigureNode(n024, loc=(1090, 711), name="Specular Map", showPreview=True, showInputs=False)

        # Bump
        n025 = mat.GetMap('bump')
        if n025:
            mat.ConfigureNode(n025, loc=(250, 428), name='Bump map', showPreview=True, showInputs=False)
            n026 = mat.AddNode('math_functions', loc=(250, 380), name='Bump map mod', Math_Argument=3.0, Value_1=p('bump_mod'), Value_2=1.0)
            mat.ConnectNodes(n025, n026, "Value_1")
            mat.ConnectNodes(n026, n012, 'Input_2')
        else:
            mat.SetPSValue('Bump', bvalue)

        # Normal
        n027 = mat.GetMap('normal')
        if n027:
            mat.ConfigureNode(n027, loc=(250, 676), name='Normal map', showPreview=True, showInputs=False)
            mat.ConnectNodes(n027, n000, 'Gradient_Bump')

        # Connect nodes
        mat.ConnectNodes(n001, n010, 'Color')
        mat.ConnectNodes(n001, n011, 'Color')
        mat.ConnectNodes(n003, n000, 'AlternateSpecular')
        mat.ConnectNodes(n004, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n005, n003, 'Specular_Color')
        mat.ConnectNodes(n006, n005, 'Input_1')
        mat.ConnectNodes(n007, n006, 'Value_2')
        mat.ConnectNodes(n008, n007, 'Value_2')
        mat.ConnectNodes(n009, n004, 'Texture_Detail')
        mat.ConnectNodes(n010, n006, 'Value_1')
        mat.ConnectNodes(n010, n009, 'Value_2')
        mat.ConnectNodes(n011, n004, 'Color')
        mat.ConnectNodes(n012, n000, 'Bump')
        mat.ConnectNodes(n013, n012, 'Input_1')
        mat.ConnectNodes(n014, n013, 'Value_1')
        mat.ConnectNodes(n015, n014, 'Value_1')

        # APPLY USER SETTINGS

        # BUMP

        b = p('bump_opt')                                        # Get the user's selections ...
        basebmp = bvalue * p('bump_mod')
        bump = mat.GetMap('bump') is not None                     # Is there a bump map?
        bmpBlend = (1, 0)[(not bump and b == 1) or (b == 2)]     # Determine the blending value

        bv = mat.PSValue('Bump')                                # The bump value
        if bv == 0: bv = bvalue                                    # Avoid division by zero

        procbmpMult = basebmp / bv                                 # Procedural bump modifier

        # Determine blender's Bump map colour chip value
        if not bump and b == 0:
            r, g, b = (0,0,0)
        else:
            r, g, b = (1,1,1)

        # Configure procedural nodes
        n012.InputByInternalName('Input_2').SetColor(r, g, b)
        n012.InputByInternalName('Blending').SetFloat(bmpBlend)
        n013.InputByInternalName('Value_2').SetFloat(procbmpMult)

        # Configure bump map nodes
        if n025:
            n026.InputByInternalName('Value_2').SetFloat((1.0, p('bump_value') / bv)[p('bump_usecustom')])

        # SPECULAR

        # Is there a specular map?
        spec = n024 is not None

        # Get the user's preference
        s = p('spec_opt')

        # Get the blending value
        bl = (1, 0)[(not spec and s == 1) or (s == 2)]

        # Determine whether or not there will be any specular mapping
        if not spec and s == 0:
            # Delete the blender node
            n005.Delete()
        else:
            # Update the blender node
            n005.InputByInternalName('Blending').SetFloat(bl)

    def MakeSpecular(self, mat, p):

        # Initialise
        n000 = self._PrepareRoot(mat, p)

        # Prepare for new shader
        mat.ClearPoserSurface(keepTrans=False, keepBump=False, use_root=n000, backup=False)

        # And configure it
        mat.SetPSValue('Transparency_Max', 1.0)
        n01 = mat.AddNode('aniso', loc=(300,400), collapsed=False)
        mat.ConnectToPS(n01, 'AlternateSpecular', value=(1.0, 1.0, 1.0))

    def MakeTeeth(self, mat, p):

        # Initialise
        bvalue = 0.005
        n000 = self._PrepareRoot(mat, p)

        # Prepare for new shader
        mat.ClearPoserSurface(keepTrans=True, keepBump=True, use_root=n000, backup=False)
        mat.SetPSValue('Diffuse_Color', mat.GetPSBackup('Diffuse_Color'))
        mat.SetPSValue('AlternateDiffuse', (1.000, 1.000, 1.000))
        mat.SetPSValue('AlternateSpecular', (1.000, 1.000, 1.000))

        # Build nodes
        n001 = mat.GetMap('color')
        n003 = mat.AddNode('skin2', name='Subsurface Skin', loc=(580, 584), collapsed=False, preview=False, Color=p('sf_teeth_colour'), Texture_Detail=0.650, Specularity=p('sf_teeth_specularity'), HighlightSize=p('sf_teeth_highlight'), maxError=1.0 - p('teeth_sss_quality'), Scatter_Group=mat.Scattergroup(), )
        n006 = mat.AddNode('blinn', name='Blinn', loc=(580, 734), collapsed=False, preview=False, Specular_Color=p('sf_teeth_spec_colour'), Eccentricity=p('sf_teeth_eccentricity'), SpecularRollOff=p('sf_teeth_rolloff'), Reflectivity=p('sf_teeth_reflectivity'), Normals_Forward=0.000, )
        n012 = mat.AddNode('blender', name='EZBump', loc=(300, 145), collapsed=False, preview=False, Input_1=(1.000, 1.000, 1.000), Input_2=(1.000, 1.000, 1.000), Blending=0.000, )
        n013 = mat.AddNode('math_functions', name='PBMultiply', loc=(580, 194), collapsed=False, preview=False, Math_Argument=3.000, Value_1=1.000, Value_2=1.000, )
        n015 = mat.AddNode('fbm', name='fBm', loc=(580, 146), collapsed=True, preview=False, Signed=1.000, Fractal_Increment=0.500, Frequency_Gap=2.186, Octaves=8.000, x_Index=0.050, y_Index=0.200, z_Index=0.050, Bottom=0.000, Bias=0.500, Gain=0.500, Noise_Type=1.000, Offset_X=0.000, Offset_Y=0.000, Offset_Z=0.000, )

        # Colour maps

        # Diffuse
        if n001:
            mat.ConfigureNode(n001, loc=(580, 336), name='Diffuse color', showPreview=True, showInputs=False)
            mat.ConnectToPS(n001, 'Diffuse_Color')
        else:
            n001 = mat.AddNode('simple_color', name='Diffuse Color', loc=(580, 336), collapsed=True, preview=True, Color=mat.GetPSBackup('Diffuse_Color'), )

        # Specular
        n024 = mat.GetMap('specular')
        if n024: mat.ConnectToPS(n024, 'Highlight_Value')

        # Bump
        n025 = mat.GetMap('bump')
        if n025:
            mat.ConfigureNode(n025, loc=(300, 292), name='Bump map', showPreview=True, showInputs=False)
            n026 = mat.AddNode('math_functions', loc=(300, 244), name='Bump map mod', Math_Argument=3.0, Value_2=1.0)
            mat.ConnectNodes(n025, n026, "Value_1")
            mat.ConnectNodes(n026, n012, 'Input_2')
        else:
            mat.SetPSValue('Bump', bvalue)

        # Normal
        n027 = mat.GetMap('normal')
        if n027:
            mat.ConfigureNode(n027, loc=(300, 540), name='Normal map', showPreview=True, showInputs=False)
            mat.ConnectNodes(n027, n000, 'Gradient_Bump')

        # Connect nodes
        mat.ConnectNodes(n001, n003, 'Color')
        mat.ConnectNodes(n003, n000, 'AlternateDiffuse')
        mat.ConnectNodes(n006, n000, 'AlternateSpecular')
        mat.ConnectNodes(n012, n000, 'Bump')
        mat.ConnectNodes(n013, n012, 'Input_1')
        mat.ConnectNodes(n015, n013, 'Value_1')

        # BUMP

        b = p('bump_opt')                                        # Get the user's selections ...
        basebmp = bvalue * p('bump_mod')
        bump = mat.GetMap('bump') is not None                     # Is there a bump map?
        bmpBlend = (1, 0)[(not bump and b == 1) or (b == 2)]     # Determine the blending value

        bv = mat.PSValue('Bump')                                # The bump value
        if bv == 0: bv = bvalue                                    # Avoid division by zero

        procbmpMult = basebmp / bv                                 # Procedural bump modifier

        # Determine blender's Bump map colour chip value
        if not bump and b == 0:
            r, g, b = (0,0,0)
        else:
            r, g, b = (1,1,1)

        # Configure procedural nodes
        n012.InputByInternalName('Input_2').SetColor(r, g, b)
        n012.InputByInternalName('Blending').SetFloat(bmpBlend)
        n013.InputByInternalName('Value_2').SetFloat(procbmpMult)

        # Configure bump map nodes
        if n025:
            n026.InputByInternalName('Value_2').SetFloat((1.0, p('bump_value') / bv)[p('bump_usecustom')])

    def MakeTongue(self, mat, p):
        # Gums and Tongue share a similar shader
        self.MakeMouth(mat, p, kind='tongue')

    def Renderer(self):
        return 'Superfly'

# MATHELPER
class MatHelper(object):

    def __init__(self, material, group=1):

        # Initialise
        self.material = material
        self.scattergroup = group
        self.tree = material.ShaderTree()
        self.roots = self.GetRootNodes()
        self.current_PS = self.FindPSRoot()
        self.mapnodes = {}                    # Dictionary of Imagemap nodes found in shader
        self.backup = {}                    # Copy of all PoserSurface values

        # Get Imagemap nodes
        if self.Root(): self.GetMapNodes()

    def AddNode(self, nodetype, name=None, loc=None, collapsed=True, preview=False, inputs=None, **kw):
        # Add a ShaderNode to the shader

        # Create the node
        node = self.tree.CreateNode(nodetype)

        # Configure it
        if loc: node.SetLocation(loc[0], loc[1])
        if name: node.SetName(name)
        node.SetInputsCollapsed(collapsed)
        node.SetPreviewCollapsed(preview)

        # Set values of any Inputs passed
        if inputs:
            for key, value in list(inputs.items()):
                inp = node.InputByInternalName(key)
                self.SetInputValue(inp, value)

        for key, value in list(kw.items()):
            inp = node.InputByInternalName(key)
            self.SetInputValue(inp, value)

        # Return a reference to the node
        return node

    def AddLayer(self, shader, mask, override=False, bump=None, displacement=None, title='New layer', maxdisp=True, connect_diffuse=True):

        # Initialise
        inp = self.Root().InputByInternalName
        ad = inp('AlternateDiffuse')

        # Add the main shader
        if shader:
            main = self.InsertBlender(ad, loc=(300, 200), name=title, connect=shader, mask=mask)

        # Connect to Diffuse channel to for Preview purposes
        if shader and connect_diffuse:
            self.ConnectToPS(main, 'Diffuse_Color', (1,1,1))
            self.SetPSValue('Diffuse_Value', 0.0)

        # Displacement
        if displacement: self.InsertAndAdd(inp('Displacement'), displacement[1], displacement[0], maxdisp)

        # Bump
        if bump: self.InsertAndAdd(inp('Bump'), bump[1], bump[0], maxdisp)

        # Transparency
        if override:

            target = inp('Transparency_Max')

            if target.Value() > 0:

                # Make a joiney up thing
                t1 = self.AddNode('color_math', Math_Argument=16, Value_1=(1,1,1), Value_2=(1,1,1))
                old = target.InNode()

                # Connect the old transparency settings
                if old:
                    old.ConnectToInput(self.InputByName(t1, 'Value 1'))
                else:
                    t1.InputByInternalName('Value_1').SetColor(0,0,0)

                # Add our mask into the mix
                x = target.Value()
                adj = 1.0 / x

                a = self.AddNode('math_functions', Math_Argument=1.0, Value_1=adj, Value_2=0.0)
                mask.ConnectToInput(self.InputByName(a, 'Value_1'))
                a.ConnectToInput(self.InputByName(t1, 'Value 2'))

                # And plug it all into the PoserSurface
                t1.ConnectToInput(target)

    def AlignNodes(self, root, left=300, separation=280, cols=False, moveroot=False):

        # We need a starting node
        if root is None: return

        # Determine leftmost starting point
        if moveroot == True:
            leftx = left
        else:
            leftx = root.Location()[0] + 300

        # Get a dictionary of all nodes
        allnodes = self.AllNodes()

        # Get all nodes connected to our root

        # Node names > nstr
        nstr = []
        self.Traverse(root, nstr, use_ext=True)

        # Actual node objects > nodes
        nodes = [allnodes[x] for x in nstr]

        # Reposition the root node
        if moveroot: root.SetLocation(10,10)

        if cols:

            dummy = []
            for i in root.Inputs():
                n = i.InNode()
                if n:
                    self.Traverse(n, dummy, xy=(leftx, separation), use_ext=True)

        else:

            base = 1000000

            for node in nodes:

                if node == root: continue

                x = node.Location()[0]
                base = min(base, x)

            base_mod = base - leftx

            for node in nodes:

                if node == root:
                    continue

                x, y = node.Location()
                x -= base_mod

                if x < leftx:
                    newx = leftx
                else:
                    nx = x - leftx
                    halfway = separation / 2
                    column = int((nx + halfway) / separation)
                    newx = leftx + (separation * column)

                node.SetLocation(newx, y)

        # Now set the Y locations ...

        columns = {}

        # Assign all nodes with the same x co-ordinate to the same list
        for n in nodes:
            # get the X co-ordinate
            x = n.Location()[0]

            # Begin a new list if necessary
            if columns.get(x, None) == None: columns[x] = []

            # Add the node to the appropriate list
            columns[x].append(n)

        # Now process each list in turn
        for l in list(columns.values()):

            # Sort in Y co-ordinate order
            l.sort(None, self.AlignNodesSort)

            # Calculate the Y co-ordinate of each node
            next_y = l[0].Location()[1]

            if next_y < 0: next_y = 10

            for node in l:

                # Relocate if necessary
                x,y  = node.Location()
                if y < next_y + 10:
                    node.SetLocation(x, next_y)

                # Calculate next y place
                height = self.AlignNodesSize(node)
                next_y = node.Location()[1] + height + 6

        self.UpdatePreview()

    def AlignNodesSize(self, n):
        height = 42
        if not n.InputsCollapsed(): height += n.NumInputs() * 17
        if n.PreviewCollapsed(): height += 200
        return height

    def AlignNodesSort(self, n):
        return n.Location()[1]

    def AllNodes(self):
        return {n.Name():n for n in self.tree.Nodes()}

    def ApplyTextureFilters(self, value):
        # Sets Texture Filtering for all identified imagemaps
        for map in list(self.mapnodes.values()):
            if map:
                map.InputByInternalName('Filtering').SetFloat(value)

    def ClearPoserSurface(self, keepTrans=False, keepBump=False, removenodes=False, disconnect=True, use_root=None, backup=True):

        # Determine which root node to use
        ps = use_root or self.Root()

        if ps:

            # Make a list of exceptions
            exceptions = []
            if keepTrans:
                exceptions.append(7)    # Transparency
                exceptions.append(8)    # Transparency_Edge
                exceptions.append(9)    # Transparency_FallOff
            if keepBump:
                exceptions.append(16)    # Bump
                exceptions.append(17)    # Displacement
                exceptions.append(22)    # Gradient bump
                exceptions.append(23)    # Gradient mode

            # Reset (nearly) all inputs
            for x in range(24):

                i = ps.Input(x)
                if backup: self.backup[i.InternalName()] = i.Value()

                # Ignore the exceptions
                if x in exceptions: continue

                # Disconnect any attached nodes
                if disconnect: i.Disconnect()

                # Reset the Input's value
                if i.Type() in [0, 4, 5, 6]:
                    i.SetFloat(0.0)
                else:
                    i.SetColor(0.0, 0.0, 0.0)

            # Remove orphaned nodes if requested
            if removenodes:
                self.RemoveOrphans()

            # Standardise appearance of PoserSurface
            self.ConfigureNode(self.Root(), loc=(10,10), showInputs=True, showPreview=True)

    def ConnectToPS(self, node, inp, value=None):
        # Connect Shader Node 'node' to PoserSurface Input 'inp', optionally setting the value too
        i = self.Root().InputByInternalName(inp)
        node.ConnectToInput(i)
        if value:
            self.SetInputValue(i, value)

    def ConsolidateShader(self, pilehere=None, useSF=False):

        # Ensure we have a valid PoserSurface root node
        if not self.Root(): return

        # For convenience...
        inp = self.PSInput

        # Obtain PoserSurface settings
        old_diffuse = self.MakeDiffuseFromPS(clear=True)
        old_specular = self.MakeSpecularFromPS(clear=True)
        old_refract = self.MakeRefractFromPS(clear=True)
        old_reflect = self.MakeReflectFromPS(clear=True)
        old_ambient = self.MakeAmbientFromPS(inp('Ambient_Color'), inp('Ambient_Value'), 'Old Ambient', clear=True)
        old_lucy = self.MakeAmbientFromPS(inp('Translucence_Color'), inp('Translucence_Value'), 'Old Lucy', clear=True)
        old_altspec = self.MakeAltSpecularFromPS(clear=True, useSF=useSF)

        # Prepare to aggregate PoserSurface settings
        if useSF:
            old = self.AddNode('ccl_AddClosure', loc=(250, 400), collapsed=False, preview=False, )
            inpname1 = 'Closure1'
            inpname2 = 'Closure2'
        else:
            old = self.AddNode('color_math', collapsed=False, Math_Argument=1.0, loc=(250, 400), Value_1=(0,0,0), Value_2=(0,0,0))
            inpname1 = 'Value_1'
            inpname2 = 'Value_2'

        # A list of all nodes to be aggregated
        ToDo = [x for x in [old_refract, old_reflect, old_ambient, old_lucy, old_altspec, old_diffuse, old_specular] if x]

        # The first one is a special case
        if len(ToDo) > 0:
            self.ConnectNodes(ToDo.pop(), old, inpname1)
            if not useSF: self.SetInputValue(old.InputByInternalName(inpname1), (1,1,1))

        available = True # True = Value_2 is available for use

        while len(ToDo) > 0:
            n = ToDo.pop()
            if available:
                self.ConnectNodes(n, old, inpname2)
                if not useSF: self.SetInputValue(old.InputByInternalName(inpname2), (1,1,1))
                available = False
            else:
                if useSF:
                    new = self.AddNode('ccl_AddClosure', loc=(250, 400), collapsed=False, preview=False,)
                else:
                    new = self.AddNode('color_math', Math_Argument=1.0, Collapsed=False, loc=(250, 400), Value_1=(1,1,1), Value_2=(1,1,1))
                self.ConnectNodes(old, new, inpname1)
                self.ConnectNodes(n, new, inpname2)
                old = new

        # Grab the existing Alt_Diffuse nodes, if any
        n = inp('AlternateDiffuse').InNode()

        # Connect the base shader
        if n:
            if useSF:
                base = self.AddNode('ccl_AddClosure', loc=(250, 400), collapsed=False, preview=False, )

            else:
                base = self.AddNode('color_math', name='EZBase', collapsed=False, Math_Argument=1.0, loc=(250, 400), Value_1=(1,1,1), Value_2=(self.PSValue('AlternateDiffuse')))

            self.ConnectNodes(old, base, inpname1)
            self.ConnectNodes(n, base, inpname2)
        else:
            base = self.AddNode('simple_color', name='EZBase', loc=(250, 400), Color=(1,1,1))
            self.ConnectNodes(old, base, 'Color')

        # Connect to PoserSurface
        self.ConnectToPS(base, 'AlternateDiffuse', (1,1,1))
        self.ConnectToPS(base, 'Diffuse_Color', (1,1,1))
        self.SetPSValue('Diffuse_Value', 0.0)

        # Tidy up if requested
        if pilehere != None: self.PileUpHere(base, pilehere)

    def ConfigureNode(self, node, loc=None, name=None, showPreview=None, showInputs=None):
        # Updates an existing node's parameters

        if loc: node.SetLocation(loc[0], loc[1])
        if name: node.SetName(name)
        if showPreview != None: node.SetPreviewCollapsed((0, 1)[showPreview == True])
        if showInputs != None: node.SetInputsCollapsed((1, 0)[showInputs == True])

    def ConnectNodes(self, nodefrom, nodeto, inp, output=None):
        # Connects two nodes via the specified Input
        if output:
            nodefrom.OutputByInternalName(output).ConnectToInput(nodeto.InputByInternalName(inp))
        else:
            try:
                nodefrom.ConnectToInput(nodeto.InputByInternalName(inp))
            except:
                if DEBUG: print(f"Error in connecting nodefrom {nodefrom.Name()} to nodeto {nodeto.Name()} inp is: {inp}")

    def FindAKindOfNode(self, node, nodetype):
        # Finds a node of nodetype 'nodetype' starting from ShaderNode 'node'

        # Make sure we have a node from which to begin the search!
        if not node:
            return None

        # Maybe we are starting with the answer
        if node.Type() == nodetype:
            return node

        # Otherwise, check each input and recursively search for the right node
        for inp in node.Inputs():
            v = inp.InNode()
            found = self.FindAKindOfNode(v, nodetype)
            if found: return found

        # Out of luck :-(
        return None

    def FindMapNode(self, inp):
        # Searches for an Imagemap node in the shader tree, starting from ShaderInput 'inp'

        node = inp.InNode()
        mapnode = self.FindAKindOfNode(node, poser.kNodeTypeCodeIMAGEMAP)
        return mapnode

    def FindPSRoot(self, renderer='firefly', roottype=poser.kNodeTypeCodePOSERSURFACE):

        # Get the PoserSurface node for this shader's Firefly renderer
        node = self.roots.get(renderer, None)

        # Check if only certain type of root is wanted
        if node is not None:
            if roottype != 'any' and node.Type() != roottype:
                node = None

        return node

    def GC(self, node, data):
        # Set gamma correction values for an imagemap node

        if node:
            if node.Type() == poser.kNodeTypeCodeIMAGEMAP:
                texture = node.InputByInternalName('Image_Source').Texture()
                if texture:
                    if not data:
                        texture.SetUseSceneGamma(1)
                    else:
                        kd = self.GetMap('color')
                        if kd:
                            if node.InputByInternalName('Image_Source').Value() != kd.InputByInternalName('Image_Source').Value():
                                texture.SetGamma(1.0)
                        else:
                            texture.SetGamma(1.0)

    def GC_AllMaps(self):
        # Set gamma correction values for all imagemap nodes

        for channel in ('specular', 'bump', 'displacement', 'transparency', 'normal'):
            self.GC(self.GetMap(channel), data=True)

        for channel in ('color', 'alt_color', 'alt_specular', 'reflection', 'specular_color'):
            self.GC(self.GetMap(channel), data=False)

    def GetConnectedNodes(self, node, usedNodes):
        usedNodes.append(node)
        for inp in node.Inputs():
            inNode = inp.InNode()
            if (inNode != None):
                self.GetConnectedNodes(inNode, usedNodes)

    def GetMap(self, mapname):
        # Returns an Imagemap node associated with the channel 'mapname', or None

        # Sanitise
        channel = mapname.lower()

        # Find the specified map
        map = self.mapnodes.get(channel, None)

        # If not found and channel is Diffuse or Specular, try the alt channels
        if channel == 'color' and map is None:
            map = self.mapnodes.get('alt_color', None)
        elif channel == 'specular' and map is None:
            map = self.mapnodes.get('alt_specular', None)

        return map

    def GetMapNodes(self):
        # Finds the Imagemap nodes associated with each channel

        channels = (
            ('color', 'Diffuse_Color'),
            ('alt_color', 'AlternateDiffuse'),
            ('specular', 'Highlight_Value'),
            ('alt_specular', 'AlternateSpecular'),
            ('bump', 'Bump'),
            ('displacement', 'Displacement'),
            ('transparency', 'Transparency_Max'),
            ('reflection', 'Reflection_Color'),
            ('specular_color', 'Highlight_Color'),
            ('normal', 'Gradient_Bump'),
            )

        for pair in channels:
            self.mapnodes[pair[0]] = self.FindMapNode(self.Root().InputByInternalName(pair[1]))

    def GetPSBackup(self, name):
        # Returns the saved value of a PoserSurface Shader Input
        return self.backup.get(name, None)

    def GetRootNodes(self):
        roots = {}
        roots['firefly'] = self.tree.RendererRootNode(poser.kRenderEngineCodeFIREFLY)
        roots['superfly'] = self.tree.RendererRootNode(poser.kRenderEngineCodeSUPERFLY)
        return roots

    def InsertAndAdd(self, target, value, connect, maxdisp=False):
        old = target.InNode()
        if old:
            # First we get the current value
            dispval = target.Value()
            if dispval == 0:
                target.SetFloat(value)
                dispamt = 1.0
            else:
                dispamt = value / dispval

            # Now we create a maths add node
            n004 = self.AddNode('math_functions', name='Math_Functions', loc=(300,100), collapsed=False, preview=False, Math_Argument=(1.000,16.0)[maxdisp], Value_1=1.000, Value_2=dispamt)

            # Now connect the old channel to the new Add node
            self.ConnectNodes(old, n004, 'Value_1')

            # Connect the Add node to the PoserSurface
            n004.ConnectToInput(target)

            # And finally connect our node to the Add node as well
            connect.ConnectToInput(n004.InputByInternalName('Value_2'))

        else:
            target.SetFloat(value)
            connect.ConnectToInput(target)

    def InsertBlender(self, target, input=1, colour=(1,1,1), value=1.0, loc=None, name=None, collapsed=False, connect=None, mask=None):

        # Calculate the position of the new node, if not specified
        if loc == None:
            x, y = target.ItsNode().Location()
            loc = (x+200,y+50)

        # Find the node currently attached to the target input
        originalNode = target.InNode()

        col = ((1,1,1), (0,0,0))[originalNode == None]

        if input == 0:
            n01 = self.AddNode('blender', name=name, loc=loc, collapsed=collapsed, Input_2=col, Input_1=colour, Blending=value)
            if originalNode:
                self.ConnectNodes(originalNode, n01, 'Input_2')
        else:
            n01 = self.AddNode('blender', name=name, loc=loc, collapsed=collapsed, Input_1=col, Input_2=colour, Blending=value)
            if originalNode:
                self.ConnectNodes(originalNode, n01, 'Input_1')

        self.Tree().AttachTreeNodes(target.ItsNode(), target.InternalName(), n01)

        # Do some joiny up stuff
        if connect: self.ConnectNodes(connect, n01, ('Input_1', 'Input_2')[input])
        if mask: self.ConnectNodes(mask, n01, 'Blending')

        return n01

    def MakeBackup(self, root):
        for i in root.Inputs():
            self.backup[i.InternalName()] = i.Value()

    def MakeRoot(self, renderer, roottype):

        # Create the root node
        newroot = self.AddNode(roottype, loc=(10,10))

        # Set the new root node
        self.SetRenderer(renderer, newroot)

        return newroot

    def MakeAltSpecularFromPS(self, clear=True, useSF=False):

        # For convenience...
        inp = self.PSInput

        o = inp('AlternateSpecular').InNode()

        if o:
            if useSF:
                inp('AlternateSpecular').Disconnect()
                return o
            else:
                n = self.AddNode('simple_color', name='Old AltSpec', collapsed=False, Color=inp('AlternateSpecular').Value())
                self.ConnectNodes(o, n, 'Color')
                inp('AlternateSpecular').Disconnect()
                return n
        else:
            return None

    def MakeAmbientFromPS(self, col, val, name="New node", clear=True):

        if col.Value() == (0,0,0) or val.Value() == 0.0:
            return None
        else:
            n1 = self.AddNode('color_math', name=name, collapsed=False, preview=False, Math_Argument=3.0, Value_1=col.Value(), Value_2=(1,1,1))
            n2 = self.AddNode('math_functions', collapsed=False, Math_Argument=3.0, Value_1=val.Value(), Value_2=1)
            self.ConnectNodes(n2, n1, 'Value_2')

            x = col.InNode()
            if x: self.ConnectNodes(x, n1, 'Value_1')
            x = val.InNode()
            if x: self.ConnectNodes(x, n2, 'Value_2')

            if clear:
                col.SetColor(0,0,0)
                val.SetFloat(0.0)
                col.Disconnect()
                val.Disconnect()

            return n1

    def MakeDiffuseFromPS(self, clear=True):

        # For convenience...
        inp = self.PSInput

        if self.PSValue('Diffuse_Color') == (0,0,0) or self.PSValue('Diffuse_Value') == 0.0:
            return None
        else:
            n = self.AddNode('diffuse', name='Old Diffuse', collapsed=False, preview=False, Diffuse_Color=self.PSValue('Diffuse_Color'), Diffuse_Value=self.PSValue('Diffuse_Value'), Normals_Forward=0.000, )

            x = inp('Diffuse_Color').InNode()
            if x: self.ConnectNodes(x, n, 'Diffuse_Color')
            x = inp('Diffuse_Value').InNode()
            if x: self.ConnectNodes(x, n, 'Diffuse_Value')

            if clear:
                self.SetPSValue('Diffuse_Color', (0,0,0))
                self.SetPSValue('Diffuse_Value' , 0.0)
                inp('Diffuse_Color').Disconnect()
                inp('Diffuse_Value').Disconnect()

            return n

    def MakeReflectFromPS(self, clear=True):

        # For convenience...
        inp = self.PSInput

        col = inp('Reflection_Color'); val = inp('Reflection_Value')

        o = col.InNode()
        if o:

            n1 = self.AddNode('color_math', name='Old Reflect', collapsed=False, preview=False, Math_Argument=3.0, Value_1=col.Value(), Value_2=(1,1,1))
            n2 = self.AddNode('math_functions', collapsed=False, Math_Argument=3.0, Value_1=val.Value(), Value_2=1)
            self.ConnectNodes(n2, n1, 'Value_2')

            self.ConnectNodes(o, n1, 'Value_1')
            x = val.InNode()
            if x: self.ConnectNodes(x, n2, 'Value_2')

            if clear: col.Disconnect()

            return n1

        else:
            return None

    def MakeRefractFromPS(self, clear=True):

        # For convenience...
        inp = self.PSInput

        col = inp('Refraction_Color'); val = inp('Refraction_Value')

        o = col.InNode()
        if o:

            n1 = self.AddNode('color_math', name='Old Refract', collapsed=False, preview=False, Math_Argument=3.0, Value_1=col.Value(), Value_2=(1,1,1))
            n2 = self.AddNode('math_functions', collapsed=False, Math_Argument=3.0, Value_1=val.Value(), Value_2=1)
            self.ConnectNodes(n2, n1, 'Value_2')

            self.ConnectNodes(o, n1, 'Value_1')
            x = val.InNode()
            if x: self.ConnectNodes(x, n2, 'Value_2')

            if clear: col.Disconnect()

            return n1

        else:
            return None

    def MakeSpecularFromPS(self, clear=True):

        # For convenience...
        inp = self.PSInput

        if self.PSValue('Highlight_Color') == (0,0,0) or self.PSValue('Highlight_Value') == 0.0:
            return None
        else:
            n = self.AddNode('specular', name='Old Specular', collapsed=False, preview=False, Specular_Color=self.PSValue('Highlight_Color'), Specular_Value=self.PSValue('Highlight_Value'), Roughness=self.PSValue('Roughness'), Normals_Forward=0.000, )

            x = inp('Highlight_Color').InNode()
            if x: self.ConnectNodes(x, n, 'Specular_Color')
            x = inp('Highlight_Value').InNode()
            if x: self.ConnectNodes(x, n, 'Specular_Value')

            if clear:
                self.SetPSValue('Highlight_Color', (0,0,0))
                self.SetPSValue('Highlight_Value' , 0.0)
                inp('Highlight_Color').Disconnect()
                inp('Highlight_Value').Disconnect()

            return n

    def Name(self):
        return self.material.Name()

    def PileUpHere(self, node, xy, collapse=True):

        node.SetLocation(xy[0], xy[1])
        if collapse:
            node.SetInputsCollapsed(True)
            node.SetPreviewCollapsed(False)

        for inp in node.Inputs():
            inNode = inp.InNode()
            if inNode: self.PileUpHere(inNode, xy)

    def PSDisconnect(self, inp):
        self.Root().InputByInternalName(inp).Disconnect()

    def PSInput(self, inp):
        return self.Root().InputByInternalName(inp)

    def PSValue(self, inp):
        return self.Root().InputByInternalName(inp).Value()

    def RemoveOrphans(self, exceptType=None):
        # Deletes all orphaned nodes

        # Keep a list of connected nodes
        usedNodes = []

        # Search the shader tree for connected nodes
        for r in self.Roots():
            self.Traverse(r, usedNodes)

        # Delete any nodes not in the usedNodes list
        for node in self.tree.Nodes():
            try:
                index = usedNodes.index(node.InternalName())
            except:
                if not node.Type() == exceptType: node.Delete()

    def Root(self):
        return self.current_PS

    def RootInput(self):
        return self.Root().InputByInternalName

    def Roots(self):
        return list(self.roots.values())

    def SameAsKdTexture(self, node):
        kd = self.GetMap('color')
        if kd:
            if node.InputByInternalName('Image_Source').Value() == kd.InputByInternalName('Image_Source').Value():
                return True
        return False

    def Scattergroup(self):
        return self.scattergroup

    def SetInputValue(self, inp, value):
        # Set the value for a ShaderInput
        if not inp: return

        # Determine the data type
        inputType = inp.Type()

        #  Set the value according to data type
        if inputType in [0, 4, 5, 6]:
            inp.SetFloat(value)
        elif inputType in [1, 2]:
            r, g, b = value
            inp.SetColor(r, g, b)
        elif inputType == 3:
            inp.SetString(value)

    def SetPSRoot(self, node):
        self.current_PS = node

    def SetPSValue(self, inp, value):
        # Set the value of a PoserSurface Input
        self.SetInputValue(self.Root().InputByInternalName(inp), value)

    def SetRenderer(self, renderer, node):

        # Remember the new root node
        self.roots[renderer] = node

        # Set the renderer in Poser
        if renderer == 'firefly':
            self.tree.SetRendererRootNode(poser.kRenderEngineCodeFIREFLY, node )
        else:
            self.tree.SetRendererRootNode(poser.kRenderEngineCodeSUPERFLY, node)

    def SharedRoot(self):
        return self.roots.get('firefly', 0) == self.roots.get('superfly', 1)

    def Traverse(self, node, usedNodes, col=0, xy=None, use_ext=False):

        # Always include the starting node
        if use_ext:
            if not node.Name() in usedNodes:
                usedNodes.append(node.Name())
        else:
            if not node.InternalName() in usedNodes:
                usedNodes.append(node.InternalName())

        # Reposition the node if requested
        if xy:
            y = node.Location()[1]
            node.SetLocation(xy[0]+(col*xy[1]), y)
            nextcol = col + 1

        # Check each Input for more nodes
        for inp in node.Inputs():

            inNode = inp.InNode()

            if (inNode != None):

                if xy:
                    self.Traverse(inNode, usedNodes, nextcol, xy, use_ext=use_ext)
                else:
                    self.Traverse(inNode, usedNodes, use_ext=use_ext)

    def Tree(self):
        return self.tree

    def UpdatePreview(self):
        self.tree.UpdatePreview()

# EZAPI
class EZAPI(object):
    def __init__(self, ezskin):
        self.ezskin = ezskin

    def Build(self):
        return BUILD

    def ConvertColour(self, wxcol):
        if isinstance(wxcol, wx.Colour):
            r, g, b = wxcol.Get(includeAlpha=False)
        elif isinstance(wxcol, tuple):
            r, g, b = wxcol
        else:
            try:
                col = wx.Colour(0,0,0)
                col.SetFromName(wxcol)
                r, g, b = col.Get()
            except:
                tmp = wxcol[3:]
                r, g, b = eval(tmp)
        return (r/255.0, g/255.0, b/255.0)

    def EZSKin(self):
        return(self.ezskin)

    def GetBuilder(self, renderer):

        # Get the requested shader engine
        builder = self.ezskin.engines.get(renderer, None)

        # Configure it
        if builder is not None: builder.parameters = self.ezskin.data.ControlValue2

        # All done
        return builder

    def HTMLColorToRGB(self, colorstring):
        """ convert #RRGGBB to an (R, G, B) tuple """
        colorstring = colorstring.strip()
        if colorstring[0] == '#': colorstring = colorstring[1:]
        if len(colorstring) != 6:
            raise ValueError("input #%s is not in #RRGGBB format" % colorstring)
        r, g, b = colorstring[:2], colorstring[2:4], colorstring[4:]
        r, g, b = [int(n, 16) for n in (r, g, b)]
        return (r, g, b)

    def PoserBuild(self):
        return POSER_BUILD

    def PoserColToHTML(self, col_tuple):
        rgb = tuple([int(x*255) for x in col_tuple])
        return RGBToHTMLColor(rgb)

    def PoserColToRGB(self, col_tuple):
        rgb = tuple([int(x*255) for x in col_tuple])
        return rgb

    def PoserVersion(self):
        return POSER_VERSION

    def RGBToHTMLColor(self, rgb_tuple):
        hexcolor = '#%02x%02x%02x' % rgb_tuple
        return hexcolor

    def RGBToPoserCol(self, rgb):
        return tuple([x/255.0 for x in rgb])

    def SetStatus(self, message):
        self.ezskin.StatusBar(message)
        wx.Yield()

    def SROfromIOR(self, ior):
        return ((ior-1.0)/(ior+1.0))**2

# OVERLAYER
class Overlay ( wx.Dialog ):

    def __init__( self, parent ):
        wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = "Overlay maker", pos = wx.DefaultPosition, size = wx.Size( 500,300 ), style = wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER )

        self.SetSizeHints( wx.DefaultSize, wx.DefaultSize )

        siz_Main = wx.BoxSizer( wx.VERTICAL )

        siz_Figure = wx.BoxSizer( wx.HORIZONTAL )

        self.lbl_Actor = wx.StaticText( self, wx.ID_ANY, "Current actor:", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_Actor.Wrap( -1 )
        siz_Figure.Add( self.lbl_Actor, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_Actor = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_READONLY )
        siz_Figure.Add( self.ctl_Actor, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL|wx.EXPAND, 5 )


        siz_Main.Add( siz_Figure, 0, wx.EXPAND, 5 )

        siz_Overlays = wx.BoxSizer( wx.HORIZONTAL )

        siz_TopMaterial = wx.BoxSizer( wx.VERTICAL )

        self.lbl_TopShader = wx.StaticText( self, wx.ID_ANY, "Overlay this material ...", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_TopShader.Wrap( -1 )
        siz_TopMaterial.Add( self.lbl_TopShader, 0, wx.ALL, 5 )

        ctl_TopShaderChoices = []
        self.ctl_TopShader = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, ctl_TopShaderChoices, wx.CB_SORT )
        self.ctl_TopShader.SetSelection( 0 )
        siz_TopMaterial.Add( self.ctl_TopShader, 0, wx.ALL|wx.EXPAND, 5 )


        siz_Overlays.Add( siz_TopMaterial, 1, wx.EXPAND, 5 )

        siz_BottomMaterial = wx.BoxSizer( wx.VERTICAL )

        self.lbl_BottomShader = wx.StaticText( self, wx.ID_ANY, "... on top of this one", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_BottomShader.Wrap( -1 )
        siz_BottomMaterial.Add( self.lbl_BottomShader, 0, wx.ALL, 5 )

        ctl_BottomShaderChoices = []
        self.ctl_BottomShader = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, ctl_BottomShaderChoices, wx.CB_SORT )
        self.ctl_BottomShader.SetSelection( 0 )
        siz_BottomMaterial.Add( self.ctl_BottomShader, 0, wx.ALL|wx.EXPAND, 5 )


        siz_Overlays.Add( siz_BottomMaterial, 1, wx.EXPAND, 5 )


        siz_Main.Add( siz_Overlays, 0, wx.EXPAND, 5 )

        self.lbl_Using = wx.StaticText( self, wx.ID_ANY, "Using this transparency mask:", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_Using.Wrap( -1 )
        siz_Main.Add( self.lbl_Using, 0, wx.ALL, 5 )

        self.ctl_Mask = wx.FilePickerCtrl( self, wx.ID_ANY, wx.EmptyString, "Select a file", "*.*", wx.DefaultPosition, wx.DefaultSize, wx.FLP_CHANGE_DIR|wx.FLP_DEFAULT_STYLE|wx.FLP_FILE_MUST_EXIST|wx.FLP_OPEN )
        siz_Main.Add( self.ctl_Mask, 0, wx.ALL|wx.EXPAND, 5 )

        self.m_staticline7 = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
        siz_Main.Add( self.m_staticline7, 0, wx.EXPAND |wx.ALL, 5 )

        siz_Renderer = wx.BoxSizer( wx.HORIZONTAL )


        #siz_Renderer.AddSpacer( ( 0, 0), 1, wx.EXPAND, 5 )

        ctl_RendererChoices = [ "Firefly", "Superfly" ]
        self.ctl_Renderer = wx.RadioBox( self, wx.ID_ANY, "Renderer", wx.DefaultPosition, wx.DefaultSize, ctl_RendererChoices, 1, wx.RA_SPECIFY_ROWS )
        self.ctl_Renderer.SetSelection( 0 )
        siz_Renderer.Add( self.ctl_Renderer, 0, wx.ALL, 5 )


        siz_Main.Add( siz_Renderer, 0, wx.EXPAND, 5 )

        siz_DialogButtons = wx.BoxSizer( wx.HORIZONTAL )


        #siz_DialogButtons.AddSpacer( ( 0, 0), 1, wx.EXPAND, 5 )

        self.ctl_Cancel = wx.Button( self, wx.ID_CANCEL, "Cancel", wx.DefaultPosition, wx.DefaultSize, 0 )
        siz_DialogButtons.Add( self.ctl_Cancel, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_OK = wx.Button( self, wx.ID_OK, "Go on, do it!", wx.DefaultPosition, wx.DefaultSize, 0 )
        siz_DialogButtons.Add( self.ctl_OK, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        siz_Main.Add( siz_DialogButtons, 0, wx.EXPAND, 5 )


        self.SetSizer( siz_Main )
        self.Layout()

        self.Centre( wx.BOTH )

    def __del__( self ):
        pass
class EZOverlay(Overlay):

    def __init__(self, parent, actor, renderer):

        # Remember parameters
        self.parent = parent
        self.actor = actor

        # Get the material names as a list
        self.materials = [m.Name() for m in self.actor.Materials()]

        # Initialise the UI
        Overlay.__init__(self, parent)
        self.ctl_Actor.SetValue(actor.Name())
        self.ctl_Renderer.SetSelection(renderer)
        self.ctl_TopShader.AppendItems(self.materials)
        self.ctl_TopShader.Select(0)
        self.ctl_BottomShader.AppendItems(self.materials)
        self.ctl_BottomShader.Select(0)

    def CopyNodes(self, to_mat, start_node):
        usedNodes = {}
        return self.CopyTraverse(to_mat, start_node, usedNodes)

    def CopyTraverse(self, to_mat, node, usedNodes, from_input=None, from_node=None, newroot=None):

        processed = usedNodes.get(node.InternalName(), None)

        # Always process the starting node
        if not processed:
            new_node = self.DuplicateNode(to_mat, node)
            usedNodes[node.InternalName()] = new_node
            root = newroot or new_node

        else:
            root = newroot
            new_node = processed

        # Do we need to connect it?
        if from_input:
            to_mat.ConnectNodes(new_node, from_node, from_input)

        # Check each input for more nodes
        for inp in node.Inputs():

            inNode = inp.InNode()

            # If we find one ...
            if not inNode is None:
                self.CopyTraverse(to_mat, inNode, usedNodes, from_input=inp.InternalName(), from_node=new_node, newroot=root)

        return root

    def DuplicateNode(self, mat, source):

        # Get the value for each of the source node's Inputs
        inputvalues = {i.InternalName():i.Value() for i in source.Inputs()}

        # And then make the new node
        new_node = mat.AddNode(
            nodetype = source.Type(),
            name = source.Name(),
            loc = source.Location(),
            collapsed = source.InputsCollapsed(),
            preview = source.PreviewCollapsed(),
            inputs = list(inputvalues.items())
            )

        return new_node

    def MakeOverlay(self):

        # Get our MatHelper objects
        top = MatHelper(self.Material(self.SelectedMaterialTop()))
        bottom = MatHelper(self.Material(self.SelectedMaterialBottom()))

        # Get the starting point
        renderer = ('firefly', 'superfly')[self.SelectedRenderer()]
        from_node = bottom.FindPSRoot(renderer)

        # And copy it to the top layer
        if from_node:
            upper_root = top.FindPSRoot(renderer)

            # Check that an upper root node was found
            if upper_root is None: return 'Overlay failed: no target root found'

            lower_root = self.CopyNodes(top, from_node)
            top.ConfigureNode(lower_root, name='Base layer')
        else:
            return 'Overlay failed: no source root found'

        # Consolidate both layers
        top.SetPSRoot(upper_root)
        top.ConsolidateShader(pilehere=(400, 20), useSF=self.SelectedRenderer())
        top.SetPSRoot(lower_root)
        top.ConsolidateShader(pilehere=(400, 120), useSF=self.SelectedRenderer())

        # Make the transparency mask node
        alpha_path = self.ctl_Mask.GetPath()
        alpha = top.AddNode('image_map', name='Overlay mask', loc=(400, 300), collapsed=False, preview=False, Image_Source=alpha_path, Auto_Fit=0.000, U_Scale=1.000, V_Scale=1.000, U_Offset=0.000, V_Offset=0.000, Texture_Coords=1.000, Image_Mapped=4.000, Background=(1.000, 1.000, 1.000), Global_Coordinates=0.000, Mirror_U=0.000, Mirror_V=0.000, Texture_Strength=1.000, Filtering=4.000, )
        top.GC(alpha, data=True)

        # Bump
        inp = upper_root.InputByInternalName
        b = inp('Bump').InNode()
        if b:
            bv = inp('Bump').Value()
            bmp = (b, bv)
        else:
            bmp = None

        # Displacement
        inp = upper_root.InputByInternalName
        d = inp('Displacement').InNode()
        if d:
            dv = inp('Displacement').Value()
            dsp = (d, dv)
        else:
            dsp = None

        # Build the overlay
        shader = upper_root.InputByInternalName('AlternateDiffuse').InNode()
        top.ConfigureNode(shader, name='Upper layer')
        top.AddLayer(shader, alpha, bump=bmp, displacement=dsp)
        top.SetRenderer(renderer, lower_root)

        return 'Overlay completed successfully'

    def Material(self, name):
        return self.actor.Material(name)

    def SelectedMaterialBottom(self):
        return self.ctl_BottomShader.GetStringSelection()

    def SelectedMaterialTop(self):
        return self.ctl_TopShader.GetStringSelection()

    def SelectedRenderer(self):
        return self.ctl_Renderer.GetSelection()

# ROOM MANAGER
class Follow(object):
    def __init__(self, manager, ui, validrooms=['Pose', 'Material']):

        self.manager = manager
        self.ui = ui
        self.validrooms = validrooms
        self.resetrooms = validrooms

        self.rooms = {
            'ScenePallette' : 'Pose',
            'MaterialFrame' : 'Material',
            'FacePreview' : 'Face',
            'HairRoomGrowthGroups' : 'Hair',
            'ClothRoomSimulators' : 'Cloth'
            }

        self.current_room = self.Room()
        self.original_room = self.current_room

    def Reset(self):
        self.validrooms = self.resetrooms

    def Room(self):

        room = 'None'

        # Get the room name
        for p in list(self.rooms.keys()):
            if self.manager.GetPane(p).IsShown():
                room = self.rooms[p]

        return room

    def SetValidRooms(self, rooms):
        self.validrooms = rooms

    def SetValidCurrentRoomOnly(self):
        self.current_room = self.Room()
        self.validrooms = [self.current_room,]

    def SetValidOriginalRoomOnly(self):
        self.current_room = self.original_room
        self.validrooms = [self.current_room,]

    def Update(self, event):

        # Find which room we are in
        room = self.Room()

        # If we have changed rooms ...
        if room != self.current_room:

            # ... make a note of the new room
            self.current_room = room

            # and show (or hide) the UI
            pane = self.manager.GetPane(self.ui)
            if room in self.validrooms:
                pane.Float()
                pane.Show()
            else:
                pane.Hide()

            self.manager.Update()

            # Let Poser do its thing
            event.Skip()

# EZSKIN
class UI ( wx.Panel ):

    def __init__( self, parent ):
        wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 300,468 ), style = wx.TAB_TRAVERSAL )

        siz_SceneData = wx.BoxSizer( wx.VERTICAL )

        self.bar_Toolbar = wx.ToolBar( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TB_HORIZONTAL )
        self.bar_Toolbar.Realize()

        siz_SceneData.Add( self.bar_Toolbar, 0, wx.EXPAND, 5 )

        self.m_staticline2 = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
        siz_SceneData.Add( self.m_staticline2, 0, wx.EXPAND |wx.ALL, 5 )

        siz_CurrentFigure = wx.BoxSizer( wx.HORIZONTAL )

        self.lbl_CurrentFigure = wx.StaticText( self, wx.ID_ANY, "Figure:", wx.DefaultPosition, wx.Size( 60,-1 ), wx.ALIGN_RIGHT )
        self.lbl_CurrentFigure.Wrap( -1 )
        siz_CurrentFigure.Add( self.lbl_CurrentFigure, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_CurrentFigure = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_READONLY )
        self.ctl_CurrentFigure.SetToolTip( "The currently selected figure" )

        siz_CurrentFigure.Add( self.ctl_CurrentFigure, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        siz_SceneData.Add( siz_CurrentFigure, 0, wx.EXPAND, 5 )

        siz_CurrentActor = wx.BoxSizer( wx.HORIZONTAL )

        self.lbl_CurrentActor = wx.StaticText( self, wx.ID_ANY, "Actor:", wx.DefaultPosition, wx.Size( 60,-1 ), wx.ALIGN_RIGHT )
        self.lbl_CurrentActor.Wrap( -1 )
        siz_CurrentActor.Add( self.lbl_CurrentActor, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_CurrentActor = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_READONLY )
        self.ctl_CurrentActor.SetToolTip( "The currently selected actor" )

        siz_CurrentActor.Add( self.ctl_CurrentActor, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        siz_SceneData.Add( siz_CurrentActor, 0, wx.EXPAND, 5 )

        siz_Type = wx.BoxSizer( wx.HORIZONTAL )

        self.lbl_FigureType = wx.StaticText( self, wx.ID_ANY, "Type:", wx.DefaultPosition, wx.Size( 60,-1 ), wx.ALIGN_RIGHT )
        self.lbl_FigureType.Wrap( -1 )
        siz_Type.Add( self.lbl_FigureType, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_FigureType = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_READONLY )
        siz_Type.Add( self.ctl_FigureType, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_Editor = wx.Button( self, wx.ID_ANY, "Editor", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
        self.ctl_Editor.SetToolTip( "Open the figure editor to assign shaders to the figures materials" )

        siz_Type.Add( self.ctl_Editor, 0, wx.ALL, 5 )


        siz_SceneData.Add( siz_Type, 0, wx.EXPAND, 5 )

        self.m_staticline1 = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
        siz_SceneData.Add( self.m_staticline1, 0, wx.EXPAND |wx.ALL, 5 )

        siz_Renderer = wx.BoxSizer( wx.HORIZONTAL )

        self.lbl_Renderer = wx.StaticText( self, wx.ID_ANY, "Shader engine:", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_Renderer.Wrap( -1 )
        siz_Renderer.Add( self.lbl_Renderer, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        ctl_EngineChoices = []
        self.ctl_Engine = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, ctl_EngineChoices, 0 )
        self.ctl_Engine.SetSelection( 0 )
        siz_Renderer.Add( self.ctl_Engine, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_Configure = wx.Button( self, wx.ID_ANY, "Cfg", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
        self.ctl_Configure.Enable( False )

        siz_Renderer.Add( self.ctl_Configure, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )


        siz_SceneData.Add( siz_Renderer, 0, wx.EXPAND, 5 )

        self.m_staticline6 = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
        siz_SceneData.Add( self.m_staticline6, 0, wx.EXPAND |wx.ALL, 5 )

        self.lbl_Process = wx.StaticText( self, wx.ID_ANY, "PROCESS:", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_Process.Wrap( -1 )
        siz_SceneData.Add( self.lbl_Process, 0, wx.ALL, 5 )

        siz_Process = wx.GridSizer( 0, 2, 0, 0 )

        self.chk_ProcessSkin = wx.CheckBox( self, wx.ID_ANY, "Skin", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.chk_ProcessSkin.SetValue(True)
        siz_Process.Add( self.chk_ProcessSkin, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT, 5 )

        self.ctl_CfgSkin = wx.Button( self, wx.ID_ANY, "Edit skin", wx.DefaultPosition, wx.DefaultSize, 0 )
        siz_Process.Add( self.ctl_CfgSkin, 0, wx.ALIGN_CENTER_VERTICAL|wx.EXPAND|wx.RIGHT, 5 )

        self.chk_ProcessEyes = wx.CheckBox( self, wx.ID_ANY, "Eyes", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.chk_ProcessEyes.SetValue(True)
        siz_Process.Add( self.chk_ProcessEyes, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT, 5 )

        self.ctl_CfgEyes = wx.Button( self, wx.ID_ANY, "Edit eyes", wx.DefaultPosition, wx.DefaultSize, 0 )
        siz_Process.Add( self.ctl_CfgEyes, 0, wx.EXPAND|wx.RIGHT, 5 )

        self.chk_ProcessNails = wx.CheckBox( self, wx.ID_ANY, "Nails", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.chk_ProcessNails.SetValue(True)
        siz_Process.Add( self.chk_ProcessNails, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT, 5 )

        self.ctl_CfgNails = wx.Button( self, wx.ID_ANY, "Edit nails", wx.DefaultPosition, wx.DefaultSize, 0 )
        siz_Process.Add( self.ctl_CfgNails, 0, wx.EXPAND|wx.RIGHT, 5 )

        self.chk_ProcessMouth = wx.CheckBox( self, wx.ID_ANY, "Mouth", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.chk_ProcessMouth.SetValue(True)
        siz_Process.Add( self.chk_ProcessMouth, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT, 5 )

        self.ctl_CfgMouth = wx.Button( self, wx.ID_ANY, "Edit mouth", wx.DefaultPosition, wx.DefaultSize, 0 )
        siz_Process.Add( self.ctl_CfgMouth, 0, wx.ALIGN_CENTER_VERTICAL|wx.EXPAND|wx.RIGHT, 5 )

        self.chk_ProcessTeeth = wx.CheckBox( self, wx.ID_ANY, "Teeth", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.chk_ProcessTeeth.SetValue(True)
        siz_Process.Add( self.chk_ProcessTeeth, 0, wx.ALIGN_CENTER_VERTICAL|wx.LEFT, 5 )

        self.ctl_CfgTeeth = wx.Button( self, wx.ID_ANY, "Edit teeth", wx.DefaultPosition, wx.DefaultSize, 0 )
        siz_Process.Add( self.ctl_CfgTeeth, 0, wx.ALIGN_CENTER_VERTICAL|wx.EXPAND|wx.RIGHT, 5 )

        self.chk_ProcessOther = wx.CheckBox( self, wx.ID_ANY, "All others", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.chk_ProcessOther.SetValue(True)
        siz_Process.Add( self.chk_ProcessOther, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_CfgOther = wx.Button( self, wx.ID_ANY, "Edit other", wx.DefaultPosition, wx.DefaultSize, 0 )
        siz_Process.Add( self.ctl_CfgOther, 0, wx.ALIGN_CENTER_VERTICAL|wx.EXPAND|wx.RIGHT, 5 )


        siz_SceneData.Add( siz_Process, 0, wx.EXPAND, 5 )

        siz_EZSkin = wx.BoxSizer( wx.HORIZONTAL )

        self.ctl_ApplyEZSkin = wx.Button( self, wx.ID_ANY, "APPLY", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
        self.ctl_ApplyEZSkin.SetToolTip( "Apply EZSkin to selected areas" )

        siz_EZSkin.Add( self.ctl_ApplyEZSkin, 1, wx.ALL, 5 )

        self.ctl_Undo = wx.Button( self, wx.ID_ANY, "Undo", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
        siz_EZSkin.Add( self.ctl_Undo, 0, wx.ALL, 5 )

        self.ctl_SaveMat = wx.Button( self, wx.ID_ANY, "Save", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
        self.ctl_SaveMat.SetToolTip( "Save all current materials to a temporary Material Collection file " )

        siz_EZSkin.Add( self.ctl_SaveMat, 0, wx.ALL, 5 )

        self.ctl_LoadMat = wx.Button( self, wx.ID_ANY, "Restore", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
        self.ctl_LoadMat.SetToolTip( "Replace current materials using the last saved temporary Material Collection file" )

        siz_EZSkin.Add( self.ctl_LoadMat, 0, wx.ALL, 5 )


        siz_SceneData.Add( siz_EZSkin, 0, wx.EXPAND, 5 )

        self.m_staticline3 = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
        siz_SceneData.Add( self.m_staticline3, 0, wx.EXPAND |wx.ALL, 5 )

        siz_Preset = wx.BoxSizer( wx.HORIZONTAL )

        self.lbl_UsePreset = wx.StaticText( self, wx.ID_ANY, "Preset", wx.DefaultPosition, wx.DefaultSize, 0 )
        self.lbl_UsePreset.Wrap( -1 )
        siz_Preset.Add( self.lbl_UsePreset, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        ctl_PresetsChoices = []
        self.ctl_Presets = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, ctl_PresetsChoices, wx.CB_SORT )
        self.ctl_Presets.SetSelection( 0 )
        siz_Preset.Add( self.ctl_Presets, 1, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_SavePreset = wx.Button( self, wx.ID_ANY, "Save", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
        siz_Preset.Add( self.ctl_SavePreset, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )

        self.ctl_Export = wx.Button( self, wx.ID_ANY, "Exp", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
        siz_Preset.Add( self.ctl_Export, 0, wx.ALIGN_CENTER_VERTICAL|wx.BOTTOM|wx.TOP, 5 )

        self.ctl_Import = wx.Button( self, wx.ID_ANY, "Imp", wx.DefaultPosition, wx.DefaultSize, wx.BU_EXACTFIT )
        siz_Preset.Add( self.ctl_Import, 0, wx.ALIGN_CENTER_VERTICAL|wx.BOTTOM|wx.RIGHT|wx.TOP, 5 )


        siz_SceneData.Add( siz_Preset, 0, wx.EXPAND, 5 )

        siz_StatusBar = wx.BoxSizer( wx.VERTICAL )

        self.m_staticline5 = wx.StaticLine( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
        siz_StatusBar.Add( self.m_staticline5, 0, wx.EXPAND, 5 )

        self.ctl_StatusBar = wx.TextCtrl( self, wx.ID_ANY, "Status bar", wx.DefaultPosition, wx.DefaultSize, wx.TE_READONLY|wx.NO_BORDER )
        siz_StatusBar.Add( self.ctl_StatusBar, 0, wx.ALL|wx.EXPAND, 5 )


        siz_SceneData.Add( siz_StatusBar, 0, wx.EXPAND, 5 )


        self.SetSizer( siz_SceneData )
        self.Layout()
        self.mnu_Menu = wx.Menu()
        self.sub_File = wx.Menu()
        self.itm_Quit = wx.MenuItem( self.sub_File, wx.ID_ANY, "Close EZSkin", wx.EmptyString, wx.ITEM_NORMAL )
        self.sub_File.Append( self.itm_Quit )

        self.itm_About = wx.MenuItem( self.sub_File, wx.ID_ANY, "About EZSkin", wx.EmptyString, wx.ITEM_NORMAL )
        self.sub_File.Append( self.itm_About )

        self.mnu_Menu.AppendSubMenu( self.sub_File, "File" )

        self.sub_Presets = wx.Menu()
        self.itm_SavePreset = wx.MenuItem( self.sub_Presets, wx.ID_ANY, "Save preset", wx.EmptyString, wx.ITEM_NORMAL )
        self.sub_Presets.Append( self.itm_SavePreset )

        self.itm_ExportPresets = wx.MenuItem( self.sub_Presets, wx.ID_ANY, "Export presets", wx.EmptyString, wx.ITEM_NORMAL )
        self.sub_Presets.Append( self.itm_ExportPresets )

        self.itm_ImportPresets = wx.MenuItem( self.sub_Presets, wx.ID_ANY, "Import presets", wx.EmptyString, wx.ITEM_NORMAL )
        self.sub_Presets.Append( self.itm_ImportPresets )

        self.itm_DeletePresets = wx.MenuItem( self.sub_Presets, wx.ID_ANY, "Delete presets", wx.EmptyString, wx.ITEM_NORMAL )
        self.sub_Presets.Append( self.itm_DeletePresets )

        self.mnu_Menu.AppendSubMenu( self.sub_Presets, "Presets" )

        self.sub_FigFefs = wx.Menu()
        self.itm_ImportDefs = wx.MenuItem( self.sub_FigFefs, wx.ID_ANY, "Import definitions", wx.EmptyString, wx.ITEM_NORMAL )
        self.sub_FigFefs.Append( self.itm_ImportDefs )

        self.itm_ExportDefs = wx.MenuItem( self.sub_FigFefs, wx.ID_ANY, "Export definitions", wx.EmptyString, wx.ITEM_NORMAL )
        self.sub_FigFefs.Append( self.itm_ExportDefs )

        self.itm_LegacyDefs = wx.MenuItem( self.sub_FigFefs, wx.ID_ANY, "Import legacy definitions", wx.EmptyString, wx.ITEM_NORMAL )
        self.sub_FigFefs.Append( self.itm_LegacyDefs )

        self.mnu_Menu.AppendSubMenu( self.sub_FigFefs, "Figure definitions" )

        self.itm_Overlay = wx.MenuItem( self.mnu_Menu, wx.ID_ANY, "Overlay materials", wx.EmptyString, wx.ITEM_NORMAL )
        self.mnu_Menu.Append( self.itm_Overlay )

        self.Bind( wx.EVT_RIGHT_DOWN, self.UIOnContextMenu )


        # Connect Events
        self.Bind( wx.EVT_SIZE, self.RedrawUI )
        self.ctl_Editor.Bind( wx.EVT_BUTTON, self.EditFigure )
        self.ctl_Engine.Bind( wx.EVT_CHOICE, self.ChangeEngine )
        self.ctl_Configure.Bind( wx.EVT_BUTTON, self.ConfigureEngine )
        self.ctl_CfgSkin.Bind( wx.EVT_BUTTON, self.ConfigureSkin )
        self.ctl_CfgEyes.Bind( wx.EVT_BUTTON, self.ConfigureEyes )
        self.ctl_CfgNails.Bind( wx.EVT_BUTTON, self.ConfigureNails )
        self.ctl_CfgMouth.Bind( wx.EVT_BUTTON, self.ConfigureMouth )
        self.ctl_CfgTeeth.Bind( wx.EVT_BUTTON, self.ConfigureTeeth )
        self.ctl_CfgOther.Bind( wx.EVT_BUTTON, self.ConfigureOther )
        self.ctl_ApplyEZSkin.Bind( wx.EVT_BUTTON, self.Apply )
        self.ctl_Undo.Bind( wx.EVT_BUTTON, self.Undo )
        self.ctl_SaveMat.Bind( wx.EVT_BUTTON, self.SaveMat )
        self.ctl_LoadMat.Bind( wx.EVT_BUTTON, self.LoadMat )
        self.ctl_Presets.Bind( wx.EVT_CHOICE, self.ApplyPreset )
        self.ctl_SavePreset.Bind( wx.EVT_BUTTON, self.SavePreset )
        self.ctl_Export.Bind( wx.EVT_BUTTON, self.Presets_Export )
        self.ctl_Import.Bind( wx.EVT_BUTTON, self.Presets_Import )
        self.Bind( wx.EVT_MENU, self.Close, id = self.itm_Quit.GetId() )
        self.Bind( wx.EVT_MENU, self.About, id = self.itm_About.GetId() )
        self.Bind( wx.EVT_MENU, self.SavePreset, id = self.itm_SavePreset.GetId() )
        self.Bind( wx.EVT_MENU, self.Presets_Export, id = self.itm_ExportPresets.GetId() )
        self.Bind( wx.EVT_MENU, self.Presets_Import, id = self.itm_ImportPresets.GetId() )
        self.Bind( wx.EVT_MENU, self.Presets_Delete, id = self.itm_DeletePresets.GetId() )
        self.Bind( wx.EVT_MENU, self.FigureDefs_Import, id = self.itm_ImportDefs.GetId() )
        self.Bind( wx.EVT_MENU, self.FigureDefs_Export, id = self.itm_ExportDefs.GetId() )
        self.Bind( wx.EVT_MENU, self.FigureDefs_Import_Legacy, id = self.itm_LegacyDefs.GetId() )
        self.Bind( wx.EVT_MENU, self.Overlay, id = self.itm_Overlay.GetId() )

    def __del__( self ):
        pass


    # Virtual event handlers, overide them in your derived class
    def RedrawUI( self, event ):
        event.Skip()

    def EditFigure( self, event ):
        event.Skip()

    def ChangeEngine( self, event ):
        event.Skip()

    def ConfigureEngine( self, event ):
        event.Skip()

    def ConfigureSkin( self, event ):
        event.Skip()

    def ConfigureEyes( self, event ):
        event.Skip()

    def ConfigureNails( self, event ):
        event.Skip()

    def ConfigureMouth( self, event ):
        event.Skip()

    def ConfigureTeeth( self, event ):
        event.Skip()

    def ConfigureOther( self, event ):
        event.Skip()

    def Apply( self, event ):
        event.Skip()

    def Undo( self, event ):
        event.Skip()

    def SaveMat( self, event ):
        event.Skip()

    def LoadMat( self, event ):
        event.Skip()

    def ApplyPreset( self, event ):
        event.Skip()

    def SavePreset( self, event ):
        event.Skip()

    def Presets_Export( self, event ):
        event.Skip()

    def Presets_Import( self, event ):
        event.Skip()

    def Close( self, event ):
        event.Skip()

    def About( self, event ):
        event.Skip()

    def Presets_Delete( self, event ):
        event.Skip()

    def FigureDefs_Import( self, event ):
        event.Skip()

    def FigureDefs_Export( self, event ):
        event.Skip()

    def FigureDefs_Import_Legacy( self, event ):
        event.Skip()

    def Overlay( self, event ):
        event.Skip()

    def UIOnContextMenu( self, event ):
        self.PopupMenu( self.mnu_Menu, event.GetPosition() )
class EZSkin(UI):
    def __init__(self, parent, main):
        UI.__init__(self, parent)

        self.main = main                     # The parent class

        # Complete UI set up
        self.icons = self.AddToolbars()
        self.main.manager.Update()

        # Initialise the timer
        self.timer = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.Update, self.timer)

        # Poser scene variables
        self.scene = poser.Scene()            # A reference to the Scene object
        self.figure = None                     # The current figure
        self.actor = None                     # The current actor
        self.old_obj = None                 # The last geometry to be evaluated

        # Additional helper objects
        self.data = EZData(self.main.cfg)    # Used to manage values from dialogue boxes etc
        self.presets = self.main.presets     # Used to save UI settings
        self.lib = Lib()                     # The figure type definition library manager
        self.engines = {}                    # Our shader engines, including custom ones
        self.api = EZAPI(self)                # The interface with custom shader engines

        # Initialise the UI
        self.ctl_Presets.AppendItems(self.presets.Sections())

        # Paths and data for Undo file
        self.mc6 = None
        self.mc6path = None
        self.undo = os.path.join(poser.TempLocation(), 'ezskin3' + os.extsep + 'mc6')

        # Initialise the dialog controls
        self.Initialise()

        # A hack for the plugins to work
        self.data.SetControlValue('custom_shaders', self.main.cfg.Get('settings', 'custom_shaders'), 'settings')

        # Get custom shader engines
        self.GetCustomEngines()

        # Reload last used settings if appropriate
        try:
            # Get the user's choice
            reload = self.main.cfg.Get('settings', 'remember')

            # Always remember the remember setting!
            if not reload is None: self.data.SetControlValue('remember', reload, 'settings')

            # Reload the settings?
            if reload == True:
                self.data.ReadFromConfig()
                self.LoadOptions()

        except:
            pass

        # Prepare shader engines
        engines = self.data.Range('renderer', 'settings')
        self.ctl_Engine.AppendItems(engines)

        # Restore renderer selection
        idx = self.main.cfg.Get('settings', 'renderer')
        if not idx is None: self.data.SetControlValue('renderer', idx, 'settings')

        # Synchronise shader engines
        self.SyncEngines()
        self.builder = None
        self.ChangeEngine(None)

        # Get our package manager
        path = self.data.ControlValue('packages', 'settings')
        self.pkg_mgr = PackageManager(path)

        # Resolve all file paths
        self.scene.ResolvePendingTextures()

        # Get the managed window
        wnd = self.main.manager.GetManagedWindow()

        # Add key combo to align nodes :-)
        randomId1 = wx.NewIdRef(count=1)    # wx.NewId() is deprecated
        wnd.Bind(wx.EVT_MENU, self.AlignNodes, id=randomId1)

        # Build the Accelerator Table
        accel_tbl = wx.AcceleratorTable([
            (wx.ACCEL_ALT,  ord('='), randomId1)]
            )

        wnd.SetAcceleratorTable(accel_tbl)

        # Multiple room support
        self.follow = Follow(self.main.manager, self, VALID_ROOMS)
        self.multiroom = self.data.ControlValue('multiroom', 'settings')

        # All done :-)

        # Start the timer ...
        self.timer.Start(500)

        # ... and let the user know we're all done
        self.StatusBar("%s initialisation complete" % APPNAME)

    def About(self, event):

        # Display an informational dialog box
        about = wx.adv.AboutDialogInfo()

        about.SetName(APPNAME)
        about.SetVersion("(Build %03d)" % BUILD)
        about.AddDeveloper('Snarlygribbly, Bagginsbill')
        about.AddDocWriter('Fverbaas')
        about.SetWebSite('http://snarlygribbly.org/poser')
        about.SetCopyright('(c) 2020 Snarlygribbly')
        about.SetArtists(['Biscuits (EZSkin icons)\n Amethystpendant, Artdude41, Bagginsbill,\n Bu_es, Gadzooks, Glitterati3d, Hauge, Jura,\n Pangurban, RedPhantom, Seeker, Wimvdb'])

        wx.adv.AboutBox(about)

    def AddPresetToMenu(self, preset):
        # Adds a preset to the Presets dropdown control
        ctl = self.ctl_Presets
        ctl.Append(preset)

    def AddMakeup(self, event):
        self.Nix(event)

    def AddOverlay(self, event):
        self.Nix(event)

    def AddToolbars(self):

        # Add toolbar icons ...
        icons = EZIcons()

        self.bar_Toolbar.SetToolBitmapSize((32, 32))
        self.bar_Toolbar.AddTool(7, "Menu", icons.GetBitmap('menu'), wx.NullBitmap, wx.ITEM_NORMAL, "Menu", "", None)
        self.bar_Toolbar.AddTool(5, "Apply", icons.GetBitmap('apply'), wx.NullBitmap, wx.ITEM_NORMAL, "Apply the shaders", "", None)
        self.bar_Toolbar.AddTool(1, "Import", icons.GetBitmap('import'), wx.NullBitmap, wx.ITEM_NORMAL, "Import figure definitions", "", None)
        self.bar_Toolbar.AddTool(2, "Export", icons.GetBitmap('export'), wx.NullBitmap, wx.ITEM_NORMAL, "Export figure definitions", "", None)
        self.bar_Toolbar.AddTool(3, "Configure", icons.GetBitmap('configure'), wx.NullBitmap, wx.ITEM_NORMAL, "Configure EZSkin", "", None)
        self.bar_Toolbar.AddTool(4, "Web", icons.GetBitmap('web'), wx.NullBitmap, wx.ITEM_NORMAL, "Get the latest version of EZSkin", "", None)
        self.bar_Toolbar.AddTool(6, "Close", icons.GetBitmap('close'), wx.NullBitmap, wx.ITEM_NORMAL, "Close EZSkin", "", None)

        self.bar_Toolbar.Realize()

        # ... and bind them to our methods
        self.Bind(wx.EVT_TOOL, self.Menu, id=7)
        self.Bind(wx.EVT_TOOL, self.FigureDefs_Import, id=1)
        self.Bind(wx.EVT_TOOL, self.FigureDefs_Export, id=2)
        self.Bind(wx.EVT_TOOL, self.ConfigureSettings, id=3)
        self.Bind(wx.EVT_TOOL, self.Web, id=4)
        self.Bind(wx.EVT_TOOL, self.Apply, id=5)
        self.Bind(wx.EVT_TOOL, self.Close, id=6)

        return icons

    def AlignNodes(self, event):

        # Get the current material
        mat = self.scene.CurrentMaterial()

        # Process all layers
        for layer in mat.Layers():
            mh = MatHelper(layer)

            # EZSkin is only concerned with PoserSurface roots
            root = mh.FindPSRoot(self.builder.Renderer().lower(), roottype='any')
            if root:
                mh.AlignNodes(root, moveroot=False)

    def Apply(self, event):
        # Creates the EZSkin shaders

        # Get the target object
        target = self.GetActor()

        # Check that the figure can be processed
        if self.ctl_FigureType.GetValue() == '(Unsupported)':
            ShowAlert('This figure is not supported')
            return

        # Make sure we have a valid target
        if not target:
            ShowAlert('no valid actor selected for processing')
        else:

            # Use the selected renderer
            self.builder = self.GetBuilder()

            # Make an undo file
            self.SaveMat(None, temp=True)

            # Obtain the corresponding EZSkin shader, checking for custom shaders first ...
            customshaders = self.data.GetCustomShaders(target)

            # Process each material in turn
            for material in target.Materials():

                if customshaders:
                    shader = customshaders.get(material.Name(), None)
                else:
                    shader = self.lib.GetTemplate(target, material.Name())

                # Determine the type of shader
                if shader is None: continue
                shadertype = self.builder.ShaderType(shader[0])

                # If appropriate, build the shader
                if shader and getattr(self, 'chk_Process%s' % ('Skin', 'Eyes', 'Nails', 'Mouth', 'Teeth', 'Other')[shadertype]).IsChecked():
                    self.StatusBar('Processing material %s' % material.Name())

                    # Delete unwanted layers
                    if self.data.ControlValue('delete_layers', 'settings'):
                        if hasattr(material, 'DeleteLayer'):
                            while len(material.Layers()) > 1:
                                material.DeleteLayer(1)

                    mh = MatHelper(material, group=int(shader[1]))
                    msg = self.builder.BuildShader(material, shader, mh)

                    self.StatusBar('Material %s %s' % (material.Name(), msg))
                    wx.Yield()

            # Advise user of success
            self.StatusBar('Processing completed for actor %s' % target.Name())

    def ApplyPreset(self,event):

        # Get the selected preset
        preset = self.ctl_Presets.GetStringSelection()

        # Update control values
        items = self.presets.Items(preset)
        for k, v in items:
            self.data.SetControlValue(k, v)

        # Synchronise engine selection
        self.SyncEngines()

        # Update main UI controls
        self.LoadOptions()

        self.StatusBar('Preset %s applied' % preset)

    def ChangeEngine(self, event):

        # Ensure UI controls are synchronised
        self.SyncEngines(False)

    def Close(self, event):

        # Stop the timer
        self.timer.Stop()

        # Get our pane object
        man = self.main.manager
        pane = man.GetPane(self)

        # Update figure definitions file
        self.lib.Write()

        # Update configuration file
        self.SaveOptions()
        self.data.WriteToConfig()
        self.main.UpdateConfiguration(pane)

        # Close the pane
        pane.Hide()
        man.Update()
        man.ClosePane(pane)

    def ConfigureEngine(self, event):
        self.data.Display(self.builder.unique_name, self.StatusBar)

    def ConfigureEyes( self, event ):
        self.data.Display('eyes', self.StatusBar)

    def ConfigureMouth( self, event ):
        self.data.Display('mouth', self.StatusBar)

    def ConfigureNails(self, event):
        self.data.Display('nails', self.StatusBar)

    def ConfigureOther(self, event):
        self.data.Display('other', self.StatusBar)

    def ConfigureSettings(self, event):
        self.data.Display('settings', self.StatusBar)
        self.pkg_mgr.SetPath(self.data.ControlValue('packages', 'settings'))
        self.SyncEngines(True)

    def ConfigureSkin( self, event ):
        self.data.Display('skin', self.StatusBar)

    def ConfigureTeeth(self, event):
        self.data.Display('teeth', self.StatusBar)

    def EditFigure(self, event):

        # Stop the timer while modal dialog is open
        self.timer.Stop()

        # Show the Editor
        fig = self.GetActor()
        if fig:
            dlg = EZEditor(self, self.lib, fig, self.data.GetCustomShaders(fig))
            dlg.ShowModal()
            custom_shaders = dlg.GetCustom()
            dlg.Destroy()
        else:
            custom_shaders = None
            ShowAlert('This object cannot be processed by EZSkin')

        # Remember custom shader settings
        self.data.UseCustomShaders(fig, custom_shaders)

        # Force refresh of figure data when timer starts again
        self.old_obj = None

        # And start the timer :-)
        self.timer.Start()

        # Feedback
        self.StatusBar('Figure editing completed')

    def FigureDefs_Export(self, event):

        # Select which definitions are to be exported
        figs = self.lib.GetAllTemplates()
        dlg = Selector(figs, 'Export figure definitions', buttontext='Export')
        retcode = dlg.ShowModal()

        if retcode == 1:

            # Ask the user for a file location
            fdlg = wx.FileDialog(None,
                'Choose a file',
                defaultDir=poser.ContentRootLocation(),
                defaultFile='',
                wildcard='*.defs',
                style=wx.FD_SAVE,
                pos=wx.DefaultPosition)

            if fdlg.ShowModal() == wx.ID_OK:            # User chose a file
                path = fdlg.GetPath()                    # get the path to the file

            # Get the ConfigParser object ...
            export = self.lib.GetFigureDefs(dlg.GetSelected())

            # ... and write it to disk
            try:
                cfgfile = open(path, 'w')
                export.write(cfgfile)
                cfgfile.close()
            except:
                ShowAlert('Error saving definitions file', 'Figure definitions exporter')
                fdlg.Destroy()
                return

            self.StatusBar('Presets successfully exported')
            fdlg.Destroy()

        else:
            self.StatusBar('Export cancelled')

        # Clean up
        dlg.Destroy()

    def FigureDefs_Import(self, event, legacy=False):

        # Ask the user for a file location
        fdlg = wx.FileDialog(None,
            'Choose a file',
            defaultDir=poser.ContentRootLocation(),
            defaultFile='',
            wildcard=('*.defs', '*.cfg')[legacy],
            style=wx.FD_OPEN,
            pos=wx.DefaultPosition)

        if fdlg.ShowModal() == wx.ID_OK:            # User chose a file

            path = fdlg.GetPath()                    # get the path to the file

            # Get a config parser
            imp = configparser.RawConfigParser()
            imp.optionxform = str

            # And read the config data from the user's chosen file
            imp.read(path)

            if imp.has_section('OBJLOOKUP'):
                figs = []
                for k, v in imp.items('OBJLOOKUP'):
                    if imp.has_section(v):
                        if v not in figs: figs.append(v)

                # Create a dialog box to select figures for import
                dlg = Selector(figs, 'Import figure definitions', buttontext='Import')
                retcode = dlg.ShowModal()

                if retcode == 1:
                    figs = dlg.GetSelected()

                    # Update the library
                    self.lib.ImportDefinitions(imp, figs, legacy)
                    self.StatusBar(('Import completed', 'No imports selected')[len(figs)==0])

                dlg.Destroy()

                # Force refresh of figure data when timer starts again
                self.old_obj = None

            else:
                ShowAlert('No figure definitions found')

        else:
            self.StatusBar('Import cancelled')

    def FigureDefs_Import_Legacy(self, event):
        self.FigureDefs_Import(event, legacy=True)

    def GetActor(self):
        # Returns an actor which EZSkin can process, or None

        if self.figure:    # Figure gets priority
            return self.figure
        else:
            if self.actor:
                if self.actor.IsBodyPart():
                    return self.actor.ItsFigure()
                else:
                    if not self.actor.Materials() is None:
                        return self.actor
                    else:
                        return None
            else:
                return None

    def GetActorObjName(self):
        # Returns the geometry name of the current figure if possible, otherwise the current actor's

        # Get the pertinent actor
        act = self.GetActor()

        # Retrieve its geometry name
        if act:
            return self.lib.GetObjName(act)
        else:
            return None

    def GetBuilder(self):
        # Returns the appropriate 'flavour' of Shadermaker object

        renderer = self.engines[self.data.Range('renderer')[self.data.ControlValue('renderer')]]
        renderer.parameters = self.data.ControlValue2
        return renderer

    def GetCustomEngines(self):

        # Get the folder
        folder = self.data.ControlValue('custom_shaders', 'settings')
        path = (None, folder)[self.IsValidPath(folder)]

        # Import the shader engines
        if path:

            # Find all the plugins in the folder
            wildcard = os.path.join(path, '*.py*')
            files = [os.path.join(path, x) for x in glob.glob1(path, '*.py*')]

            # And check that they are valid plugins
            for f in files:
                newplg, name = self.ImportPlugin(f)

                # If valid
                if newplg:

                    # Update the control to include the name of the new plugin
                    ctl = self.data.ControlCfg('renderer', 'settings')
                    ctl.minmax.append(name)

                    # Add it to our dictionary of engines
                    self.engines[name] = newplg

                else:
                    ShowAlert('Invalid EZSkin plugin found:\n%s' % f)

    def GetCustomEngineControls(self, engine):

        # The list of controls
        data = list()

        # Make them ...
        for ctl in engine.Controls():
            data.append(EZCtlCfg(ctl[0],ctl[1],ctl[2],ctl[3],ctl[4],ctl[5],ctl[6],ctl[7]))

        # ... and add them to our Data Manager
        self.data.AddControlSet(engine.unique_name, data)

    def ImportPlugin(self, filename, prefix='ezplugin'):

        # Parse the plugin name
        path, name = os.path.split(filename)
        name, ext = os.path.splitext(name)

        # Assign a new name to avoid overwriting system modules
        modname = "%s_%s" % (prefix, name)

        # Get the plugin ...
        f, filename, data = imp.find_module(name, [path])

        # ... and load it as a module
#         try:
#             mod = imp.load_module(name, f, filename, data)
#         except:
#             mod = None
        mod = imp.load_module(name, f, filename, data)

        if mod:
            return self.ValidatePlugin(mod)
        else:
            ShowAlert('Invalid EZSkin plugin found:\n%s' % filename)
            return None, None

    def Initialise(self):

        # Get the built-in shader engines
        renderers = ['Firefly', 'Superfly']
        self.engines = {'Firefly' : Shadermaker(self.api), 'Superfly' : ShadermakerCycles(self.api)}

        # Define the controls
        # (self, name, label, kind, minmax, default, increment=None, tooltip='', category=None)

        process = (
            EZCtlCfg('chk_ProcessSkin', 'Skin', EZBOOL, (None), 1, None, '', 'Main'),
            EZCtlCfg('chk_ProcessEyes', 'Skin', EZBOOL, (None), 0, None, '', 'Main'),
            EZCtlCfg('chk_ProcessNails', 'Skin', EZBOOL, (None), 0, None, '', 'Main'),
            EZCtlCfg('chk_ProcessMouth', 'Skin', EZBOOL, (None), 0, None, '', 'Main'),
            EZCtlCfg('chk_ProcessTeeth', 'Skin', EZBOOL, (None), 0, None, '', 'Main'),
            EZCtlCfg('chk_ProcessOther', 'Skin', EZBOOL, (None), 1, None, '', 'Main'),
            )

        cfgdata = (
            EZCtlCfg('lbl01', 'Main settings', EZLABEL, (None), None, None, '', 'Main'),
            EZCtlCfg('remember', 'Remember last used', EZBOOL, (None), 1, None, 'If ticked EZSkin will set the value of each control to that which was last used.', 'Main'),
            EZCtlCfg('gc', 'Apply gamma correction', EZBOOL, (None), 1, None, 'Configure image maps for correct gamma correction', 'Main'),
            EZCtlCfg('filter', 'Apply texture filter', EZBOOL, (None), 0, None, 'Set texture filter on all image maps', 'Main'),
            EZCtlCfg('filtering', 'Texture filter', EZISELECT, ('None', 'Fast', 'Quality', 'Crisp'), 3, None, 'The type of texture filtering to be applied', 'Main'),
            EZCtlCfg('renderer', 'Shader engine', EZISELECT, renderers, 0, None, 'The target render engine', 'Main'),
            EZCtlCfg('make_root', 'Create root nodes', EZBOOL, (None), 1, None, 'Create new root nodes to avoid overwriting shaders', 'Main'),
            EZCtlCfg('delete_layers', 'Remove Superfly layers', EZBOOL, (None), 0, None, 'Disable or remove Superfly layers', 'Main'),
            EZCtlCfg('custom_shaders', 'Custom shader engines', EZDIR, (None), "", None, 'The folder used to store custom shaders', 'Main'),
            EZCtlCfg('packages', 'Figure support folder', EZDIR, (None), "", None, 'The folder used to store EZSkin character packages', 'Main'),
            EZCtlCfg('multiroom', 'Open in both rooms', EZBOOL, (None), 1, None, 'If ticked EZSkin will open as instances in both the Pose room and the Material room. Changing this setting require you to restart EZSkin for the change to take affect.', 'Main'),
            )

        skindata = (

            EZCtlCfg('lblbasic', 'Main settings', EZLABEL, (None), None, None, '', 'Main'),
            EZCtlCfg('sss_scale', 'SSS Scale', EZFLOAT, (0.1, 5.0), 1.5, 0.1, 'SSS strength', 'Main'),
            EZCtlCfg('sss_type', 'Type', EZISELECT, SSSTYPES, 8, None, 'SSS type', 'Main'),
            EZCtlCfg('sss_quality', 'SSS Quality', EZFLOAT, (0.0, 1.0), 0.8, 0.1, 'Higher values = better quality but longer render times', 'Main'),
            EZCtlCfg('specularity', 'Specularity', EZFLOAT, (0.0, 6.0), 1.1, 0.1, 'Strength of specular reflection', 'Main'),
            EZCtlCfg('add_fresnel', 'Add fresnel reflection', EZBOOL, (None), 0, None, 'Add fresnel reflection to skin', 'Main'),

            EZCtlCfg('lblbump', 'Bump', EZLABEL, (None), None, None, '', 'Bump'),
            EZCtlCfg('bump_opt', 'Type', EZISELECT, ('Bump map or nothing', 'Prefer bump map', 'Procedural bump'), 2, None, '', 'Bump'),
            EZCtlCfg('bump_mod', 'Bump modifier', EZFLOAT, (0.5, 1.5), 1.0, 0.1, 'Bump amount is multiplied by this value', 'Bump'),
            EZCtlCfg('bump_usecustom', 'Use custom bump value', EZBOOL, (None), 0, None, 'Override exisitng bump value', 'Specular'),
            EZCtlCfg('bump_value', 'Bump value', EZFLOAT, (0.0, 2.0), 0.0, 0.1, 'Value used to overrides existing bump value', 'Bump'),

            EZCtlCfg('lblnodes', 'Specular', EZLABEL, (None), None, None, '', 'Specular'),
            EZCtlCfg('spec_opt', 'Type', EZISELECT, ('Specular map or nothing', 'Prefer specular map', 'Procedural specular'), 2, None, '', 'Specular'),
            EZCtlCfg('spec_eccentricity', 'Eccentricity', EZFLOAT, (0.0, 1.0), 0.3, 0.1, 'See Poser manual', 'Specular'),
            EZCtlCfg('spec_rolloff', 'RollOff', EZFLOAT, (0.0, 1.0), 0.4, 0.1, 'See Poser manual', 'Specular'),
            EZCtlCfg('modulate_spec', 'Modulate specular softness', EZBOOL, (None), 0, None, 'See user guide', 'Specular'),
            EZCtlCfg('spec_softness', 'Specular softness factor', EZFLOAT, (0.0, 10.0), 4.5, 0.1, 'See user guide', 'Specular'),

            EZCtlCfg('lblreflect', 'Reflection', EZLABEL, (None), None, None, '', 'Reflection'),
            EZCtlCfg('reflect_quality', 'Quality', EZFLOAT, (0.0, 1.0), 0.4, 0.1, 'Higher values = better quality but longer render time', 'Reflection'),
            EZCtlCfg('reflect_softness', 'Softness', EZFLOAT, (0.0, 5.0), 2.5, 0.1, 'How blurred the reflections are', 'Reflection'),
            EZCtlCfg('reflect_raybias', 'Ray bias', EZFLOAT, (0.0, 1.0), 0.3, 0.01, 'See Poser manual', 'Reflection'),

            EZCtlCfg('lblfresnel', 'Fresnel', EZLABEL, (None), None, None, '', 'Fresnel'),
            EZCtlCfg('fresnel_ior', 'IOR', EZFLOAT, (0.0, 2.0), 1.4, 0.1, 'IOR for skin', 'Fresnel'),
            EZCtlCfg('fresnel_mult', 'Multiplier', EZFLOAT, (0.0, 1.0), 1.0, 0.1, 'Fresnel effect multiplier', 'Fresnel'),
            EZCtlCfg('rough_fresnel', 'Use rough surface fresnel', EZBOOL, (None), 0, None, 'True = better accuracy but longer render times', 'Fresnel'),
            EZCtlCfg('rough_ior', 'Alt rough IOR', EZFLOAT, (0.0, 2.0), 1.65, 0.1, 'IOR for rough surface', 'Fresnel'),
            EZCtlCfg('rough_mult', 'Alt. multiplier', EZFLOAT, (0.0, 1.0), 0.5, 0.1, 'See user guide', 'Fresnel'),

            EZCtlCfg('lblHSV', 'HSV', EZLABEL, (None), None, None, '', 'HSV'),
            EZCtlCfg('HSV_colour', 'Colour', EZCOLOURP, (None), (1.0, 1.0, 1.0), None, 'Modifies the colour of the skin', 'HSV'),
            EZCtlCfg('HSV_H', 'Hue', EZFLOAT, (0.0, 10.0), 1.0, 0.1, 'Modifies the hue of the skin', 'HSV'),
            EZCtlCfg('HSV_S', 'Saturation', EZFLOAT, (0.0, 2.0), 1.0, 0.1, 'Modifies the saturation of the skin', 'HSV'),
            EZCtlCfg('HSV_V', 'Value', EZFLOAT, (0.0, 2.0), 1.0, 0.1, 'Intensity modifier', 'HSV'),

            EZCtlCfg('lblscatter', 'BB Max Scatter Trick', EZLABEL, (None), None, None, '', 'BB Max Scatter'),
            EZCtlCfg('maxscatter', 'Use BB Max Scatter trick', EZBOOL, (None), 0, None, 'Reduces the possibility of skin turning blue when SSS is applied', 'BB Max Scatter'),
            EZCtlCfg('bbSSSscale', 'SSS Scale', EZFLOAT, (0.0, 5.0), 2.0, 0.1, 'See user guide', 'BB Max Scatter'),
            EZCtlCfg('bbHSVscale', 'HSV scale', EZFLOAT, (0.0, 2.0), 0.9, 0.1, 'See user guide', 'BB Max Scatter'),

            EZCtlCfg('sf_lblnodes', 'Superfly specular', EZLABEL, (None), None, None, '', 'Superfly'),
            EZCtlCfg('sf_spec_opt', 'Type', EZISELECT, ('Specular map or nothing', 'Prefer specular map', 'Procedural specular'), 2, None, '', 'Superfly'),
            EZCtlCfg('sf_spec_eccentricity', 'Eccentricity', EZFLOAT, (0.0, 1.0), 0.3, 0.05, 'See Poser manual', 'Superfly'),
            EZCtlCfg('sf_spec_rolloff', 'RollOff', EZFLOAT, (0.0, 1.0), 0.4, 0.05, 'See Poser manual', 'Superfly'),
            EZCtlCfg('sf_spec_ior', 'Calculte RollOff from IOR', EZBUTTON, (None), self.UseIOR, None, 'Calculates SpecularRollOff from IOR value', 'Superfly'),
            EZCtlCfg('sf_spec_reflectivity', 'Reflectivity', EZFLOAT, (0.0, 100.0), 0.50, 0.10, 'See Poser manual', 'Superfly'),
            )

        teethdata = (

            EZCtlCfg('teeth_lblmain', 'Main settings', EZLABEL, (None), None, None, '', 'Main'),
            EZCtlCfg('teeth_sss_scale', 'SSS Scale', EZFLOAT, (0.1, 2.0), 0.9, 0.1, 'SSS strength', 'Main'),
            EZCtlCfg('teeth_spec_roughness', 'Specular roughness', EZFLOAT, (0.0, 1.0), 0.4, 0.1, 'How blurred the specular effect is', 'Main'),
            EZCtlCfg('teeth_spec_ks', 'Specular Ks', EZFLOAT, (0.0, 1.0), 0.1, 0.1, 'The strength of the specular effect', 'Main'),
            EZCtlCfg('teeth_sss_quality', 'SSS Quality', EZFLOAT, (0.0, 1.0), 0.8, 0.1, 'Higher values = better quality but longer render time', 'Main'),
            EZCtlCfg('teeth_sss_type', 'Type', EZISELECT, SSSTYPES, 5, None, 'SSS Type', 'Main'),
            EZCtlCfg('teeth_fresnel_ior', 'IOR', EZFLOAT, (0.0, 2.0), 1.33, 0.1, 'IOR of the teeth', 'Main'),

            EZCtlCfg('teeth_lblreflect', 'Reflection', EZLABEL, (None), None, None, '', 'Reflection'),
            EZCtlCfg('teeth_reflect_quality', 'Quality', EZFLOAT, (0.0, 1.0), 0.4, 0.1, 'Higher values = better quality but longer render time', 'Reflection'),
            EZCtlCfg('teeth_reflect_softness', 'Softness', EZFLOAT, (0.0, 5.0), 0.1, 0.1, 'How blurred the reflections are', 'Reflection'),
            EZCtlCfg('teeth_reflect_raybias', 'Ray bias', EZFLOAT, (0.0, 1.0), 0.3, 0.01, 'See Poser manual', 'Reflection'),

            EZCtlCfg('teeth_lblHSV', 'HSV', EZLABEL, (None), None, None, '', 'HSV'),
            EZCtlCfg('teeth_HSV_colour', 'Colour', EZCOLOURP, (None), (1.0, 1.0, 1.0), None, 'The colour of the teeth', 'HSV'),
            EZCtlCfg('teeth_HSV_H', 'Hue', EZFLOAT, (0.0, 10.0), 1.0, 0.1, 'Hue modifier', 'HSV'),
            EZCtlCfg('teeth_HSV_S', 'Saturation', EZFLOAT, (0.0, 2.0), 1.0, 0.1, 'Saturation modifier', 'HSV'),
            EZCtlCfg('teeth_HSV_V', 'Value', EZFLOAT, (0.0, 2.0), 1.0, 0.1, 'Intensity modifier', 'HSV'),

            EZCtlCfg('sf_lblteeth', 'Superfly teeth', EZLABEL, (None), None, None, '', 'Superfly'),
            EZCtlCfg('sf_teeth_colour', 'Colour', EZCOLOURP, (None), (0.980, 0.957, 0.890), None, 'The colour of the teeth', 'Superfly'),
            EZCtlCfg('sf_teeth_specularity', 'Specularity', EZFLOAT, (0.0, 1.0), 1.0, 0.1, 'See Poser manual', 'Superfly'),
            EZCtlCfg('sf_teeth_highlight', 'Highlight', EZFLOAT, (0.0, 1.0), 0.3, 0.1, 'See Poser manual', 'Superfly'),
            EZCtlCfg('sf_teeth_spec_colour', 'Specular colour', EZCOLOURP, (None), (1.0, 1.0, 1.0), None, 'The specular colour of the teeth', 'Superfly'),
            EZCtlCfg('sf_teeth_eccentricity', 'Specular eccentricity', EZFLOAT, (0.0, 1.0), 0.002, 0.001, 'See Poser manual', 'Superfly'),
            EZCtlCfg('sf_teeth_rolloff', 'Specular rolloff', EZFLOAT, (0.0, 1.0), 0.30, 0.05, 'See Poser manual', 'Superfly'),
            EZCtlCfg('sf_teeth_ior', 'Calculte RollOff from IOR', EZBUTTON, (None), self.UseIOR, None, 'Calculates SpecularRollOff from IOR value', 'Superfly'),
            EZCtlCfg('sf_teeth_reflectivity', 'Specular reflectivity', EZFLOAT, (0.0, 100.0), 1.0, 0.10, 'See Poser manual', 'Superfly'),
            )

        mouthdata = (

            EZCtlCfg('mouth_lblmain', 'Main settings', EZLABEL, (None), None, None, '', 'Main'),
            EZCtlCfg('mouth_colour', 'Colour', EZCOLOURP, (None), (0.840, 0.730, 0.790), None, 'The colour of the mouth', 'Main'),
            EZCtlCfg('mouth_sss_scale', 'SSS Scale', EZFLOAT, (0.1, 2.0), 0.9, 0.1, 'SSS strength', 'Main'),
            EZCtlCfg('mouth_sss_quality', 'SSS Quality', EZFLOAT, (0.0, 1.0), 0.8, 0.1, 'Higher values = better quality but longer render time', 'Main'),
            EZCtlCfg('mouth_sss_type', 'Type', EZISELECT, SSSTYPES, 5, None, 'SSS type', 'Main'),
            EZCtlCfg('mouth_fresnel_ior', 'IOR', EZFLOAT, (0.0, 2.0), 1.33, 0.1, 'IOR for the mouth', 'Main'),
            EZCtlCfg('mouth_gum_bump', 'Gums bump value', EZFLOAT, (0.0, 1.0), 0.005, 0.1, 'Strength of bump effect for the gums', 'Main'),
            EZCtlCfg('mouth_tongue_bump', 'Tongue bump value', EZFLOAT, (0.0, 1.0), 0.03, 0.1, 'Strength of bump effect for the mouth', 'Main'),

            EZCtlCfg('lblblips', 'Lips', EZLABEL, (None), None, None, '', 'Lips'),
            EZCtlCfg('lips_use_skin', 'Use skin shader', EZBOOL, (None), 0, None, 'use the skin shader instead for lips', 'Lips'),
            EZCtlCfg('lips_colour', 'Colour', EZCOLOURP, (None), (1.000, 1.000, 1.000), None, 'The colour of the lips', 'Lips'),
            EZCtlCfg('lips_V', 'Intensity', EZFLOAT, (0.0, 2.0), 1.0, 0.1, 'Intensity modifier', 'Lips'),
            EZCtlCfg('lips_specularity', 'Specularity', EZFLOAT, (0.0, 6.0), 1.1, 0.1, 'Strength of specular reflection', 'Lips'),
            EZCtlCfg('lips_spec_eccentricity', 'Eccentricity', EZFLOAT, (0.0, 1.0), 0.3, 0.1, 'See Poser manual', 'Lips'),
            EZCtlCfg('lips_spec_rolloff', 'RollOff', EZFLOAT, (0.0, 1.0), 0.4, 0.1, 'See Poser manual', 'Lips'),
            EZCtlCfg('lblclips', 'Lips (SF specific)', EZLABEL, (None), None, None, '', 'Lips'),
            EZCtlCfg('sf_lips_spec_eccentricity', 'Eccentricity', EZFLOAT, (0.0, 1.0), 0.3, 0.05, 'See Poser manual', 'Lips'),
            EZCtlCfg('sf_lips_spec_rolloff', 'RollOff', EZFLOAT, (0.0, 1.0), 0.2, 0.05, 'See Poser manual', 'Lips'),
            EZCtlCfg('sf_lips_spec_reflectivity', 'Reflectivity', EZFLOAT, (0.0, 100.0), 0.50, 0.10, 'See Poser manual', 'Lips'),
            EZCtlCfg('sf_lips_ior', 'Calculte RollOff from IOR', EZBUTTON, (None), self.UseIOR, None, 'Calculates SpecularRollOff from IOR value', 'Lips'),

            EZCtlCfg('mouth_lblspec', 'Specular', EZLABEL, (None), None, None, '', 'Specular'),
            EZCtlCfg('mouth_spec_roughness', 'Specular roughness', EZFLOAT, (0.0, 1.0), 0.1, 0.1, 'How blurred the specular effect is', 'Specular'),
            EZCtlCfg('mouth_spec_ks', 'Mouth Ks', EZFLOAT, (0.0, 1.0), 0.1, 0.1, 'Strength of specular effect in the mouth', 'Specular'),
            EZCtlCfg('mouth_spec_ks_gum', 'Gum Ks', EZFLOAT, (0.0, 1.0), 0.3, 0.1, 'Strength of specular effect in the gums', 'Specular'),
            EZCtlCfg('mouth_spec_ks_tongue', 'Tongue Ks', EZFLOAT, (0.0, 1.0), 0.6, 0.1, 'Strength of specular effect on the tongue', 'Specular'),

            EZCtlCfg('mouth_lblreflect', 'Reflection', EZLABEL, (None), None, None, '', 'Reflection'),
            EZCtlCfg('mouth_reflect_quality', 'Quality', EZFLOAT, (0.0, 1.0), 0.4, 0.1, 'Higher values = better quality but longer render time', 'Reflection'),
            EZCtlCfg('mouth_reflect_softness', 'Softness', EZFLOAT, (0.0, 5.0), 1.0, 0.1, 'How blurred the reflections are', 'Reflection'),
            EZCtlCfg('mouth_reflect_raybias', 'Ray bias', EZFLOAT, (0.0, 1.0), 0.3, 0.1, 'See Poser Manual', 'Reflection'),
            )

        eyedata = (

            EZCtlCfg('eye_lblmain', 'Main settings', EZLABEL, (None), None, None, '', 'Main'),
            EZCtlCfg('eye_blur', 'Blur', EZFLOAT, (0.0, 5.0), 0.1, 0.01, '', 'Main'),
            EZCtlCfg('eye_ior', 'IOR', EZFLOAT, (0.0, 4.0), 1.40, 0.1, 'IOR of the reflective surface of the eye', 'Main'),
            EZCtlCfg('eye_sss_quality', 'SSS Quality', EZFLOAT, (0.0, 1.0), 0.8, 0.1, 'Higher values = better quality but longer render time', 'Main'),
            EZCtlCfg('eye_reflect_quality', 'Reflection quality', EZFLOAT, (0.0, 1.0), 0.4, 0.1, 'Higher values = better quality but longer render time', 'Main'),
            EZCtlCfg('eye_bump_opt', 'Bump type', EZISELECT, ('Bump map or nothing', 'Prefer bump map', 'Procedural bump'), 2, None, '', 'Main'),

            EZCtlCfg('eye_lbliris', 'Iris', EZLABEL, (None), None, None, '', 'Iris'),
            EZCtlCfg('iris_HSV_colour', 'Colour', EZCOLOURP, (None), (1.0, 1.0, 1.0), None, 'Colour modifier', 'Iris'),
            EZCtlCfg('iris_HSV_H', 'Hue', EZFLOAT, (0.0, 10.0), 1.0, 0.1, 'Hue modifier', 'Iris'),
            EZCtlCfg('iris_HSV_S', 'Saturation', EZFLOAT, (0.0, 2.0), 1.0, 0.1, 'Saturation modifier', 'Iris'),
            EZCtlCfg('iris_HSV_V', 'Value', EZFLOAT, (0.0, 2.0), 1.2, 0.1, 'Intensity modifier', 'Iris'),

            EZCtlCfg('eye_lblsclera', 'Sclera', EZLABEL, (None), None, None, '', 'Sclera'),
            EZCtlCfg('sclera_HSV_colour', 'Colour', EZCOLOURP, (None), (1.0, 1.0, 1.0), None, '', 'Sclera'),
            EZCtlCfg('sclera_HSV_H', 'Hue', EZFLOAT, (0.0, 10.0), 1.0, 0.1, 'Hue modifier', 'Sclera'),
            EZCtlCfg('sclera_HSV_S', 'Saturation', EZFLOAT, (0.0, 2.0), 1.0, 0.1, 'Saturation modifier', 'Sclera'),
            EZCtlCfg('sclera_HSV_V', 'Value', EZFLOAT, (0.0, 2.0), 1.2, 0.1, 'Intensity modifier', 'Sclera'),

            EZCtlCfg('sf_sclera_lblmain', 'Superfly sclera', EZLABEL, (None), None, None, '', 'Superfly'),
            EZCtlCfg('sf_sclera_sss_scale', 'SSS Scale', EZFLOAT, (0.1, 2.0), 0.6, 0.1, 'SSS strength', 'Superfly'),
            EZCtlCfg('sf_sclera_sss_quality', 'SSS Quality', EZFLOAT, (0.0, 1.0), 0.8, 0.1, 'Higher values = better quality but longer render time', 'Superfly'),
            EZCtlCfg('sf_sclera_sss_type', 'Type', EZISELECT, SSSTYPES, 7, None, 'SSS type', 'Superfly'),
            EZCtlCfg('sf_sclera_specularity', 'Specularity', EZFLOAT, (0.1, 2.0), 1.0, 0.1, 'Specular strength', 'Superfly'),
            EZCtlCfg('sf_sclera_roughness', 'Roughness', EZFLOAT, (0.0, 1.0), 0.05, 0.01, 'Specular roughness', 'Superfly'),
            )

        naildata = (

            EZCtlCfg('nail_lblmain', 'Main', EZLABEL, (None), None, None, '', 'Main'),
            EZCtlCfg('nail_usecolour', 'Overide texture with colour', EZBOOL, (None), 0, None, 'Use nail colour rather instead of existing texture map', 'Main'),
            EZCtlCfg('nail_colour', 'Colour', EZCOLOURP, (None), (1.0, 1.0, 1.0), None, 'The nail colour', 'Main'),

            EZCtlCfg('nail_lblspec', 'Specular', EZLABEL, (None), None, None, '', 'Specular'),
            EZCtlCfg('nail_spec_strength', 'Specular strength', EZFLOAT, (0.0, 1.0), 0.6, 0.1, 'The brightness of the specular effect', 'Specular'),
            EZCtlCfg('nail_spec_size', 'Highlight size', EZFLOAT, (0.0, 1.0), 0.1, 0.1, 'The size of the specular effect', 'Specular'),

            EZCtlCfg('nail_lblreflect', 'Reflection', EZLABEL, (None), None, None, '', 'Reflection'),
            EZCtlCfg('nail_reflect_quality', 'Reflect quality', EZFLOAT, (0.0, 1.0), 0.4, 0.1, 'Higher values = better quality but longer render time', 'Reflection'),
            EZCtlCfg('nail_reflect_softness', 'Reflect softness', EZFLOAT, (0.0, 5.0), 0.4, 0.1, 'How blurred the reflections are', 'Reflection'),
            EZCtlCfg('nail_ior', 'Reflect strength', EZFLOAT, (0.0, 10.0), 5.0, 0.1, 'How reflective the surface is', 'Reflection'),

            EZCtlCfg('nail_lblSSS', 'SSS', EZLABEL, (None), None, None, '', 'SSS'),
            EZCtlCfg('nail_sss_type', 'Type', EZISELECT, SSSTYPES, 8, None, 'SSS Type', 'SSS'),
            EZCtlCfg('nail_sss_scale', 'SSS Scale', EZFLOAT, (0.1, 2.0), 1.5, 0.1, 'SSS strength', 'SSS'),
            EZCtlCfg('nail_sss_quality', 'SSS Quality', EZFLOAT, (0.0, 1.0), 0.8, 0.1, 'Higher values = better quality but longer render time', 'SSS'),

            EZCtlCfg('nail_lblscatter', 'BB Max Scatter Trick', EZLABEL, (None), None, None, '', 'BB Max Scatter'),
            EZCtlCfg('nail_maxscatter', 'Use BB Max Scatter trick', EZBOOL, (None), 0, None, 'Reduces the possibility of skin turning blue when SSS is applied', 'BB Max Scatter'),
            EZCtlCfg('nail_bbSSSscale', 'SSS Scale', EZFLOAT, (0.0, 5.0), 2.0, 0.1, 'See user guide', 'BB Max Scatter'),
            EZCtlCfg('nail_bbHSVscale', 'HSV scale', EZFLOAT, (0.0, 2.0), 0.9, 0.1, 'See user guide', 'BB Max Scatter'),

            )

        otherdata = (
            EZCtlCfg('hair_lblmain', 'Hair', EZLABEL, (None), None, None, '', 'Hair'),
            EZCtlCfg('hair_keepKd', 'Keep hair diffuse colour', EZBOOL, (None), 1, None, 'If true then hair colour is modified by the PoserSurface Diffuse color chip', 'Hair'),
            EZCtlCfg('hair_colour', 'Hair colour', EZCOLOURP, (None), (1.0, 1.0, 1.0), None, 'The colour of the hair', 'Hair'),
            EZCtlCfg('hair_specular', 'Hair specular value', EZFLOAT, (0.0, 5.0), 0.6, 0.1, 'Hair specular intensity', 'Hair'),
            EZCtlCfg('hair_useSSS', 'Use SSS on hair', EZBOOL, (None), 0, None, 'Apply SSS to hair', 'SSS'),
            EZCtlCfg('hair_sss_scale', 'Hair SSS Scale', EZFLOAT, (0.1, 2.0), 1.0, 0.1, 'SSS strength', 'SSS'),
            EZCtlCfg('hair_sss_quality', 'Hair SSS Quality', EZFLOAT, (0.0, 1.0), 0.8, 0.1, 'Higher values = better quality but longer render time', 'SSS'),
            EZCtlCfg('hair_lblsf', 'Superfly only', EZLABEL, (None), None, None, '', 'Superfly'),
            EZCtlCfg('hair_uvdir', 'U direction', EZISELECT, ('Horizontal', 'Vertical'), 0, None, 'U direction of the UV map', 'Superfly'),
            EZCtlCfg('hair_useReflection', 'Add reflection', EZBOOL, (None), 0, None, 'Apply some reflection to the hair', 'Superfly'),
            EZCtlCfg('hair_V', 'Colour intensity', EZFLOAT, (0.0, 5.0), 1.0, 0.5, 'Changes the intensity of the hair colour', 'Superfly'),
            EZCtlCfg('hair_translucence', 'Translucence', EZFLOAT, (0.0, 5.0), 1.0, 0.1, 'Determines how translucent the hair is', 'Superfly'),
            EZCtlCfg('hair_bw', 'Monochrome', EZBOOL, (None), 0, None, 'Make diffuse colour monochrome', 'Superfly'),
            )

        # Add them to the dataset
        self.data.AddControlSet('options', process)
        self.data.AddControlSet('settings', cfgdata)
        self.data.AddControlSet('skin', skindata)
        self.data.AddControlSet('teeth', teethdata)
        self.data.AddControlSet('mouth', mouthdata)
        self.data.AddControlSet('eyes', eyedata)
        self.data.AddControlSet('nails', naildata)
        self.data.AddControlSet('other', otherdata)

    def IsValidFile(self, file):
        if file != None:
            if os.path.exists(file):
                return True
        return False

    def IsValidPath(self, folder):
        if folder != None:
            if os.path.exists(folder):
                if os.path.isdir(folder):
                    return True
        return False

    def LoadMat(self, event):
        if self.actor and self.mc6path:
            try:
                self.actor.LoadMaterialCollection(self.mc6path)
                self.scene.DrawAll()
                self.StatusBar('Material collection applied')
            except:
                self.StatusBar('Unable to apply material collection %s' % self.mc6path)
        else:
            self.StatusBar('No material collection available')

    def LoadOptions(self):
        cget = self.data.ControlValue

        self.chk_ProcessSkin.SetValue(cget('chk_ProcessSkin'))
        self.chk_ProcessEyes.SetValue(cget('chk_ProcessEyes'))
        self.chk_ProcessNails.SetValue(cget('chk_ProcessNails'))
        self.chk_ProcessMouth.SetValue(cget('chk_ProcessMouth'))
        self.chk_ProcessTeeth.SetValue(cget('chk_ProcessTeeth'))
        self.chk_ProcessOther.SetValue(cget('chk_ProcessOther'))

    def MatOwner(self, ext=False):
        actor = self.GetActor()
        if actor:
            if ext:
                return actor.Name()
            else:
                return actor.InternalName()
        else:
            return None

    def Menu(self, event):
        self.PopupMenu( self.mnu_Menu, (20,20) )

    def NailSalon(self, event):
        # Not implemented yet
        self.Nix(event)

    def Nix(self, event):
        ShowAlert('This feature is not yet available')

    def Overlay(self, event):

        # Obtain the overlay object
        dlg = EZOverlay(self, self.GetActor(), self.data.ControlValue('renderer'))

        # Check for valid actor
        actor = self.GetActor()
        if actor:

            # Show the dialog
            result = dlg.ShowModal()

            if result == wx.ID_OK:
                result = dlg.MakeOverlay()
                self.StatusBar(result)

        # Clean up
        dlg.Destroy()

    def PoserRenderer(self):
        return (poser.kRenderEngineCodeFIREFLY, poser.kRenderEngineCodeSUPERFLY)[self.builder.Renderer() == 'Superfly']

    def Presets_Delete(self, event):

        # Create the dialog and show it
        dlg = Selector(self.presets.Sections(), 'Delete presets', 'Delete selected')
        retcode = dlg.ShowModal()

        # Process the user's selection
        if retcode == 1:

            # For convenience
            cfg = self.presets.ConfigFile()
            ctl = self.ctl_Presets

            # Delete each selected preset
            for preset in dlg.GetSelected():
                cfg.remove_section(preset)
                id = ctl.FindString(preset)
                if not id is None: ctl.Delete(id)

            self.StatusBar('Selected presets deleted')

        # Delete the dialog
        dlg.Destroy()

    def Presets_Export(self, event):
        # Exports a user selected collection of presets to a file

        # Create a dialog box to select presets for export
        dlg = Selector(self.presets.Sections(), 'Export presets', buttontext='Export')
        retcode = dlg.ShowModal()

        # Process the selection
        if retcode == 1:                        # User pressed the 'Export' button

            presets = dlg.GetSelected()
            if len(presets) > 0:                # Make sure something was selected

                # Get a config parser ...
                export = configparser.RawConfigParser()
                export.optionxform = str

                # ... and copy selected presets into it
                for p in presets:
                    export.add_section(p)
                    for opt, val in self.presets.ConfigFile().items(p):
                        export.set(p, opt, val)

                # Ask the user for a save location
                fdlg = wx.FileDialog(None,
                    'Choose a file',
                    defaultDir=poser.ContentRootLocation(),
                    defaultFile='ezskin3_export.presets',
                    wildcard='*.presets',
                    style=wx.FD_SAVE,
                    pos=wx.DefaultPosition)

                # Save the file
                if fdlg.ShowModal() == wx.ID_OK:
                    path = fdlg.GetPath()
                    try:
                        cfgfile = open(path, 'w')
                        export.write(cfgfile)
                        cfgfile.close()
                    except:
                        ShowAlert('Error saving presets file', 'Preset exporter')
                        fdlg.Destroy()
                        return

                    self.StatusBar('Presets successfully exported')

                # Clean up
                fdlg.Destroy()

    def Presets_Import(self, event):

        # Ask the user for a file location
        fdlg = wx.FileDialog(None,
            'Choose a file',
            defaultDir=poser.ContentRootLocation(),
            defaultFile='',
            wildcard='*.presets',
            style=wx.FD_OPEN,
            pos=wx.DefaultPosition)

        if fdlg.ShowModal() == wx.ID_OK:            # User chose a file

            path = fdlg.GetPath()                    # get the path to the file

            # Get a config parser
            imp = configparser.RawConfigParser()
            imp.optionxform = str

            # And read the config data from the user's chosen file
            imp.read(path)

            presets = imp.sections()

            # Create a dialog box to select presets for import
            dlg = Selector(presets, 'Import presets', buttontext='Import')

            if dlg.ShowModal() == True:

                # Initialise
                presets = dlg.GetSelected()
                cfg = self.presets.ConfigFile()

                # Get each preset in turn
                for preset in presets:

                    # We need to make sure preset names are unique
                    id = 0
                    newPreset = preset

                    # Rename if necessary ...
                    while cfg.has_section(newPreset):
                        newPreset = '%s_%s' % (preset, id)
                        id += 1

                    # Add the imported preset to our presets
                    cfg.add_section(newPreset)
                    for opt, val in imp.items(preset):
                        cfg.set(newPreset, opt, val)

                    # Update the script's menu
                    self.AddPresetToMenu(newPreset)

                # Advise the user of completion of import
                if len(presets) > 0: self.StatusBar('Presets import successful')

            else:
                self.StatusBar('Presets import cancelled by user')

            # Clean up
            dlg.Destroy()

    def SaveMat( self, event, path="", temp=False):

        # Stop the timer
        self.timer.Stop()

        target = path
        actor = self.actor
        if actor:

            # Make a note of the top level actor
            fig = self.GetActor()

            # Get list of selected materials as strings
            mats = [x.Name() for x in actor.Materials()]

            if temp == True:
                selected_mats = mats
                retcode = True
            else:
                dlg = Selector(mats, 'Select the materials to be saved', buttontext='Select')
                dlg.SelectAll(None)
                retcode = dlg.ShowModal()
                selected_mats = dlg.GetSelected()
                dlg.Destroy()

            # Process selected materials
            if retcode == True and len(selected_mats) > 0:

                # Determine path
                if temp:
                    target = self.undo
                else:
                    if target == "":
                        # Ask the user for a file location
                        fdlg = wx.FileDialog(None,
                            'Save material collection',
                            defaultDir=poser.ContentRootLocation(),
                            defaultFile='%s.mc6' % fig.Name(),
                            wildcard='*.mc6',
                            style=wx.FD_SAVE,
                            pos=wx.DefaultPosition)

                    if fdlg.ShowModal() == wx.ID_OK:            # User chose a file
                        target = fdlg.GetPath()                    # get the path to the file
                    else:
                        return

                # Set selection status of each material
                for mat in actor.Materials():
                    mat.SetSelected(mat.Name() in selected_mats)

                # Save the material set
                actor.SaveMaterialCollection(target)
                self.StatusBar('Material collection saved')

                # Make a note of which actor the material set refers to
                self.mc6 = fig.InternalName()

                # And the saved location
                if not temp:
                    self.mc6path = target
                    self.ctl_LoadMat.SetToolTip(wx.ToolTip(fig.Name()))

            else:
                self.StatusBar('No materials selected')

        else:
            self.StatusBar('No actor selected')

        # And start the timer :-)
        self.timer.Start()

    def SavePreset(self, event):
        # Save a new preset

        # First make sure Ui options are saved
        self.SaveOptions()

        # Select the categories of controls to be saved
        sets = sorted(self.data.ControlSets())

        # Show the dialog
        dlg = Selector(sets, 'Select the controls to be saved', buttontext='Select')
        retcode = dlg.ShowModal()
        selected_sets = dlg.GetSelected()
        dlg.Destroy()

        # Save it
        if retcode == True and len(selected_sets) > 0:

            # Obtain a preset name from the user
            newsection = GetText(msg='Enter a preset name', default=self.ctl_Presets.GetStringSelection())

            if newsection:
                self.data.WriteToConfigCopy(self.presets, newsection, selected_sets)
                ctl = self.ctl_Presets
                ctl.Clear()
                ctl.AppendItems(self.presets.Sections())
                ctl.SetSelection(ctl.FindString(newsection))
                self.StatusBar('Preset %s saved' % newsection)

        else:
                self.StatusBar('No controls were selected')

    def SaveOptions(self):
        cset = self.data.SetControlValue

        cset('chk_ProcessSkin', self.chk_ProcessSkin.IsChecked())
        cset('chk_ProcessEyes', self.chk_ProcessEyes.IsChecked())
        cset('chk_ProcessNails', self.chk_ProcessNails.IsChecked())
        cset('chk_ProcessMouth', self.chk_ProcessMouth.IsChecked())
        cset('chk_ProcessTeeth', self.chk_ProcessTeeth.IsChecked())
        cset('chk_ProcessOther', self.chk_ProcessOther.IsChecked())

    def SyncEngines(self, usecfg=True):

        if usecfg == True:
            self.ctl_Engine.Select(self.data.ControlValue('renderer', 'settings'))
        else:
            self.data.SetControlValue('renderer', self.ctl_Engine.GetSelection(), 'settings')

        # Get the Shadermaker object used to build the shaders
        self.builder = self.GetBuilder()

        # Check for engine's custom controls
        if hasattr(self.builder, 'has_controls'):
            self.ctl_Configure.Enable(self.builder.has_controls)
        else:
            self.ctl_Configure.Disable()

    def StatusBar(self, msg):
        self.ctl_StatusBar.SetValue(msg)
        if VERBOSE: print(msg)

    def Undo(self, event):
        if self.MatOwner() == self.mc6:
            try:
                self.actor.LoadMaterialCollection(self.undo)
                self.StatusBar('Undo completed')
            except:
                self.StatusBar('Undo failed')
        else:
            self.StatusBar('Undo file does not match selected actor')

    def Update(self, event):

        # Determine which rooms should be used
        if self.multiroom:
            self.follow.Reset()
        else:
            self.follow.SetValidOriginalRoomOnly()

        # Hide / Show as appropriate
        self.follow.Update(event)

        try:

            # Get the current selections
            self.figure = self.scene.CurrentFigure()
            self.actor = self.scene.CurrentActor()
            if not hasattr(self.actor, 'Materials') or self.actor.Materials() is None: self.actor = None

            # Update the UI
            if self.actor:
                self.ctl_CurrentActor.SetValue(self.actor.Name())
            else:
                self.ctl_CurrentActor.SetValue("No valid actor selected")

            if self.figure:
                if self.actor:
                    if self.actor.IsBodyPart():
                        self.ctl_CurrentFigure.SetValue(self.figure.Name())
                    else:
                        self.figure = None
                        self.ctl_CurrentFigure.SetValue("No valid figure selected")
                else:
                    self.ctl_CurrentFigure.SetValue(self.figure.Name())
            else:
                self.ctl_CurrentFigure.SetValue("No valid figure selected")

            # Get the current actor's obj name
            obj = self.GetActorObjName()

            # Check to see if it has changed
            if obj == None or not obj == self.old_obj:
                self.old_obj = obj

                # Check for custom settings
                if self.data.GetCustomShaders(self.GetActor()):
                    self.ctl_FigureType.SetValue('Custom')
                else:
                    if obj == None:
                        self.ctl_FigureType.SetValue('(Unsupported)')
                    else:
                        self.ctl_FigureType.SetValue(self.lib.GetTypeStr(obj))

        except:
            pass

    def UseIOR(self, event):
        ior = GetText('Enter an IOR value', title='RollOff calculator', default='1.00')
        if ior is not None:
            try:
                sro = round(self.api.SROfromIOR(float(ior)), 3)
                ShowAlert('IOR of %s = SpecularRollOff of %s' % (ior, sro), 'SpecularRollOff calculator')
            except:
                ShowAlert('Invalid IOR value provided', 'SpecularRollOff calculator')

    def ValidatePlugin(self, module):

        # Get the plugin name
        try:
            plugin = module.EZBuilder(self.api)
            name = plugin.BuilderName()
        except:
            return None, None

        # Check for custom controls
        if hasattr(plugin, 'has_controls'):
            if plugin.has_controls:
                self.GetCustomEngineControls(plugin)

        return plugin, name

    def Web(self, event):
        url = "http://snarlygribbly.org/poser"
        webbrowser.open(url, new=2)

# EZDATA
class EZData(object):
    def __init__(self, cfg):
        self.custom = dict()        # Dictionary of custom shaders
        self.data = dict()            # Configuration data for custom controls
        self.cfg = cfg                 # Configuration file manager
        self.lookup = dict()        # Crossreferences control names with control sets
        self.scattergroup = 0        # Last used scatter group

    def AddControlSet(self, name, data):
        # Defines the controls for a specific dialog box 'name'

        self.data[name] = OrderedDict()

        for ctl in data:
            self.data[name][ctl.Name()] = ctl
            self.lookup[ctl.Name()] = name

    def ControlCfg(self, name, controlset=None):
        # Returns the EZCtlCfg object associated with the control
        ctls = controlset or self.lookup[name]
        return self.data[ctls][name]

    def ControlSets(self):
        return list(self.data.keys())

    def ControlValue(self, name, controlset=None):
        # Returns the value of a control in a control set
        ctls = controlset or self.lookup[name]
        return self.data[ctls][name].Value()

    def ControlValue2(self, name, controlset=None):
        # Returns the value of a control in a control set
        if controlset == '@':
            if name == 'group':
                self.scattergroup += 1
                return self.scattergroup
            else:
                raise Exception('Invalid parameter requested')
        else:
            ctls = controlset or self.lookup[name]
            return self.data[ctls][name].Value()

    def Display(self, name, output=None):

        # Create the dialog object
        dlg = EZDialog('Configure %s' % name, self.data[name], output, pos=self.cfg.GetD('dlg_pos'), size=self.cfg.GetD('dlg_siz'))

        # Display it :-)
        if dlg.ShowModal():

            # Retrieve all the values
            for k, v in list(dlg.Values().items()):
                self.data[name][k].SetValue(v)

        # Tidy up
        self.cfg.SetD('dlg_pos', dlg.Position())
        self.cfg.SetD('dlg_siz', dlg.Size())
        dlg.Destroy()

    def GetCustomShaders(self, target):
        return self.custom.get(target.InternalName(), None)

    def Range(self, name, controlset=None):
        # Returns the minimum and maximum values of a control
        ctls = controlset or self.lookup[name]
        return self.data[ctls][name].MinMax()

    def ReadFromConfig(self):
        for section in self.cfg.Sections():
            if section in list(self.data.keys()):
                for option in self.cfg.Options(section):
                    value = self.cfg.Get(section, option)
                    if not value is None: self.SetControlValue(option, value, section)

    def SetControlValue(self, name, value, controlset=None):
        # Sets the value of a control in a control set
        ctls = controlset or self.lookup.get(name, None)
        if not ctls is None:
            if name in list(self.data[ctls].keys()):
                self.data[ctls][name].SetValue(value)

    def UseCustomShaders(self, target, shaders):
        if target:
            self.custom[target.InternalName()] = shaders

    def WriteToConfig(self):
        # Writes all controlsets to the configuration file
        for section, controlset in list(self.data.items()):
            for item, ctl in list(controlset.items()):
                self.cfg.Set(section, item, ctl.Value())

    def WriteToConfigCopy(self, cfg, name, selected_sets):
        # Writes all controlsets to the specified configuration file
        for section, controlset in list(self.data.items()):
            if section in selected_sets:
                for item, ctl in list(controlset.items()):
                    cfg.Set(name, item, ctl.Value())

# MAIN
class Main(object):
    def __init__(self):

        # Initialise ...

        # Configuration file
        self.defaultcfg = self.GetDefaults()
        self.cfgfile = self.GetCfgFile('ezkin3.cfg')
        self.presetsfile = self.GetCfgFile('ezkin3.presets')
        self.cfg = ConfigManager(self.cfgfile, None, 'ezskin3cfg')
        self.presets = ConfigManager(self.presetsfile, None, None)
        self.cfg.SetDMany(self.defaultcfg, first_time_only=True)

        # User interface
        self.manager = poser.WxAuiManager()
        self.root = self.manager.GetManagedWindow()
        self.pane = self.InitialisePane(wx.aui.AuiPaneInfo())

        # Ensure only one instance is running
        if self.AlreadyRunning():
            ShowAlert("%s is already running" % APPNAME, "Oy!")
            return

        # Launch the UI
        self.ezskin = EZSkin(self.root, self)
        self.manager.AddPane(self.ezskin, self.pane)
        self.manager.GetPane(self.ezskin).Show()
        self.manager.Update()

    def AlreadyRunning(self):
        # Returns True if script is already running, else False

        for pane in self.manager.GetAllPanes():
            if pane.caption == APPCAPTION:
                return True
        return False

    def GetCfgFile(self, filename):
        folder = os.path.join(poser.PrefsLocation(), 'ezskin3')
        if not os.path.isdir(folder):
            os.makedirs(folder)
        return os.path.join(folder, filename)

    def GetDefaults(self):
        return {
            'ui_pos' : (0, 0),
            'ui_size' : APPSIZE,
            'dlg_pos' : wx.DefaultPosition,
            'dlg_siz' : ( 425, 400 ),
            }

    def InitialisePane(self, pane):
        # Provides the default configuration for our AuiPaneInfo

        pane.Caption(APPCAPTION)
        pane.CaptionVisible(False)
        pane.Dockable(False)
        pane.Name(APPNAME)
        pane.FloatingSize(self.cfg.GetD('ui_size'))
        pane.MinSize(APPSIZE)
        pane.BestSize(APPSIZE)
        pane.DestroyOnClose(True)
        pane.Gripper(False)
        pane.CloseButton(False)
        pane.Float()
        pane.Hide()
        pane.FloatingPosition(self.cfg.GetD('ui_pos'))
        pane.Layer(0)
        pane.Position(0)
        pane.Row(1)
        pane.Direction(4)

        return pane

    def UpdateConfiguration(self, pane):
        # Remember the size and position of the EZSkin UI
        self.cfg.SetD('ui_size', pane.floating_size)
        self.cfg.SetD('ui_pos', pane.floating_pos)
        self.cfg.WriteFile()
        self.presets.WriteFile()

if __name__ == "__main__":
    app = Main()
